/****************************************************************************
** Meta object code from reading C++ file 'layoutproperties.h'
**
** Created by: The Qt Meta Object Compiler version 67 (Qt 5.15.19)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include <memory>
#include "../../../../../KDStateMachineEditor-1.2.8/src/core/layout/layoutproperties.h"
#include <QtCore/qbytearray.h>
#include <QtCore/qmetatype.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'layoutproperties.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 67
#error "This file was generated using the moc from 5.15.19. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

QT_BEGIN_MOC_NAMESPACE
QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
struct qt_meta_stringdata_KDSME__LayoutProperties_t {
    QByteArrayData data[13];
    char stringdata0[258];
};
#define QT_MOC_LITERAL(idx, ofs, len) \
    Q_STATIC_BYTE_ARRAY_DATA_HEADER_INITIALIZER_WITH_OFFSET(len, \
    qptrdiff(offsetof(qt_meta_stringdata_KDSME__LayoutProperties_t, stringdata0) + ofs \
        - idx * sizeof(QByteArrayData)) \
    )
static const qt_meta_stringdata_KDSME__LayoutProperties_t qt_meta_stringdata_KDSME__LayoutProperties = {
    {
QT_MOC_LITERAL(0, 0, 23), // "KDSME::LayoutProperties"
QT_MOC_LITERAL(1, 24, 20), // "regionMarginsChanged"
QT_MOC_LITERAL(2, 45, 0), // ""
QT_MOC_LITERAL(3, 46, 22), // "regionLabelFontChanged"
QT_MOC_LITERAL(4, 69, 25), // "regionLabelMarginsChanged"
QT_MOC_LITERAL(5, 95, 31), // "regionLabelButtonBoxSizeChanged"
QT_MOC_LITERAL(6, 127, 27), // "showTransitionLabelsChanged"
QT_MOC_LITERAL(7, 155, 7), // "changed"
QT_MOC_LITERAL(8, 163, 13), // "regionMargins"
QT_MOC_LITERAL(9, 177, 15), // "regionLabelFont"
QT_MOC_LITERAL(10, 193, 18), // "regionLabelMargins"
QT_MOC_LITERAL(11, 212, 24), // "regionLabelButtonBoxSize"
QT_MOC_LITERAL(12, 237, 20) // "showTransitionLabels"

    },
    "KDSME::LayoutProperties\0regionMarginsChanged\0"
    "\0regionLabelFontChanged\0"
    "regionLabelMarginsChanged\0"
    "regionLabelButtonBoxSizeChanged\0"
    "showTransitionLabelsChanged\0changed\0"
    "regionMargins\0regionLabelFont\0"
    "regionLabelMargins\0regionLabelButtonBoxSize\0"
    "showTransitionLabels"
};
#undef QT_MOC_LITERAL

static const uint qt_meta_data_KDSME__LayoutProperties[] = {

 // content:
       8,       // revision
       0,       // classname
       0,    0, // classinfo
       6,   14, // methods
       5,   60, // properties
       0,    0, // enums/sets
       0,    0, // constructors
       0,       // flags
       6,       // signalCount

 // signals: name, argc, parameters, tag, flags
       1,    1,   44,    2, 0x06 /* Public */,
       3,    1,   47,    2, 0x06 /* Public */,
       4,    1,   50,    2, 0x06 /* Public */,
       5,    1,   53,    2, 0x06 /* Public */,
       6,    1,   56,    2, 0x06 /* Public */,
       7,    0,   59,    2, 0x06 /* Public */,

 // signals: parameters
    QMetaType::Void, QMetaType::QReal,    2,
    QMetaType::Void, QMetaType::QFont,    2,
    QMetaType::Void, QMetaType::QReal,    2,
    QMetaType::Void, QMetaType::QSizeF,    2,
    QMetaType::Void, QMetaType::Bool,    2,
    QMetaType::Void,

 // properties: name, type, flags
       8, QMetaType::QReal, 0x00495903,
       9, QMetaType::QFont, 0x00495903,
      10, QMetaType::QReal, 0x00495903,
      11, QMetaType::QSizeF, 0x00495903,
      12, QMetaType::Bool, 0x00495903,

 // properties: notify_signal_id
       0,
       1,
       2,
       3,
       4,

       0        // eod
};

void KDSME::LayoutProperties::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    if (_c == QMetaObject::InvokeMetaMethod) {
        auto *_t = static_cast<LayoutProperties *>(_o);
        (void)_t;
        switch (_id) {
        case 0: _t->regionMarginsChanged((*reinterpret_cast< qreal(*)>(_a[1]))); break;
        case 1: _t->regionLabelFontChanged((*reinterpret_cast< const QFont(*)>(_a[1]))); break;
        case 2: _t->regionLabelMarginsChanged((*reinterpret_cast< qreal(*)>(_a[1]))); break;
        case 3: _t->regionLabelButtonBoxSizeChanged((*reinterpret_cast< const QSizeF(*)>(_a[1]))); break;
        case 4: _t->showTransitionLabelsChanged((*reinterpret_cast< bool(*)>(_a[1]))); break;
        case 5: _t->changed(); break;
        default: ;
        }
    } else if (_c == QMetaObject::IndexOfMethod) {
        int *result = reinterpret_cast<int *>(_a[0]);
        {
            using _t = void (LayoutProperties::*)(qreal );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&LayoutProperties::regionMarginsChanged)) {
                *result = 0;
                return;
            }
        }
        {
            using _t = void (LayoutProperties::*)(const QFont & );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&LayoutProperties::regionLabelFontChanged)) {
                *result = 1;
                return;
            }
        }
        {
            using _t = void (LayoutProperties::*)(qreal );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&LayoutProperties::regionLabelMarginsChanged)) {
                *result = 2;
                return;
            }
        }
        {
            using _t = void (LayoutProperties::*)(const QSizeF & );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&LayoutProperties::regionLabelButtonBoxSizeChanged)) {
                *result = 3;
                return;
            }
        }
        {
            using _t = void (LayoutProperties::*)(bool );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&LayoutProperties::showTransitionLabelsChanged)) {
                *result = 4;
                return;
            }
        }
        {
            using _t = void (LayoutProperties::*)();
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&LayoutProperties::changed)) {
                *result = 5;
                return;
            }
        }
    }
#ifndef QT_NO_PROPERTIES
    else if (_c == QMetaObject::ReadProperty) {
        auto *_t = static_cast<LayoutProperties *>(_o);
        (void)_t;
        void *_v = _a[0];
        switch (_id) {
        case 0: *reinterpret_cast< qreal*>(_v) = _t->regionMargins(); break;
        case 1: *reinterpret_cast< QFont*>(_v) = _t->regionLabelFont(); break;
        case 2: *reinterpret_cast< qreal*>(_v) = _t->regionLabelMargins(); break;
        case 3: *reinterpret_cast< QSizeF*>(_v) = _t->regionLabelButtonBoxSize(); break;
        case 4: *reinterpret_cast< bool*>(_v) = _t->showTransitionLabels(); break;
        default: break;
        }
    } else if (_c == QMetaObject::WriteProperty) {
        auto *_t = static_cast<LayoutProperties *>(_o);
        (void)_t;
        void *_v = _a[0];
        switch (_id) {
        case 0: _t->setRegionMargins(*reinterpret_cast< qreal*>(_v)); break;
        case 1: _t->setRegionLabelFont(*reinterpret_cast< QFont*>(_v)); break;
        case 2: _t->setRegionLabelMargins(*reinterpret_cast< qreal*>(_v)); break;
        case 3: _t->setRegionLabelButtonBoxSize(*reinterpret_cast< QSizeF*>(_v)); break;
        case 4: _t->setShowTransitionLabels(*reinterpret_cast< bool*>(_v)); break;
        default: break;
        }
    } else if (_c == QMetaObject::ResetProperty) {
    }
#endif // QT_NO_PROPERTIES
}

QT_INIT_METAOBJECT const QMetaObject KDSME::LayoutProperties::staticMetaObject = { {
    QMetaObject::SuperData::link<QObject::staticMetaObject>(),
    qt_meta_stringdata_KDSME__LayoutProperties.data,
    qt_meta_data_KDSME__LayoutProperties,
    qt_static_metacall,
    nullptr,
    nullptr
} };


const QMetaObject *KDSME::LayoutProperties::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *KDSME::LayoutProperties::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_meta_stringdata_KDSME__LayoutProperties.stringdata0))
        return static_cast<void*>(this);
    return QObject::qt_metacast(_clname);
}

int KDSME::LayoutProperties::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QObject::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 6)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 6;
    } else if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 6)
            *reinterpret_cast<int*>(_a[0]) = -1;
        _id -= 6;
    }
#ifndef QT_NO_PROPERTIES
    else if (_c == QMetaObject::ReadProperty || _c == QMetaObject::WriteProperty
            || _c == QMetaObject::ResetProperty || _c == QMetaObject::RegisterPropertyMetaType) {
        qt_static_metacall(this, _c, _id, _a);
        _id -= 5;
    } else if (_c == QMetaObject::QueryPropertyDesignable) {
        _id -= 5;
    } else if (_c == QMetaObject::QueryPropertyScriptable) {
        _id -= 5;
    } else if (_c == QMetaObject::QueryPropertyStored) {
        _id -= 5;
    } else if (_c == QMetaObject::QueryPropertyEditable) {
        _id -= 5;
    } else if (_c == QMetaObject::QueryPropertyUser) {
        _id -= 5;
    }
#endif // QT_NO_PROPERTIES
    return _id;
}

// SIGNAL 0
void KDSME::LayoutProperties::regionMarginsChanged(qreal _t1)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))) };
    QMetaObject::activate(this, &staticMetaObject, 0, _a);
}

// SIGNAL 1
void KDSME::LayoutProperties::regionLabelFontChanged(const QFont & _t1)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))) };
    QMetaObject::activate(this, &staticMetaObject, 1, _a);
}

// SIGNAL 2
void KDSME::LayoutProperties::regionLabelMarginsChanged(qreal _t1)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))) };
    QMetaObject::activate(this, &staticMetaObject, 2, _a);
}

// SIGNAL 3
void KDSME::LayoutProperties::regionLabelButtonBoxSizeChanged(const QSizeF & _t1)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))) };
    QMetaObject::activate(this, &staticMetaObject, 3, _a);
}

// SIGNAL 4
void KDSME::LayoutProperties::showTransitionLabelsChanged(bool _t1)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))) };
    QMetaObject::activate(this, &staticMetaObject, 4, _a);
}

// SIGNAL 5
void KDSME::LayoutProperties::changed()
{
    QMetaObject::activate(this, &staticMetaObject, 5, nullptr);
}
QT_WARNING_POP
QT_END_MOC_NAMESPACE
