/****************************************************************************
** Meta object code from reading C++ file 'quickrecursiveinstantiator_p.h'
**
** Created by: The Qt Meta Object Compiler version 67 (Qt 5.15.19)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include <memory>
#include "../../../../../KDStateMachineEditor-1.2.8/src/view/quick/quickrecursiveinstantiator_p.h"
#include <QtCore/qbytearray.h>
#include <QtCore/qmetatype.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'quickrecursiveinstantiator_p.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 67
#error "This file was generated using the moc from 5.15.19. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

QT_BEGIN_MOC_NAMESPACE
QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
struct qt_meta_stringdata_QuickRecursiveInstantiator_t {
    QByteArrayData data[17];
    char stringdata0[213];
};
#define QT_MOC_LITERAL(idx, ofs, len) \
    Q_STATIC_BYTE_ARRAY_DATA_HEADER_INITIALIZER_WITH_OFFSET(len, \
    qptrdiff(offsetof(qt_meta_stringdata_QuickRecursiveInstantiator_t, stringdata0) + ofs \
        - idx * sizeof(QByteArrayData)) \
    )
static const qt_meta_stringdata_QuickRecursiveInstantiator_t qt_meta_stringdata_QuickRecursiveInstantiator = {
    {
QT_MOC_LITERAL(0, 0, 26), // "QuickRecursiveInstantiator"
QT_MOC_LITERAL(1, 27, 15), // "DefaultProperty"
QT_MOC_LITERAL(2, 43, 8), // "delegate"
QT_MOC_LITERAL(3, 52, 12), // "modelChanged"
QT_MOC_LITERAL(4, 65, 0), // ""
QT_MOC_LITERAL(5, 66, 19), // "QAbstractItemModel*"
QT_MOC_LITERAL(6, 86, 5), // "model"
QT_MOC_LITERAL(7, 92, 15), // "delegateChanged"
QT_MOC_LITERAL(8, 108, 14), // "QQmlComponent*"
QT_MOC_LITERAL(9, 123, 10), // "modelReset"
QT_MOC_LITERAL(10, 134, 14), // "modelDestroyed"
QT_MOC_LITERAL(11, 149, 12), // "rowsInserted"
QT_MOC_LITERAL(12, 162, 11), // "QModelIndex"
QT_MOC_LITERAL(13, 174, 6), // "parent"
QT_MOC_LITERAL(14, 181, 5), // "first"
QT_MOC_LITERAL(15, 187, 4), // "last"
QT_MOC_LITERAL(16, 192, 20) // "rowsAboutToBeRemoved"

    },
    "QuickRecursiveInstantiator\0DefaultProperty\0"
    "delegate\0modelChanged\0\0QAbstractItemModel*\0"
    "model\0delegateChanged\0QQmlComponent*\0"
    "modelReset\0modelDestroyed\0rowsInserted\0"
    "QModelIndex\0parent\0first\0last\0"
    "rowsAboutToBeRemoved"
};
#undef QT_MOC_LITERAL

static const uint qt_meta_data_QuickRecursiveInstantiator[] = {

 // content:
       8,       // revision
       0,       // classname
       1,   14, // classinfo
       6,   16, // methods
       2,   68, // properties
       0,    0, // enums/sets
       0,    0, // constructors
       0,       // flags
       2,       // signalCount

 // classinfo: key, value
       1,    2,

 // signals: name, argc, parameters, tag, flags
       3,    1,   46,    4, 0x06 /* Public */,
       7,    1,   49,    4, 0x06 /* Public */,

 // slots: name, argc, parameters, tag, flags
       9,    0,   52,    4, 0x08 /* Private */,
      10,    0,   53,    4, 0x08 /* Private */,
      11,    3,   54,    4, 0x08 /* Private */,
      16,    3,   61,    4, 0x08 /* Private */,

 // signals: parameters
    QMetaType::Void, 0x80000000 | 5,    6,
    QMetaType::Void, 0x80000000 | 8,    2,

 // slots: parameters
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void, 0x80000000 | 12, QMetaType::Int, QMetaType::Int,   13,   14,   15,
    QMetaType::Void, 0x80000000 | 12, QMetaType::Int, QMetaType::Int,   13,   14,   15,

 // properties: name, type, flags
       6, 0x80000000 | 5, 0x0049510b,
       2, 0x80000000 | 8, 0x0049510b,

 // properties: notify_signal_id
       0,
       1,

       0        // eod
};

void QuickRecursiveInstantiator::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    if (_c == QMetaObject::InvokeMetaMethod) {
        auto *_t = static_cast<QuickRecursiveInstantiator *>(_o);
        (void)_t;
        switch (_id) {
        case 0: _t->modelChanged((*reinterpret_cast< QAbstractItemModel*(*)>(_a[1]))); break;
        case 1: _t->delegateChanged((*reinterpret_cast< QQmlComponent*(*)>(_a[1]))); break;
        case 2: _t->modelReset(); break;
        case 3: _t->modelDestroyed(); break;
        case 4: _t->rowsInserted((*reinterpret_cast< const QModelIndex(*)>(_a[1])),(*reinterpret_cast< int(*)>(_a[2])),(*reinterpret_cast< int(*)>(_a[3]))); break;
        case 5: _t->rowsAboutToBeRemoved((*reinterpret_cast< const QModelIndex(*)>(_a[1])),(*reinterpret_cast< int(*)>(_a[2])),(*reinterpret_cast< int(*)>(_a[3]))); break;
        default: ;
        }
    } else if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        switch (_id) {
        default: *reinterpret_cast<int*>(_a[0]) = -1; break;
        case 0:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<int*>(_a[0]) = -1; break;
            case 0:
                *reinterpret_cast<int*>(_a[0]) = qRegisterMetaType< QAbstractItemModel* >(); break;
            }
            break;
        case 1:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<int*>(_a[0]) = -1; break;
            case 0:
                *reinterpret_cast<int*>(_a[0]) = qRegisterMetaType< QQmlComponent* >(); break;
            }
            break;
        }
    } else if (_c == QMetaObject::IndexOfMethod) {
        int *result = reinterpret_cast<int *>(_a[0]);
        {
            using _t = void (QuickRecursiveInstantiator::*)(QAbstractItemModel * );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&QuickRecursiveInstantiator::modelChanged)) {
                *result = 0;
                return;
            }
        }
        {
            using _t = void (QuickRecursiveInstantiator::*)(QQmlComponent * );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&QuickRecursiveInstantiator::delegateChanged)) {
                *result = 1;
                return;
            }
        }
    } else if (_c == QMetaObject::RegisterPropertyMetaType) {
        switch (_id) {
        default: *reinterpret_cast<int*>(_a[0]) = -1; break;
        case 0:
            *reinterpret_cast<int*>(_a[0]) = qRegisterMetaType< QAbstractItemModel* >(); break;
        case 1:
            *reinterpret_cast<int*>(_a[0]) = qRegisterMetaType< QQmlComponent* >(); break;
        }
    }

#ifndef QT_NO_PROPERTIES
    else if (_c == QMetaObject::ReadProperty) {
        auto *_t = static_cast<QuickRecursiveInstantiator *>(_o);
        (void)_t;
        void *_v = _a[0];
        switch (_id) {
        case 0: *reinterpret_cast< QAbstractItemModel**>(_v) = _t->model(); break;
        case 1: *reinterpret_cast< QQmlComponent**>(_v) = _t->delegate(); break;
        default: break;
        }
    } else if (_c == QMetaObject::WriteProperty) {
        auto *_t = static_cast<QuickRecursiveInstantiator *>(_o);
        (void)_t;
        void *_v = _a[0];
        switch (_id) {
        case 0: _t->setModel(*reinterpret_cast< QAbstractItemModel**>(_v)); break;
        case 1: _t->setDelegate(*reinterpret_cast< QQmlComponent**>(_v)); break;
        default: break;
        }
    } else if (_c == QMetaObject::ResetProperty) {
    }
#endif // QT_NO_PROPERTIES
}

QT_INIT_METAOBJECT const QMetaObject QuickRecursiveInstantiator::staticMetaObject = { {
    QMetaObject::SuperData::link<QQuickItem::staticMetaObject>(),
    qt_meta_stringdata_QuickRecursiveInstantiator.data,
    qt_meta_data_QuickRecursiveInstantiator,
    qt_static_metacall,
    nullptr,
    nullptr
} };


const QMetaObject *QuickRecursiveInstantiator::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *QuickRecursiveInstantiator::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_meta_stringdata_QuickRecursiveInstantiator.stringdata0))
        return static_cast<void*>(this);
    if (!strcmp(_clname, "InstantiatorInterface"))
        return static_cast< InstantiatorInterface*>(this);
    return QQuickItem::qt_metacast(_clname);
}

int QuickRecursiveInstantiator::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QQuickItem::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 6)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 6;
    } else if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 6)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 6;
    }
#ifndef QT_NO_PROPERTIES
    else if (_c == QMetaObject::ReadProperty || _c == QMetaObject::WriteProperty
            || _c == QMetaObject::ResetProperty || _c == QMetaObject::RegisterPropertyMetaType) {
        qt_static_metacall(this, _c, _id, _a);
        _id -= 2;
    } else if (_c == QMetaObject::QueryPropertyDesignable) {
        _id -= 2;
    } else if (_c == QMetaObject::QueryPropertyScriptable) {
        _id -= 2;
    } else if (_c == QMetaObject::QueryPropertyStored) {
        _id -= 2;
    } else if (_c == QMetaObject::QueryPropertyEditable) {
        _id -= 2;
    } else if (_c == QMetaObject::QueryPropertyUser) {
        _id -= 2;
    }
#endif // QT_NO_PROPERTIES
    return _id;
}

// SIGNAL 0
void QuickRecursiveInstantiator::modelChanged(QAbstractItemModel * _t1)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))) };
    QMetaObject::activate(this, &staticMetaObject, 0, _a);
}

// SIGNAL 1
void QuickRecursiveInstantiator::delegateChanged(QQmlComponent * _t1)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))) };
    QMetaObject::activate(this, &staticMetaObject, 1, _a);
}
QT_WARNING_POP
QT_END_MOC_NAMESPACE
