/********************************************************************************
** Form generated from reading UI file 'transitionpropertyeditor.ui'
**
** Created by: Qt User Interface Compiler version 5.15.19
**
** WARNING! All changes made in this file will be lost when recompiling UI file!
********************************************************************************/

#ifndef UI_TRANSITIONPROPERTYEDITOR_H
#define UI_TRANSITIONPROPERTYEDITOR_H

#include <QtCore/QVariant>
#include <QtWidgets/QApplication>
#include <QtWidgets/QComboBox>
#include <QtWidgets/QFormLayout>
#include <QtWidgets/QHBoxLayout>
#include <QtWidgets/QLabel>
#include <QtWidgets/QLineEdit>
#include <QtWidgets/QSpacerItem>
#include <QtWidgets/QSpinBox>
#include <QtWidgets/QVBoxLayout>
#include <QtWidgets/QWidget>
#include "widgets/codeeditor_p.h"

QT_BEGIN_NAMESPACE

class Ui_TransitionPropertyEditor
{
public:
    QVBoxLayout *verticalLayout_2;
    QFormLayout *formLayout;
    QLabel *label;
    QLineEdit *labelLineEdit;
    QLabel *label_2;
    QComboBox *sourceStateComboBox;
    QLabel *label_3;
    QComboBox *targetStateComboBox;
    QLabel *signalLabel;
    QLineEdit *signalEdit;
    QLabel *timeoutLabel;
    QSpinBox *timeoutEdit;
    QVBoxLayout *verticalLayout;
    QHBoxLayout *horizontalLayout;
    QLabel *label_4;
    QSpacerItem *horizontalSpacer;
    CodeEditor *guardEditor;

    void setupUi(QWidget *TransitionPropertyEditor)
    {
        if (TransitionPropertyEditor->objectName().isEmpty())
            TransitionPropertyEditor->setObjectName(QString::fromUtf8("TransitionPropertyEditor"));
        TransitionPropertyEditor->resize(400, 300);
        verticalLayout_2 = new QVBoxLayout(TransitionPropertyEditor);
        verticalLayout_2->setObjectName(QString::fromUtf8("verticalLayout_2"));
        formLayout = new QFormLayout();
        formLayout->setObjectName(QString::fromUtf8("formLayout"));
        label = new QLabel(TransitionPropertyEditor);
        label->setObjectName(QString::fromUtf8("label"));
        label->setAlignment(Qt::AlignRight|Qt::AlignTrailing|Qt::AlignVCenter);

        formLayout->setWidget(0, QFormLayout::LabelRole, label);

        labelLineEdit = new QLineEdit(TransitionPropertyEditor);
        labelLineEdit->setObjectName(QString::fromUtf8("labelLineEdit"));

        formLayout->setWidget(0, QFormLayout::FieldRole, labelLineEdit);

        label_2 = new QLabel(TransitionPropertyEditor);
        label_2->setObjectName(QString::fromUtf8("label_2"));
        label_2->setAlignment(Qt::AlignRight|Qt::AlignTrailing|Qt::AlignVCenter);

        formLayout->setWidget(1, QFormLayout::LabelRole, label_2);

        sourceStateComboBox = new QComboBox(TransitionPropertyEditor);
        sourceStateComboBox->setObjectName(QString::fromUtf8("sourceStateComboBox"));
        sourceStateComboBox->setEditable(true);

        formLayout->setWidget(1, QFormLayout::FieldRole, sourceStateComboBox);

        label_3 = new QLabel(TransitionPropertyEditor);
        label_3->setObjectName(QString::fromUtf8("label_3"));
        label_3->setAlignment(Qt::AlignRight|Qt::AlignTrailing|Qt::AlignVCenter);

        formLayout->setWidget(2, QFormLayout::LabelRole, label_3);

        targetStateComboBox = new QComboBox(TransitionPropertyEditor);
        targetStateComboBox->setObjectName(QString::fromUtf8("targetStateComboBox"));
        targetStateComboBox->setEditable(true);

        formLayout->setWidget(2, QFormLayout::FieldRole, targetStateComboBox);

        signalLabel = new QLabel(TransitionPropertyEditor);
        signalLabel->setObjectName(QString::fromUtf8("signalLabel"));

        formLayout->setWidget(3, QFormLayout::LabelRole, signalLabel);

        signalEdit = new QLineEdit(TransitionPropertyEditor);
        signalEdit->setObjectName(QString::fromUtf8("signalEdit"));

        formLayout->setWidget(3, QFormLayout::FieldRole, signalEdit);

        timeoutLabel = new QLabel(TransitionPropertyEditor);
        timeoutLabel->setObjectName(QString::fromUtf8("timeoutLabel"));

        formLayout->setWidget(4, QFormLayout::LabelRole, timeoutLabel);

        timeoutEdit = new QSpinBox(TransitionPropertyEditor);
        timeoutEdit->setObjectName(QString::fromUtf8("timeoutEdit"));
        timeoutEdit->setMaximum(999999999);

        formLayout->setWidget(4, QFormLayout::FieldRole, timeoutEdit);


        verticalLayout_2->addLayout(formLayout);

        verticalLayout = new QVBoxLayout();
        verticalLayout->setObjectName(QString::fromUtf8("verticalLayout"));
        horizontalLayout = new QHBoxLayout();
        horizontalLayout->setObjectName(QString::fromUtf8("horizontalLayout"));
        label_4 = new QLabel(TransitionPropertyEditor);
        label_4->setObjectName(QString::fromUtf8("label_4"));
        label_4->setAlignment(Qt::AlignRight|Qt::AlignTrailing|Qt::AlignVCenter);

        horizontalLayout->addWidget(label_4);

        horizontalSpacer = new QSpacerItem(40, 20, QSizePolicy::Expanding, QSizePolicy::Minimum);

        horizontalLayout->addItem(horizontalSpacer);


        verticalLayout->addLayout(horizontalLayout);

        guardEditor = new CodeEditor(TransitionPropertyEditor);
        guardEditor->setObjectName(QString::fromUtf8("guardEditor"));
        QSizePolicy sizePolicy(QSizePolicy::Preferred, QSizePolicy::Expanding);
        sizePolicy.setHorizontalStretch(0);
        sizePolicy.setVerticalStretch(0);
        sizePolicy.setHeightForWidth(guardEditor->sizePolicy().hasHeightForWidth());
        guardEditor->setSizePolicy(sizePolicy);
        guardEditor->setFocusPolicy(Qt::StrongFocus);

        verticalLayout->addWidget(guardEditor);


        verticalLayout_2->addLayout(verticalLayout);

#if QT_CONFIG(shortcut)
        label->setBuddy(labelLineEdit);
        label_2->setBuddy(sourceStateComboBox);
        label_3->setBuddy(targetStateComboBox);
        signalLabel->setBuddy(signalEdit);
        timeoutLabel->setBuddy(timeoutEdit);
        label_4->setBuddy(guardEditor);
#endif // QT_CONFIG(shortcut)

        retranslateUi(TransitionPropertyEditor);

        QMetaObject::connectSlotsByName(TransitionPropertyEditor);
    } // setupUi

    void retranslateUi(QWidget *TransitionPropertyEditor)
    {
        TransitionPropertyEditor->setWindowTitle(QCoreApplication::translate("TransitionPropertyEditor", "Form", nullptr));
        label->setText(QCoreApplication::translate("TransitionPropertyEditor", "&Label:", nullptr));
        label_2->setText(QCoreApplication::translate("TransitionPropertyEditor", "&Source state:", nullptr));
        label_3->setText(QCoreApplication::translate("TransitionPropertyEditor", "&Target state:", nullptr));
        signalLabel->setText(QCoreApplication::translate("TransitionPropertyEditor", "&Signal:", nullptr));
        timeoutLabel->setText(QCoreApplication::translate("TransitionPropertyEditor", "&Timeout:", nullptr));
        timeoutEdit->setSuffix(QCoreApplication::translate("TransitionPropertyEditor", "ms", nullptr));
        label_4->setText(QCoreApplication::translate("TransitionPropertyEditor", "&Guard:", nullptr));
    } // retranslateUi

};

namespace Ui {
    class TransitionPropertyEditor: public Ui_TransitionPropertyEditor {};
} // namespace Ui

QT_END_NAMESPACE

#endif // UI_TRANSITIONPROPERTYEDITOR_H
