/****************************************************************************
** Meta object code from reading C++ file 'commandfactory_p.h'
**
** Created by: The Qt Meta Object Compiler version 67 (Qt 5.15.19)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include <memory>
#include "../../../../../KDStateMachineEditor-1.2.8/src/view/command/commandfactory_p.h"
#include <QtCore/qbytearray.h>
#include <QtCore/qmetatype.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'commandfactory_p.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 67
#error "This file was generated using the moc from 5.15.19. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

QT_BEGIN_MOC_NAMESPACE
QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
struct qt_meta_stringdata_CommandFactory_t {
    QByteArrayData data[39];
    char stringdata0[644];
};
#define QT_MOC_LITERAL(idx, ofs, len) \
    Q_STATIC_BYTE_ARRAY_DATA_HEADER_INITIALIZER_WITH_OFFSET(len, \
    qptrdiff(offsetof(qt_meta_stringdata_CommandFactory_t, stringdata0) + ofs \
        - idx * sizeof(QByteArrayData)) \
    )
static const qt_meta_stringdata_CommandFactory_t qt_meta_stringdata_CommandFactory = {
    {
QT_MOC_LITERAL(0, 0, 14), // "CommandFactory"
QT_MOC_LITERAL(1, 15, 12), // "modelChanged"
QT_MOC_LITERAL(2, 28, 0), // ""
QT_MOC_LITERAL(3, 29, 18), // "KDSME::StateModel*"
QT_MOC_LITERAL(4, 48, 5), // "model"
QT_MOC_LITERAL(5, 54, 13), // "createElement"
QT_MOC_LITERAL(6, 68, 28), // "KDSME::CreateElementCommand*"
QT_MOC_LITERAL(7, 97, 20), // "KDSME::Element::Type"
QT_MOC_LITERAL(8, 118, 4), // "type"
QT_MOC_LITERAL(9, 123, 15), // "KDSME::Element*"
QT_MOC_LITERAL(10, 139, 13), // "parentElement"
QT_MOC_LITERAL(11, 153, 13), // "deleteElement"
QT_MOC_LITERAL(12, 167, 28), // "KDSME::DeleteElementCommand*"
QT_MOC_LITERAL(13, 196, 25), // "KDSME::StateMachineScene*"
QT_MOC_LITERAL(14, 222, 4), // "view"
QT_MOC_LITERAL(15, 227, 7), // "element"
QT_MOC_LITERAL(16, 235, 14), // "layoutSnapshot"
QT_MOC_LITERAL(17, 250, 29), // "KDSME::LayoutSnapshotCommand*"
QT_MOC_LITERAL(18, 280, 4), // "text"
QT_MOC_LITERAL(19, 285, 14), // "modifyProperty"
QT_MOC_LITERAL(20, 300, 29), // "KDSME::ModifyPropertyCommand*"
QT_MOC_LITERAL(21, 330, 6), // "object"
QT_MOC_LITERAL(22, 337, 11), // "const char*"
QT_MOC_LITERAL(23, 349, 8), // "property"
QT_MOC_LITERAL(24, 358, 5), // "value"
QT_MOC_LITERAL(25, 364, 11), // "propertyMap"
QT_MOC_LITERAL(26, 376, 13), // "modifyElement"
QT_MOC_LITERAL(27, 390, 28), // "KDSME::ModifyElementCommand*"
QT_MOC_LITERAL(28, 419, 4), // "item"
QT_MOC_LITERAL(29, 424, 16), // "modifyTransition"
QT_MOC_LITERAL(30, 441, 31), // "KDSME::ModifyTransitionCommand*"
QT_MOC_LITERAL(31, 473, 18), // "KDSME::Transition*"
QT_MOC_LITERAL(32, 492, 10), // "transition"
QT_MOC_LITERAL(33, 503, 15), // "reparentElement"
QT_MOC_LITERAL(34, 519, 30), // "KDSME::ReparentElementCommand*"
QT_MOC_LITERAL(35, 550, 25), // "changeStateMachineElement"
QT_MOC_LITERAL(36, 576, 33), // "KDSME::ChangeStateMachineComm..."
QT_MOC_LITERAL(37, 610, 20), // "KDSME::StateMachine*"
QT_MOC_LITERAL(38, 631, 12) // "statemachine"

    },
    "CommandFactory\0modelChanged\0\0"
    "KDSME::StateModel*\0model\0createElement\0"
    "KDSME::CreateElementCommand*\0"
    "KDSME::Element::Type\0type\0KDSME::Element*\0"
    "parentElement\0deleteElement\0"
    "KDSME::DeleteElementCommand*\0"
    "KDSME::StateMachineScene*\0view\0element\0"
    "layoutSnapshot\0KDSME::LayoutSnapshotCommand*\0"
    "text\0modifyProperty\0KDSME::ModifyPropertyCommand*\0"
    "object\0const char*\0property\0value\0"
    "propertyMap\0modifyElement\0"
    "KDSME::ModifyElementCommand*\0item\0"
    "modifyTransition\0KDSME::ModifyTransitionCommand*\0"
    "KDSME::Transition*\0transition\0"
    "reparentElement\0KDSME::ReparentElementCommand*\0"
    "changeStateMachineElement\0"
    "KDSME::ChangeStateMachineCommand*\0"
    "KDSME::StateMachine*\0statemachine"
};
#undef QT_MOC_LITERAL

static const uint qt_meta_data_CommandFactory[] = {

 // content:
       8,       // revision
       0,       // classname
       0,    0, // classinfo
      14,   14, // methods
       1,  158, // properties
       0,    0, // enums/sets
       0,    0, // constructors
       0,       // flags
       1,       // signalCount

 // signals: name, argc, parameters, tag, flags
       1,    1,   84,    2, 0x06 /* Public */,

 // methods: name, argc, parameters, tag, flags
       5,    3,   87,    2, 0x02 /* Public */,
       5,    2,   94,    2, 0x22 /* Public | MethodCloned */,
      11,    2,   99,    2, 0x02 /* Public */,
      16,    2,  104,    2, 0x02 /* Public */,
      16,    1,  109,    2, 0x22 /* Public | MethodCloned */,
      19,    4,  112,    2, 0x02 /* Public */,
      19,    3,  121,    2, 0x22 /* Public | MethodCloned */,
      19,    3,  128,    2, 0x02 /* Public */,
      19,    2,  135,    2, 0x22 /* Public | MethodCloned */,
      26,    1,  140,    2, 0x02 /* Public */,
      29,    2,  143,    2, 0x02 /* Public */,
      33,    2,  148,    2, 0x02 /* Public */,
      35,    2,  153,    2, 0x02 /* Public */,

 // signals: parameters
    QMetaType::Void, 0x80000000 | 3,    4,

 // methods: parameters
    0x80000000 | 6, 0x80000000 | 3, 0x80000000 | 7, 0x80000000 | 9,    4,    8,   10,
    0x80000000 | 6, 0x80000000 | 3, 0x80000000 | 7,    4,    8,
    0x80000000 | 12, 0x80000000 | 13, 0x80000000 | 9,   14,   15,
    0x80000000 | 17, 0x80000000 | 13, QMetaType::QString,   14,   18,
    0x80000000 | 17, 0x80000000 | 13,   14,
    0x80000000 | 20, QMetaType::QObjectStar, 0x80000000 | 22, QMetaType::QVariant, QMetaType::QString,   21,   23,   24,   18,
    0x80000000 | 20, QMetaType::QObjectStar, 0x80000000 | 22, QMetaType::QVariant,   21,   23,   24,
    0x80000000 | 20, QMetaType::QObjectStar, QMetaType::QJsonObject, QMetaType::QString,   21,   25,   18,
    0x80000000 | 20, QMetaType::QObjectStar, QMetaType::QJsonObject,   21,   25,
    0x80000000 | 27, 0x80000000 | 9,   28,
    0x80000000 | 30, 0x80000000 | 31, 0x80000000 | 3,   32,    4,
    0x80000000 | 34, 0x80000000 | 13, 0x80000000 | 9,   14,   15,
    0x80000000 | 36, 0x80000000 | 13, 0x80000000 | 37,   14,   38,

 // properties: name, type, flags
       4, 0x80000000 | 3, 0x0049510b,

 // properties: notify_signal_id
       0,

       0        // eod
};

void CommandFactory::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    if (_c == QMetaObject::InvokeMetaMethod) {
        auto *_t = static_cast<CommandFactory *>(_o);
        (void)_t;
        switch (_id) {
        case 0: _t->modelChanged((*reinterpret_cast< KDSME::StateModel*(*)>(_a[1]))); break;
        case 1: { KDSME::CreateElementCommand* _r = _t->createElement((*reinterpret_cast< KDSME::StateModel*(*)>(_a[1])),(*reinterpret_cast< KDSME::Element::Type(*)>(_a[2])),(*reinterpret_cast< KDSME::Element*(*)>(_a[3])));
            if (_a[0]) *reinterpret_cast< KDSME::CreateElementCommand**>(_a[0]) = std::move(_r); }  break;
        case 2: { KDSME::CreateElementCommand* _r = _t->createElement((*reinterpret_cast< KDSME::StateModel*(*)>(_a[1])),(*reinterpret_cast< KDSME::Element::Type(*)>(_a[2])));
            if (_a[0]) *reinterpret_cast< KDSME::CreateElementCommand**>(_a[0]) = std::move(_r); }  break;
        case 3: { KDSME::DeleteElementCommand* _r = _t->deleteElement((*reinterpret_cast< KDSME::StateMachineScene*(*)>(_a[1])),(*reinterpret_cast< KDSME::Element*(*)>(_a[2])));
            if (_a[0]) *reinterpret_cast< KDSME::DeleteElementCommand**>(_a[0]) = std::move(_r); }  break;
        case 4: { KDSME::LayoutSnapshotCommand* _r = _t->layoutSnapshot((*reinterpret_cast< KDSME::StateMachineScene*(*)>(_a[1])),(*reinterpret_cast< const QString(*)>(_a[2])));
            if (_a[0]) *reinterpret_cast< KDSME::LayoutSnapshotCommand**>(_a[0]) = std::move(_r); }  break;
        case 5: { KDSME::LayoutSnapshotCommand* _r = _t->layoutSnapshot((*reinterpret_cast< KDSME::StateMachineScene*(*)>(_a[1])));
            if (_a[0]) *reinterpret_cast< KDSME::LayoutSnapshotCommand**>(_a[0]) = std::move(_r); }  break;
        case 6: { KDSME::ModifyPropertyCommand* _r = _t->modifyProperty((*reinterpret_cast< QObject*(*)>(_a[1])),(*reinterpret_cast< const char*(*)>(_a[2])),(*reinterpret_cast< const QVariant(*)>(_a[3])),(*reinterpret_cast< const QString(*)>(_a[4])));
            if (_a[0]) *reinterpret_cast< KDSME::ModifyPropertyCommand**>(_a[0]) = std::move(_r); }  break;
        case 7: { KDSME::ModifyPropertyCommand* _r = _t->modifyProperty((*reinterpret_cast< QObject*(*)>(_a[1])),(*reinterpret_cast< const char*(*)>(_a[2])),(*reinterpret_cast< const QVariant(*)>(_a[3])));
            if (_a[0]) *reinterpret_cast< KDSME::ModifyPropertyCommand**>(_a[0]) = std::move(_r); }  break;
        case 8: { KDSME::ModifyPropertyCommand* _r = _t->modifyProperty((*reinterpret_cast< QObject*(*)>(_a[1])),(*reinterpret_cast< const QJsonObject(*)>(_a[2])),(*reinterpret_cast< const QString(*)>(_a[3])));
            if (_a[0]) *reinterpret_cast< KDSME::ModifyPropertyCommand**>(_a[0]) = std::move(_r); }  break;
        case 9: { KDSME::ModifyPropertyCommand* _r = _t->modifyProperty((*reinterpret_cast< QObject*(*)>(_a[1])),(*reinterpret_cast< const QJsonObject(*)>(_a[2])));
            if (_a[0]) *reinterpret_cast< KDSME::ModifyPropertyCommand**>(_a[0]) = std::move(_r); }  break;
        case 10: { KDSME::ModifyElementCommand* _r = _t->modifyElement((*reinterpret_cast< KDSME::Element*(*)>(_a[1])));
            if (_a[0]) *reinterpret_cast< KDSME::ModifyElementCommand**>(_a[0]) = std::move(_r); }  break;
        case 11: { KDSME::ModifyTransitionCommand* _r = _t->modifyTransition((*reinterpret_cast< KDSME::Transition*(*)>(_a[1])),(*reinterpret_cast< KDSME::StateModel*(*)>(_a[2])));
            if (_a[0]) *reinterpret_cast< KDSME::ModifyTransitionCommand**>(_a[0]) = std::move(_r); }  break;
        case 12: { KDSME::ReparentElementCommand* _r = _t->reparentElement((*reinterpret_cast< KDSME::StateMachineScene*(*)>(_a[1])),(*reinterpret_cast< KDSME::Element*(*)>(_a[2])));
            if (_a[0]) *reinterpret_cast< KDSME::ReparentElementCommand**>(_a[0]) = std::move(_r); }  break;
        case 13: { KDSME::ChangeStateMachineCommand* _r = _t->changeStateMachineElement((*reinterpret_cast< KDSME::StateMachineScene*(*)>(_a[1])),(*reinterpret_cast< KDSME::StateMachine*(*)>(_a[2])));
            if (_a[0]) *reinterpret_cast< KDSME::ChangeStateMachineCommand**>(_a[0]) = std::move(_r); }  break;
        default: ;
        }
    } else if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        switch (_id) {
        default: *reinterpret_cast<int*>(_a[0]) = -1; break;
        case 1:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<int*>(_a[0]) = -1; break;
            case 2:
                *reinterpret_cast<int*>(_a[0]) = qRegisterMetaType< KDSME::Element* >(); break;
            case 1:
                *reinterpret_cast<int*>(_a[0]) = qRegisterMetaType< KDSME::Element::Type >(); break;
            }
            break;
        case 2:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<int*>(_a[0]) = -1; break;
            case 1:
                *reinterpret_cast<int*>(_a[0]) = qRegisterMetaType< KDSME::Element::Type >(); break;
            }
            break;
        case 3:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<int*>(_a[0]) = -1; break;
            case 1:
                *reinterpret_cast<int*>(_a[0]) = qRegisterMetaType< KDSME::Element* >(); break;
            }
            break;
        case 10:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<int*>(_a[0]) = -1; break;
            case 0:
                *reinterpret_cast<int*>(_a[0]) = qRegisterMetaType< KDSME::Element* >(); break;
            }
            break;
        case 12:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<int*>(_a[0]) = -1; break;
            case 1:
                *reinterpret_cast<int*>(_a[0]) = qRegisterMetaType< KDSME::Element* >(); break;
            }
            break;
        }
    } else if (_c == QMetaObject::IndexOfMethod) {
        int *result = reinterpret_cast<int *>(_a[0]);
        {
            using _t = void (CommandFactory::*)(KDSME::StateModel * );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&CommandFactory::modelChanged)) {
                *result = 0;
                return;
            }
        }
    }
#ifndef QT_NO_PROPERTIES
    else if (_c == QMetaObject::ReadProperty) {
        auto *_t = static_cast<CommandFactory *>(_o);
        (void)_t;
        void *_v = _a[0];
        switch (_id) {
        case 0: *reinterpret_cast< KDSME::StateModel**>(_v) = _t->model(); break;
        default: break;
        }
    } else if (_c == QMetaObject::WriteProperty) {
        auto *_t = static_cast<CommandFactory *>(_o);
        (void)_t;
        void *_v = _a[0];
        switch (_id) {
        case 0: _t->setModel(*reinterpret_cast< KDSME::StateModel**>(_v)); break;
        default: break;
        }
    } else if (_c == QMetaObject::ResetProperty) {
    }
#endif // QT_NO_PROPERTIES
}

QT_INIT_METAOBJECT const QMetaObject CommandFactory::staticMetaObject = { {
    QMetaObject::SuperData::link<QObject::staticMetaObject>(),
    qt_meta_stringdata_CommandFactory.data,
    qt_meta_data_CommandFactory,
    qt_static_metacall,
    nullptr,
    nullptr
} };


const QMetaObject *CommandFactory::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *CommandFactory::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_meta_stringdata_CommandFactory.stringdata0))
        return static_cast<void*>(this);
    return QObject::qt_metacast(_clname);
}

int CommandFactory::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QObject::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 14)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 14;
    } else if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 14)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 14;
    }
#ifndef QT_NO_PROPERTIES
    else if (_c == QMetaObject::ReadProperty || _c == QMetaObject::WriteProperty
            || _c == QMetaObject::ResetProperty || _c == QMetaObject::RegisterPropertyMetaType) {
        qt_static_metacall(this, _c, _id, _a);
        _id -= 1;
    } else if (_c == QMetaObject::QueryPropertyDesignable) {
        _id -= 1;
    } else if (_c == QMetaObject::QueryPropertyScriptable) {
        _id -= 1;
    } else if (_c == QMetaObject::QueryPropertyStored) {
        _id -= 1;
    } else if (_c == QMetaObject::QueryPropertyEditable) {
        _id -= 1;
    } else if (_c == QMetaObject::QueryPropertyUser) {
        _id -= 1;
    }
#endif // QT_NO_PROPERTIES
    return _id;
}

// SIGNAL 0
void CommandFactory::modelChanged(KDSME::StateModel * _t1)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))) };
    QMetaObject::activate(this, &staticMetaObject, 0, _a);
}
QT_WARNING_POP
QT_END_MOC_NAMESPACE
