/****************************************************************************
** Meta object code from reading C++ file 'quicksceneitem_p.h'
**
** Created by: The Qt Meta Object Compiler version 67 (Qt 5.15.19)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include <memory>
#include "../../../../../KDStateMachineEditor-1.2.8/src/view/quick/quicksceneitem_p.h"
#include <QtCore/qbytearray.h>
#include <QtCore/qmetatype.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'quicksceneitem_p.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 67
#error "This file was generated using the moc from 5.15.19. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

QT_BEGIN_MOC_NAMESPACE
QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
struct qt_meta_stringdata_QuickSceneItem_t {
    QByteArrayData data[15];
    char stringdata0[184];
};
#define QT_MOC_LITERAL(idx, ofs, len) \
    Q_STATIC_BYTE_ARRAY_DATA_HEADER_INITIALIZER_WITH_OFFSET(len, \
    qptrdiff(offsetof(qt_meta_stringdata_QuickSceneItem_t, stringdata0) + ofs \
        - idx * sizeof(QByteArrayData)) \
    )
static const qt_meta_stringdata_QuickSceneItem_t qt_meta_stringdata_QuickSceneItem = {
    {
QT_MOC_LITERAL(0, 0, 14), // "QuickSceneItem"
QT_MOC_LITERAL(1, 15, 12), // "sceneChanged"
QT_MOC_LITERAL(2, 28, 0), // ""
QT_MOC_LITERAL(3, 29, 25), // "KDSME::StateMachineScene*"
QT_MOC_LITERAL(4, 55, 5), // "scene"
QT_MOC_LITERAL(5, 61, 14), // "elementChanged"
QT_MOC_LITERAL(6, 76, 15), // "KDSME::Element*"
QT_MOC_LITERAL(7, 92, 7), // "element"
QT_MOC_LITERAL(8, 100, 12), // "shapeChanged"
QT_MOC_LITERAL(9, 113, 12), // "QPainterPath"
QT_MOC_LITERAL(10, 126, 5), // "shape"
QT_MOC_LITERAL(11, 132, 17), // "activenessChanged"
QT_MOC_LITERAL(12, 150, 10), // "activeness"
QT_MOC_LITERAL(13, 161, 7), // "clicked"
QT_MOC_LITERAL(14, 169, 14) // "sendClickEvent"

    },
    "QuickSceneItem\0sceneChanged\0\0"
    "KDSME::StateMachineScene*\0scene\0"
    "elementChanged\0KDSME::Element*\0element\0"
    "shapeChanged\0QPainterPath\0shape\0"
    "activenessChanged\0activeness\0clicked\0"
    "sendClickEvent"
};
#undef QT_MOC_LITERAL

static const uint qt_meta_data_QuickSceneItem[] = {

 // content:
       8,       // revision
       0,       // classname
       0,    0, // classinfo
       6,   14, // methods
       4,   58, // properties
       0,    0, // enums/sets
       0,    0, // constructors
       0,       // flags
       5,       // signalCount

 // signals: name, argc, parameters, tag, flags
       1,    1,   44,    2, 0x06 /* Public */,
       5,    1,   47,    2, 0x06 /* Public */,
       8,    1,   50,    2, 0x06 /* Public */,
      11,    1,   53,    2, 0x06 /* Public */,
      13,    0,   56,    2, 0x06 /* Public */,

 // methods: name, argc, parameters, tag, flags
      14,    0,   57,    2, 0x01 /* Protected */,

 // signals: parameters
    QMetaType::Void, 0x80000000 | 3,    4,
    QMetaType::Void, 0x80000000 | 6,    7,
    QMetaType::Void, 0x80000000 | 9,   10,
    QMetaType::Void, QMetaType::QReal,   12,
    QMetaType::Void,

 // methods: parameters
    QMetaType::Void,

 // properties: name, type, flags
       4, 0x80000000 | 3, 0x0049590b,
       7, 0x80000000 | 6, 0x0049590b,
      10, 0x80000000 | 9, 0x0049590b,
      12, QMetaType::QReal, 0x00495903,

 // properties: notify_signal_id
       0,
       1,
       2,
       3,

       0        // eod
};

void QuickSceneItem::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    if (_c == QMetaObject::InvokeMetaMethod) {
        auto *_t = static_cast<QuickSceneItem *>(_o);
        (void)_t;
        switch (_id) {
        case 0: _t->sceneChanged((*reinterpret_cast< KDSME::StateMachineScene*(*)>(_a[1]))); break;
        case 1: _t->elementChanged((*reinterpret_cast< KDSME::Element*(*)>(_a[1]))); break;
        case 2: _t->shapeChanged((*reinterpret_cast< const QPainterPath(*)>(_a[1]))); break;
        case 3: _t->activenessChanged((*reinterpret_cast< qreal(*)>(_a[1]))); break;
        case 4: _t->clicked(); break;
        case 5: _t->sendClickEvent(); break;
        default: ;
        }
    } else if (_c == QMetaObject::IndexOfMethod) {
        int *result = reinterpret_cast<int *>(_a[0]);
        {
            using _t = void (QuickSceneItem::*)(KDSME::StateMachineScene * );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&QuickSceneItem::sceneChanged)) {
                *result = 0;
                return;
            }
        }
        {
            using _t = void (QuickSceneItem::*)(KDSME::Element * );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&QuickSceneItem::elementChanged)) {
                *result = 1;
                return;
            }
        }
        {
            using _t = void (QuickSceneItem::*)(const QPainterPath & );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&QuickSceneItem::shapeChanged)) {
                *result = 2;
                return;
            }
        }
        {
            using _t = void (QuickSceneItem::*)(qreal );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&QuickSceneItem::activenessChanged)) {
                *result = 3;
                return;
            }
        }
        {
            using _t = void (QuickSceneItem::*)();
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&QuickSceneItem::clicked)) {
                *result = 4;
                return;
            }
        }
    }
#ifndef QT_NO_PROPERTIES
    else if (_c == QMetaObject::ReadProperty) {
        auto *_t = static_cast<QuickSceneItem *>(_o);
        (void)_t;
        void *_v = _a[0];
        switch (_id) {
        case 0: *reinterpret_cast< KDSME::StateMachineScene**>(_v) = _t->scene(); break;
        case 1: *reinterpret_cast< KDSME::Element**>(_v) = _t->element(); break;
        case 2: *reinterpret_cast< QPainterPath*>(_v) = _t->shape(); break;
        case 3: *reinterpret_cast< qreal*>(_v) = _t->activeness(); break;
        default: break;
        }
    } else if (_c == QMetaObject::WriteProperty) {
        auto *_t = static_cast<QuickSceneItem *>(_o);
        (void)_t;
        void *_v = _a[0];
        switch (_id) {
        case 0: _t->setScene(*reinterpret_cast< KDSME::StateMachineScene**>(_v)); break;
        case 1: _t->setElement(*reinterpret_cast< KDSME::Element**>(_v)); break;
        case 2: _t->setShape(*reinterpret_cast< QPainterPath*>(_v)); break;
        case 3: _t->setActiveness(*reinterpret_cast< qreal*>(_v)); break;
        default: break;
        }
    } else if (_c == QMetaObject::ResetProperty) {
    }
#endif // QT_NO_PROPERTIES
}

QT_INIT_METAOBJECT const QMetaObject QuickSceneItem::staticMetaObject = { {
    QMetaObject::SuperData::link<QQuickItem::staticMetaObject>(),
    qt_meta_stringdata_QuickSceneItem.data,
    qt_meta_data_QuickSceneItem,
    qt_static_metacall,
    nullptr,
    nullptr
} };


const QMetaObject *QuickSceneItem::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *QuickSceneItem::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_meta_stringdata_QuickSceneItem.stringdata0))
        return static_cast<void*>(this);
    return QQuickItem::qt_metacast(_clname);
}

int QuickSceneItem::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QQuickItem::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 6)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 6;
    } else if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 6)
            *reinterpret_cast<int*>(_a[0]) = -1;
        _id -= 6;
    }
#ifndef QT_NO_PROPERTIES
    else if (_c == QMetaObject::ReadProperty || _c == QMetaObject::WriteProperty
            || _c == QMetaObject::ResetProperty || _c == QMetaObject::RegisterPropertyMetaType) {
        qt_static_metacall(this, _c, _id, _a);
        _id -= 4;
    } else if (_c == QMetaObject::QueryPropertyDesignable) {
        _id -= 4;
    } else if (_c == QMetaObject::QueryPropertyScriptable) {
        _id -= 4;
    } else if (_c == QMetaObject::QueryPropertyStored) {
        _id -= 4;
    } else if (_c == QMetaObject::QueryPropertyEditable) {
        _id -= 4;
    } else if (_c == QMetaObject::QueryPropertyUser) {
        _id -= 4;
    }
#endif // QT_NO_PROPERTIES
    return _id;
}

// SIGNAL 0
void QuickSceneItem::sceneChanged(KDSME::StateMachineScene * _t1)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))) };
    QMetaObject::activate(this, &staticMetaObject, 0, _a);
}

// SIGNAL 1
void QuickSceneItem::elementChanged(KDSME::Element * _t1)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))) };
    QMetaObject::activate(this, &staticMetaObject, 1, _a);
}

// SIGNAL 2
void QuickSceneItem::shapeChanged(const QPainterPath & _t1)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))) };
    QMetaObject::activate(this, &staticMetaObject, 2, _a);
}

// SIGNAL 3
void QuickSceneItem::activenessChanged(qreal _t1)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))) };
    QMetaObject::activate(this, &staticMetaObject, 3, _a);
}

// SIGNAL 4
void QuickSceneItem::clicked()
{
    QMetaObject::activate(this, &staticMetaObject, 4, nullptr);
}
struct qt_meta_stringdata_QuickStateItem_t {
    QByteArrayData data[1];
    char stringdata0[15];
};
#define QT_MOC_LITERAL(idx, ofs, len) \
    Q_STATIC_BYTE_ARRAY_DATA_HEADER_INITIALIZER_WITH_OFFSET(len, \
    qptrdiff(offsetof(qt_meta_stringdata_QuickStateItem_t, stringdata0) + ofs \
        - idx * sizeof(QByteArrayData)) \
    )
static const qt_meta_stringdata_QuickStateItem_t qt_meta_stringdata_QuickStateItem = {
    {
QT_MOC_LITERAL(0, 0, 14) // "QuickStateItem"

    },
    "QuickStateItem"
};
#undef QT_MOC_LITERAL

static const uint qt_meta_data_QuickStateItem[] = {

 // content:
       8,       // revision
       0,       // classname
       0,    0, // classinfo
       0,    0, // methods
       0,    0, // properties
       0,    0, // enums/sets
       0,    0, // constructors
       0,       // flags
       0,       // signalCount

       0        // eod
};

void QuickStateItem::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    (void)_o;
    (void)_id;
    (void)_c;
    (void)_a;
}

QT_INIT_METAOBJECT const QMetaObject QuickStateItem::staticMetaObject = { {
    QMetaObject::SuperData::link<QuickSceneItem::staticMetaObject>(),
    qt_meta_stringdata_QuickStateItem.data,
    qt_meta_data_QuickStateItem,
    qt_static_metacall,
    nullptr,
    nullptr
} };


const QMetaObject *QuickStateItem::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *QuickStateItem::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_meta_stringdata_QuickStateItem.stringdata0))
        return static_cast<void*>(this);
    return QuickSceneItem::qt_metacast(_clname);
}

int QuickStateItem::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QuickSceneItem::qt_metacall(_c, _id, _a);
    return _id;
}
struct qt_meta_stringdata_QuickTransitionItem_t {
    QByteArrayData data[5];
    char stringdata0[62];
};
#define QT_MOC_LITERAL(idx, ofs, len) \
    Q_STATIC_BYTE_ARRAY_DATA_HEADER_INITIALIZER_WITH_OFFSET(len, \
    qptrdiff(offsetof(qt_meta_stringdata_QuickTransitionItem_t, stringdata0) + ofs \
        - idx * sizeof(QByteArrayData)) \
    )
static const qt_meta_stringdata_QuickTransitionItem_t qt_meta_stringdata_QuickTransitionItem = {
    {
QT_MOC_LITERAL(0, 0, 19), // "QuickTransitionItem"
QT_MOC_LITERAL(1, 20, 14), // "updatePosition"
QT_MOC_LITERAL(2, 35, 0), // ""
QT_MOC_LITERAL(3, 36, 12), // "updateSource"
QT_MOC_LITERAL(4, 49, 12) // "updateTarget"

    },
    "QuickTransitionItem\0updatePosition\0\0"
    "updateSource\0updateTarget"
};
#undef QT_MOC_LITERAL

static const uint qt_meta_data_QuickTransitionItem[] = {

 // content:
       8,       // revision
       0,       // classname
       0,    0, // classinfo
       3,   14, // methods
       0,    0, // properties
       0,    0, // enums/sets
       0,    0, // constructors
       0,       // flags
       0,       // signalCount

 // slots: name, argc, parameters, tag, flags
       1,    0,   29,    2, 0x08 /* Private */,
       3,    0,   30,    2, 0x08 /* Private */,
       4,    0,   31,    2, 0x08 /* Private */,

 // slots: parameters
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,

       0        // eod
};

void QuickTransitionItem::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    if (_c == QMetaObject::InvokeMetaMethod) {
        auto *_t = static_cast<QuickTransitionItem *>(_o);
        (void)_t;
        switch (_id) {
        case 0: _t->updatePosition(); break;
        case 1: _t->updateSource(); break;
        case 2: _t->updateTarget(); break;
        default: ;
        }
    }
    (void)_a;
}

QT_INIT_METAOBJECT const QMetaObject QuickTransitionItem::staticMetaObject = { {
    QMetaObject::SuperData::link<QuickSceneItem::staticMetaObject>(),
    qt_meta_stringdata_QuickTransitionItem.data,
    qt_meta_data_QuickTransitionItem,
    qt_static_metacall,
    nullptr,
    nullptr
} };


const QMetaObject *QuickTransitionItem::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *QuickTransitionItem::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_meta_stringdata_QuickTransitionItem.stringdata0))
        return static_cast<void*>(this);
    return QuickSceneItem::qt_metacast(_clname);
}

int QuickTransitionItem::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QuickSceneItem::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 3)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 3;
    } else if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 3)
            *reinterpret_cast<int*>(_a[0]) = -1;
        _id -= 3;
    }
    return _id;
}
QT_WARNING_POP
QT_END_MOC_NAMESPACE
