/****************************************************************************
** Meta object code from reading C++ file 'quickpainterpath_p.h'
**
** Created by: The Qt Meta Object Compiler version 67 (Qt 5.15.19)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include <memory>
#include "../../../../../KDStateMachineEditor-1.2.8/src/view/quick/quickpainterpath_p.h"
#include <QtCore/qbytearray.h>
#include <QtCore/qmetatype.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'quickpainterpath_p.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 67
#error "This file was generated using the moc from 5.15.19. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

QT_BEGIN_MOC_NAMESPACE
QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
struct qt_meta_stringdata_QuickPainterPathStroker_t {
    QByteArrayData data[7];
    char stringdata0[75];
};
#define QT_MOC_LITERAL(idx, ofs, len) \
    Q_STATIC_BYTE_ARRAY_DATA_HEADER_INITIALIZER_WITH_OFFSET(len, \
    qptrdiff(offsetof(qt_meta_stringdata_QuickPainterPathStroker_t, stringdata0) + ofs \
        - idx * sizeof(QByteArrayData)) \
    )
static const qt_meta_stringdata_QuickPainterPathStroker_t qt_meta_stringdata_QuickPainterPathStroker = {
    {
QT_MOC_LITERAL(0, 0, 23), // "QuickPainterPathStroker"
QT_MOC_LITERAL(1, 24, 12), // "widthChanged"
QT_MOC_LITERAL(2, 37, 0), // ""
QT_MOC_LITERAL(3, 38, 5), // "width"
QT_MOC_LITERAL(4, 44, 12), // "createStroke"
QT_MOC_LITERAL(5, 57, 12), // "QPainterPath"
QT_MOC_LITERAL(6, 70, 4) // "path"

    },
    "QuickPainterPathStroker\0widthChanged\0"
    "\0width\0createStroke\0QPainterPath\0path"
};
#undef QT_MOC_LITERAL

static const uint qt_meta_data_QuickPainterPathStroker[] = {

 // content:
       8,       // revision
       0,       // classname
       0,    0, // classinfo
       2,   14, // methods
       1,   30, // properties
       0,    0, // enums/sets
       0,    0, // constructors
       0,       // flags
       1,       // signalCount

 // signals: name, argc, parameters, tag, flags
       1,    1,   24,    2, 0x06 /* Public */,

 // methods: name, argc, parameters, tag, flags
       4,    1,   27,    2, 0x02 /* Public */,

 // signals: parameters
    QMetaType::Void, QMetaType::QReal,    3,

 // methods: parameters
    0x80000000 | 5, 0x80000000 | 5,    6,

 // properties: name, type, flags
       3, QMetaType::QReal, 0x00495103,

 // properties: notify_signal_id
       0,

       0        // eod
};

void QuickPainterPathStroker::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    if (_c == QMetaObject::InvokeMetaMethod) {
        auto *_t = static_cast<QuickPainterPathStroker *>(_o);
        (void)_t;
        switch (_id) {
        case 0: _t->widthChanged((*reinterpret_cast< qreal(*)>(_a[1]))); break;
        case 1: { QPainterPath _r = _t->createStroke((*reinterpret_cast< const QPainterPath(*)>(_a[1])));
            if (_a[0]) *reinterpret_cast< QPainterPath*>(_a[0]) = std::move(_r); }  break;
        default: ;
        }
    } else if (_c == QMetaObject::IndexOfMethod) {
        int *result = reinterpret_cast<int *>(_a[0]);
        {
            using _t = void (QuickPainterPathStroker::*)(qreal );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&QuickPainterPathStroker::widthChanged)) {
                *result = 0;
                return;
            }
        }
    }
#ifndef QT_NO_PROPERTIES
    else if (_c == QMetaObject::ReadProperty) {
        auto *_t = static_cast<QuickPainterPathStroker *>(_o);
        (void)_t;
        void *_v = _a[0];
        switch (_id) {
        case 0: *reinterpret_cast< qreal*>(_v) = _t->width(); break;
        default: break;
        }
    } else if (_c == QMetaObject::WriteProperty) {
        auto *_t = static_cast<QuickPainterPathStroker *>(_o);
        (void)_t;
        void *_v = _a[0];
        switch (_id) {
        case 0: _t->setWidth(*reinterpret_cast< qreal*>(_v)); break;
        default: break;
        }
    } else if (_c == QMetaObject::ResetProperty) {
    }
#endif // QT_NO_PROPERTIES
}

QT_INIT_METAOBJECT const QMetaObject QuickPainterPathStroker::staticMetaObject = { {
    QMetaObject::SuperData::link<QObject::staticMetaObject>(),
    qt_meta_stringdata_QuickPainterPathStroker.data,
    qt_meta_data_QuickPainterPathStroker,
    qt_static_metacall,
    nullptr,
    nullptr
} };


const QMetaObject *QuickPainterPathStroker::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *QuickPainterPathStroker::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_meta_stringdata_QuickPainterPathStroker.stringdata0))
        return static_cast<void*>(this);
    return QObject::qt_metacast(_clname);
}

int QuickPainterPathStroker::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QObject::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 2)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 2;
    } else if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 2)
            *reinterpret_cast<int*>(_a[0]) = -1;
        _id -= 2;
    }
#ifndef QT_NO_PROPERTIES
    else if (_c == QMetaObject::ReadProperty || _c == QMetaObject::WriteProperty
            || _c == QMetaObject::ResetProperty || _c == QMetaObject::RegisterPropertyMetaType) {
        qt_static_metacall(this, _c, _id, _a);
        _id -= 1;
    } else if (_c == QMetaObject::QueryPropertyDesignable) {
        _id -= 1;
    } else if (_c == QMetaObject::QueryPropertyScriptable) {
        _id -= 1;
    } else if (_c == QMetaObject::QueryPropertyStored) {
        _id -= 1;
    } else if (_c == QMetaObject::QueryPropertyEditable) {
        _id -= 1;
    } else if (_c == QMetaObject::QueryPropertyUser) {
        _id -= 1;
    }
#endif // QT_NO_PROPERTIES
    return _id;
}

// SIGNAL 0
void QuickPainterPathStroker::widthChanged(qreal _t1)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))) };
    QMetaObject::activate(this, &staticMetaObject, 0, _a);
}
struct qt_meta_stringdata_QuickPainterPath_t {
    QByteArrayData data[18];
    char stringdata0[177];
};
#define QT_MOC_LITERAL(idx, ofs, len) \
    Q_STATIC_BYTE_ARRAY_DATA_HEADER_INITIALIZER_WITH_OFFSET(len, \
    qptrdiff(offsetof(qt_meta_stringdata_QuickPainterPath_t, stringdata0) + ofs \
        - idx * sizeof(QByteArrayData)) \
    )
static const qt_meta_stringdata_QuickPainterPath_t qt_meta_stringdata_QuickPainterPath = {
    {
QT_MOC_LITERAL(0, 0, 16), // "QuickPainterPath"
QT_MOC_LITERAL(1, 17, 11), // "pathChanged"
QT_MOC_LITERAL(2, 29, 0), // ""
QT_MOC_LITERAL(3, 30, 12), // "QPainterPath"
QT_MOC_LITERAL(4, 43, 4), // "path"
QT_MOC_LITERAL(5, 48, 17), // "startPointChanged"
QT_MOC_LITERAL(6, 66, 10), // "startPoint"
QT_MOC_LITERAL(7, 77, 15), // "endPointChanged"
QT_MOC_LITERAL(8, 93, 8), // "endPoint"
QT_MOC_LITERAL(9, 102, 14), // "isEmptyChanged"
QT_MOC_LITERAL(10, 117, 7), // "isEmpty"
QT_MOC_LITERAL(11, 125, 6), // "moveTo"
QT_MOC_LITERAL(12, 132, 5), // "point"
QT_MOC_LITERAL(13, 138, 6), // "lineTo"
QT_MOC_LITERAL(14, 145, 5), // "clear"
QT_MOC_LITERAL(15, 151, 8), // "contains"
QT_MOC_LITERAL(16, 160, 14), // "angleAtPercent"
QT_MOC_LITERAL(17, 175, 1) // "t"

    },
    "QuickPainterPath\0pathChanged\0\0"
    "QPainterPath\0path\0startPointChanged\0"
    "startPoint\0endPointChanged\0endPoint\0"
    "isEmptyChanged\0isEmpty\0moveTo\0point\0"
    "lineTo\0clear\0contains\0angleAtPercent\0"
    "t"
};
#undef QT_MOC_LITERAL

static const uint qt_meta_data_QuickPainterPath[] = {

 // content:
       8,       // revision
       0,       // classname
       0,    0, // classinfo
       9,   14, // methods
       4,   84, // properties
       0,    0, // enums/sets
       0,    0, // constructors
       0,       // flags
       4,       // signalCount

 // signals: name, argc, parameters, tag, flags
       1,    1,   59,    2, 0x06 /* Public */,
       5,    1,   62,    2, 0x06 /* Public */,
       7,    1,   65,    2, 0x06 /* Public */,
       9,    1,   68,    2, 0x06 /* Public */,

 // methods: name, argc, parameters, tag, flags
      11,    1,   71,    2, 0x02 /* Public */,
      13,    1,   74,    2, 0x02 /* Public */,
      14,    0,   77,    2, 0x02 /* Public */,
      15,    1,   78,    2, 0x02 /* Public */,
      16,    1,   81,    2, 0x02 /* Public */,

 // signals: parameters
    QMetaType::Void, 0x80000000 | 3,    4,
    QMetaType::Void, QMetaType::QPointF,    6,
    QMetaType::Void, QMetaType::QPointF,    8,
    QMetaType::Void, QMetaType::Bool,   10,

 // methods: parameters
    QMetaType::Void, QMetaType::QPointF,   12,
    QMetaType::Void, QMetaType::QPointF,    8,
    QMetaType::Void,
    QMetaType::Bool, QMetaType::QPointF,   12,
    QMetaType::QReal, QMetaType::QReal,   17,

 // properties: name, type, flags
       4, 0x80000000 | 3, 0x0049510b,
       6, QMetaType::QPointF, 0x00495001,
       8, QMetaType::QPointF, 0x00495001,
      10, QMetaType::Bool, 0x00495001,

 // properties: notify_signal_id
       0,
       1,
       2,
       3,

       0        // eod
};

void QuickPainterPath::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    if (_c == QMetaObject::InvokeMetaMethod) {
        auto *_t = static_cast<QuickPainterPath *>(_o);
        (void)_t;
        switch (_id) {
        case 0: _t->pathChanged((*reinterpret_cast< const QPainterPath(*)>(_a[1]))); break;
        case 1: _t->startPointChanged((*reinterpret_cast< const QPointF(*)>(_a[1]))); break;
        case 2: _t->endPointChanged((*reinterpret_cast< const QPointF(*)>(_a[1]))); break;
        case 3: _t->isEmptyChanged((*reinterpret_cast< bool(*)>(_a[1]))); break;
        case 4: _t->moveTo((*reinterpret_cast< const QPointF(*)>(_a[1]))); break;
        case 5: _t->lineTo((*reinterpret_cast< const QPointF(*)>(_a[1]))); break;
        case 6: _t->clear(); break;
        case 7: { bool _r = _t->contains((*reinterpret_cast< const QPointF(*)>(_a[1])));
            if (_a[0]) *reinterpret_cast< bool*>(_a[0]) = std::move(_r); }  break;
        case 8: { qreal _r = _t->angleAtPercent((*reinterpret_cast< qreal(*)>(_a[1])));
            if (_a[0]) *reinterpret_cast< qreal*>(_a[0]) = std::move(_r); }  break;
        default: ;
        }
    } else if (_c == QMetaObject::IndexOfMethod) {
        int *result = reinterpret_cast<int *>(_a[0]);
        {
            using _t = void (QuickPainterPath::*)(const QPainterPath & );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&QuickPainterPath::pathChanged)) {
                *result = 0;
                return;
            }
        }
        {
            using _t = void (QuickPainterPath::*)(const QPointF & );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&QuickPainterPath::startPointChanged)) {
                *result = 1;
                return;
            }
        }
        {
            using _t = void (QuickPainterPath::*)(const QPointF & );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&QuickPainterPath::endPointChanged)) {
                *result = 2;
                return;
            }
        }
        {
            using _t = void (QuickPainterPath::*)(bool );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&QuickPainterPath::isEmptyChanged)) {
                *result = 3;
                return;
            }
        }
    }
#ifndef QT_NO_PROPERTIES
    else if (_c == QMetaObject::ReadProperty) {
        auto *_t = static_cast<QuickPainterPath *>(_o);
        (void)_t;
        void *_v = _a[0];
        switch (_id) {
        case 0: *reinterpret_cast< QPainterPath*>(_v) = _t->path(); break;
        case 1: *reinterpret_cast< QPointF*>(_v) = _t->startPoint(); break;
        case 2: *reinterpret_cast< QPointF*>(_v) = _t->endPoint(); break;
        case 3: *reinterpret_cast< bool*>(_v) = _t->isEmpty(); break;
        default: break;
        }
    } else if (_c == QMetaObject::WriteProperty) {
        auto *_t = static_cast<QuickPainterPath *>(_o);
        (void)_t;
        void *_v = _a[0];
        switch (_id) {
        case 0: _t->setPath(*reinterpret_cast< QPainterPath*>(_v)); break;
        default: break;
        }
    } else if (_c == QMetaObject::ResetProperty) {
    }
#endif // QT_NO_PROPERTIES
}

QT_INIT_METAOBJECT const QMetaObject QuickPainterPath::staticMetaObject = { {
    QMetaObject::SuperData::link<QObject::staticMetaObject>(),
    qt_meta_stringdata_QuickPainterPath.data,
    qt_meta_data_QuickPainterPath,
    qt_static_metacall,
    nullptr,
    nullptr
} };


const QMetaObject *QuickPainterPath::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *QuickPainterPath::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_meta_stringdata_QuickPainterPath.stringdata0))
        return static_cast<void*>(this);
    return QObject::qt_metacast(_clname);
}

int QuickPainterPath::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QObject::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 9)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 9;
    } else if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 9)
            *reinterpret_cast<int*>(_a[0]) = -1;
        _id -= 9;
    }
#ifndef QT_NO_PROPERTIES
    else if (_c == QMetaObject::ReadProperty || _c == QMetaObject::WriteProperty
            || _c == QMetaObject::ResetProperty || _c == QMetaObject::RegisterPropertyMetaType) {
        qt_static_metacall(this, _c, _id, _a);
        _id -= 4;
    } else if (_c == QMetaObject::QueryPropertyDesignable) {
        _id -= 4;
    } else if (_c == QMetaObject::QueryPropertyScriptable) {
        _id -= 4;
    } else if (_c == QMetaObject::QueryPropertyStored) {
        _id -= 4;
    } else if (_c == QMetaObject::QueryPropertyEditable) {
        _id -= 4;
    } else if (_c == QMetaObject::QueryPropertyUser) {
        _id -= 4;
    }
#endif // QT_NO_PROPERTIES
    return _id;
}

// SIGNAL 0
void QuickPainterPath::pathChanged(const QPainterPath & _t1)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))) };
    QMetaObject::activate(this, &staticMetaObject, 0, _a);
}

// SIGNAL 1
void QuickPainterPath::startPointChanged(const QPointF & _t1)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))) };
    QMetaObject::activate(this, &staticMetaObject, 1, _a);
}

// SIGNAL 2
void QuickPainterPath::endPointChanged(const QPointF & _t1)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))) };
    QMetaObject::activate(this, &staticMetaObject, 2, _a);
}

// SIGNAL 3
void QuickPainterPath::isEmptyChanged(bool _t1)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))) };
    QMetaObject::activate(this, &staticMetaObject, 3, _a);
}
struct qt_meta_stringdata_QuickPainterPathGeometryItem_t {
    QByteArrayData data[5];
    char stringdata0[60];
};
#define QT_MOC_LITERAL(idx, ofs, len) \
    Q_STATIC_BYTE_ARRAY_DATA_HEADER_INITIALIZER_WITH_OFFSET(len, \
    qptrdiff(offsetof(qt_meta_stringdata_QuickPainterPathGeometryItem_t, stringdata0) + ofs \
        - idx * sizeof(QByteArrayData)) \
    )
static const qt_meta_stringdata_QuickPainterPathGeometryItem_t qt_meta_stringdata_QuickPainterPathGeometryItem = {
    {
QT_MOC_LITERAL(0, 0, 28), // "QuickPainterPathGeometryItem"
QT_MOC_LITERAL(1, 29, 11), // "pathChanged"
QT_MOC_LITERAL(2, 41, 0), // ""
QT_MOC_LITERAL(3, 42, 12), // "QPainterPath"
QT_MOC_LITERAL(4, 55, 4) // "path"

    },
    "QuickPainterPathGeometryItem\0pathChanged\0"
    "\0QPainterPath\0path"
};
#undef QT_MOC_LITERAL

static const uint qt_meta_data_QuickPainterPathGeometryItem[] = {

 // content:
       8,       // revision
       0,       // classname
       0,    0, // classinfo
       1,   14, // methods
       1,   22, // properties
       0,    0, // enums/sets
       0,    0, // constructors
       0,       // flags
       1,       // signalCount

 // signals: name, argc, parameters, tag, flags
       1,    1,   19,    2, 0x06 /* Public */,

 // signals: parameters
    QMetaType::Void, 0x80000000 | 3,    4,

 // properties: name, type, flags
       4, 0x80000000 | 3, 0x0049510b,

 // properties: notify_signal_id
       0,

       0        // eod
};

void QuickPainterPathGeometryItem::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    if (_c == QMetaObject::InvokeMetaMethod) {
        auto *_t = static_cast<QuickPainterPathGeometryItem *>(_o);
        (void)_t;
        switch (_id) {
        case 0: _t->pathChanged((*reinterpret_cast< const QPainterPath(*)>(_a[1]))); break;
        default: ;
        }
    } else if (_c == QMetaObject::IndexOfMethod) {
        int *result = reinterpret_cast<int *>(_a[0]);
        {
            using _t = void (QuickPainterPathGeometryItem::*)(const QPainterPath & );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&QuickPainterPathGeometryItem::pathChanged)) {
                *result = 0;
                return;
            }
        }
    }
#ifndef QT_NO_PROPERTIES
    else if (_c == QMetaObject::ReadProperty) {
        auto *_t = static_cast<QuickPainterPathGeometryItem *>(_o);
        (void)_t;
        void *_v = _a[0];
        switch (_id) {
        case 0: *reinterpret_cast< QPainterPath*>(_v) = _t->path(); break;
        default: break;
        }
    } else if (_c == QMetaObject::WriteProperty) {
        auto *_t = static_cast<QuickPainterPathGeometryItem *>(_o);
        (void)_t;
        void *_v = _a[0];
        switch (_id) {
        case 0: _t->setPath(*reinterpret_cast< QPainterPath*>(_v)); break;
        default: break;
        }
    } else if (_c == QMetaObject::ResetProperty) {
    }
#endif // QT_NO_PROPERTIES
}

QT_INIT_METAOBJECT const QMetaObject QuickPainterPathGeometryItem::staticMetaObject = { {
    QMetaObject::SuperData::link<QuickGeometryItem::staticMetaObject>(),
    qt_meta_stringdata_QuickPainterPathGeometryItem.data,
    qt_meta_data_QuickPainterPathGeometryItem,
    qt_static_metacall,
    nullptr,
    nullptr
} };


const QMetaObject *QuickPainterPathGeometryItem::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *QuickPainterPathGeometryItem::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_meta_stringdata_QuickPainterPathGeometryItem.stringdata0))
        return static_cast<void*>(this);
    return QuickGeometryItem::qt_metacast(_clname);
}

int QuickPainterPathGeometryItem::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QuickGeometryItem::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 1)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 1;
    } else if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 1)
            *reinterpret_cast<int*>(_a[0]) = -1;
        _id -= 1;
    }
#ifndef QT_NO_PROPERTIES
    else if (_c == QMetaObject::ReadProperty || _c == QMetaObject::WriteProperty
            || _c == QMetaObject::ResetProperty || _c == QMetaObject::RegisterPropertyMetaType) {
        qt_static_metacall(this, _c, _id, _a);
        _id -= 1;
    } else if (_c == QMetaObject::QueryPropertyDesignable) {
        _id -= 1;
    } else if (_c == QMetaObject::QueryPropertyScriptable) {
        _id -= 1;
    } else if (_c == QMetaObject::QueryPropertyStored) {
        _id -= 1;
    } else if (_c == QMetaObject::QueryPropertyEditable) {
        _id -= 1;
    } else if (_c == QMetaObject::QueryPropertyUser) {
        _id -= 1;
    }
#endif // QT_NO_PROPERTIES
    return _id;
}

// SIGNAL 0
void QuickPainterPathGeometryItem::pathChanged(const QPainterPath & _t1)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))) };
    QMetaObject::activate(this, &staticMetaObject, 0, _a);
}
QT_WARNING_POP
QT_END_MOC_NAMESPACE
