/****************************************************************************
** Meta object code from reading C++ file 'quickprimitiveitem_p.h'
**
** Created by: The Qt Meta Object Compiler version 67 (Qt 5.15.19)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include <memory>
#include "../../../../../KDStateMachineEditor-1.2.8/src/view/quick/quickprimitiveitem_p.h"
#include <QtCore/qbytearray.h>
#include <QtCore/qmetatype.h>
#include <QtCore/QList>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'quickprimitiveitem_p.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 67
#error "This file was generated using the moc from 5.15.19. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

QT_BEGIN_MOC_NAMESPACE
QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
struct qt_meta_stringdata_QuickGeometryItem_t {
    QByteArrayData data[27];
    char stringdata0[346];
};
#define QT_MOC_LITERAL(idx, ofs, len) \
    Q_STATIC_BYTE_ARRAY_DATA_HEADER_INITIALIZER_WITH_OFFSET(len, \
    qptrdiff(offsetof(qt_meta_stringdata_QuickGeometryItem_t, stringdata0) + ofs \
        - idx * sizeof(QByteArrayData)) \
    )
static const qt_meta_stringdata_QuickGeometryItem_t qt_meta_stringdata_QuickGeometryItem = {
    {
QT_MOC_LITERAL(0, 0, 17), // "QuickGeometryItem"
QT_MOC_LITERAL(1, 18, 17), // "vertexDataChanged"
QT_MOC_LITERAL(2, 36, 0), // ""
QT_MOC_LITERAL(3, 37, 12), // "QList<qreal>"
QT_MOC_LITERAL(4, 50, 10), // "vertexData"
QT_MOC_LITERAL(5, 61, 21), // "vertexDataTypeChanged"
QT_MOC_LITERAL(6, 83, 14), // "VertexDataType"
QT_MOC_LITERAL(7, 98, 4), // "type"
QT_MOC_LITERAL(8, 103, 18), // "drawingModeChanged"
QT_MOC_LITERAL(9, 122, 6), // "GLenum"
QT_MOC_LITERAL(10, 129, 11), // "drawingMode"
QT_MOC_LITERAL(11, 141, 16), // "lineWidthChanged"
QT_MOC_LITERAL(12, 158, 9), // "lineWidth"
QT_MOC_LITERAL(13, 168, 7), // "changed"
QT_MOC_LITERAL(14, 176, 4), // "data"
QT_MOC_LITERAL(15, 181, 33), // "QuickGeometryItem::VertexData..."
QT_MOC_LITERAL(16, 215, 11), // "Point2DType"
QT_MOC_LITERAL(17, 227, 18), // "ColoredPoint2DType"
QT_MOC_LITERAL(18, 246, 19), // "TexturedPoint2DType"
QT_MOC_LITERAL(19, 266, 11), // "DrawingMode"
QT_MOC_LITERAL(20, 278, 6), // "Points"
QT_MOC_LITERAL(21, 285, 5), // "Lines"
QT_MOC_LITERAL(22, 291, 8), // "LineLoop"
QT_MOC_LITERAL(23, 300, 9), // "LineStrip"
QT_MOC_LITERAL(24, 310, 9), // "Triangles"
QT_MOC_LITERAL(25, 320, 13), // "TriangleStrip"
QT_MOC_LITERAL(26, 334, 11) // "TriangleFan"

    },
    "QuickGeometryItem\0vertexDataChanged\0"
    "\0QList<qreal>\0vertexData\0vertexDataTypeChanged\0"
    "VertexDataType\0type\0drawingModeChanged\0"
    "GLenum\0drawingMode\0lineWidthChanged\0"
    "lineWidth\0changed\0data\0"
    "QuickGeometryItem::VertexDataType\0"
    "Point2DType\0ColoredPoint2DType\0"
    "TexturedPoint2DType\0DrawingMode\0Points\0"
    "Lines\0LineLoop\0LineStrip\0Triangles\0"
    "TriangleStrip\0TriangleFan"
};
#undef QT_MOC_LITERAL

static const uint qt_meta_data_QuickGeometryItem[] = {

 // content:
       8,       // revision
       0,       // classname
       0,    0, // classinfo
       5,   14, // methods
       4,   52, // properties
       2,   68, // enums/sets
       0,    0, // constructors
       0,       // flags
       5,       // signalCount

 // signals: name, argc, parameters, tag, flags
       1,    1,   39,    2, 0x06 /* Public */,
       5,    1,   42,    2, 0x06 /* Public */,
       8,    1,   45,    2, 0x06 /* Public */,
      11,    1,   48,    2, 0x06 /* Public */,
      13,    0,   51,    2, 0x06 /* Public */,

 // signals: parameters
    QMetaType::Void, 0x80000000 | 3,    4,
    QMetaType::Void, 0x80000000 | 6,    7,
    QMetaType::Void, 0x80000000 | 9,   10,
    QMetaType::Void, QMetaType::Float,   12,
    QMetaType::Void,

 // properties: name, type, flags
      14, 0x80000000 | 3, 0x0049500b,
       7, 0x80000000 | 15, 0x0049500b,
      12, QMetaType::Float, 0x00495103,
      10, QMetaType::UInt, 0x00495103,

 // properties: notify_signal_id
       0,
       1,
       3,
       2,

 // enums: name, alias, flags, count, data
       6,    6, 0x0,    3,   78,
      19,   19, 0x0,    7,   84,

 // enum data: key, value
      16, uint(QuickGeometryItem::Point2DType),
      17, uint(QuickGeometryItem::ColoredPoint2DType),
      18, uint(QuickGeometryItem::TexturedPoint2DType),
      20, uint(QuickGeometryItem::Points),
      21, uint(QuickGeometryItem::Lines),
      22, uint(QuickGeometryItem::LineLoop),
      23, uint(QuickGeometryItem::LineStrip),
      24, uint(QuickGeometryItem::Triangles),
      25, uint(QuickGeometryItem::TriangleStrip),
      26, uint(QuickGeometryItem::TriangleFan),

       0        // eod
};

void QuickGeometryItem::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    if (_c == QMetaObject::InvokeMetaMethod) {
        auto *_t = static_cast<QuickGeometryItem *>(_o);
        (void)_t;
        switch (_id) {
        case 0: _t->vertexDataChanged((*reinterpret_cast< const QList<qreal>(*)>(_a[1]))); break;
        case 1: _t->vertexDataTypeChanged((*reinterpret_cast< VertexDataType(*)>(_a[1]))); break;
        case 2: _t->drawingModeChanged((*reinterpret_cast< GLenum(*)>(_a[1]))); break;
        case 3: _t->lineWidthChanged((*reinterpret_cast< float(*)>(_a[1]))); break;
        case 4: _t->changed(); break;
        default: ;
        }
    } else if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        switch (_id) {
        default: *reinterpret_cast<int*>(_a[0]) = -1; break;
        case 0:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<int*>(_a[0]) = -1; break;
            case 0:
                *reinterpret_cast<int*>(_a[0]) = qRegisterMetaType< QList<qreal> >(); break;
            }
            break;
        }
    } else if (_c == QMetaObject::IndexOfMethod) {
        int *result = reinterpret_cast<int *>(_a[0]);
        {
            using _t = void (QuickGeometryItem::*)(const QList<qreal> & );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&QuickGeometryItem::vertexDataChanged)) {
                *result = 0;
                return;
            }
        }
        {
            using _t = void (QuickGeometryItem::*)(VertexDataType );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&QuickGeometryItem::vertexDataTypeChanged)) {
                *result = 1;
                return;
            }
        }
        {
            using _t = void (QuickGeometryItem::*)(GLenum );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&QuickGeometryItem::drawingModeChanged)) {
                *result = 2;
                return;
            }
        }
        {
            using _t = void (QuickGeometryItem::*)(float );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&QuickGeometryItem::lineWidthChanged)) {
                *result = 3;
                return;
            }
        }
        {
            using _t = void (QuickGeometryItem::*)();
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&QuickGeometryItem::changed)) {
                *result = 4;
                return;
            }
        }
    } else if (_c == QMetaObject::RegisterPropertyMetaType) {
        switch (_id) {
        default: *reinterpret_cast<int*>(_a[0]) = -1; break;
        case 0:
            *reinterpret_cast<int*>(_a[0]) = qRegisterMetaType< QList<qreal> >(); break;
        case 1:
            *reinterpret_cast<int*>(_a[0]) = qRegisterMetaType< QuickGeometryItem::VertexDataType >(); break;
        }
    }

#ifndef QT_NO_PROPERTIES
    else if (_c == QMetaObject::ReadProperty) {
        auto *_t = static_cast<QuickGeometryItem *>(_o);
        (void)_t;
        void *_v = _a[0];
        switch (_id) {
        case 0: *reinterpret_cast< QList<qreal>*>(_v) = _t->vertexData(); break;
        case 1: *reinterpret_cast< QuickGeometryItem::VertexDataType*>(_v) = _t->vertexDataType(); break;
        case 2: *reinterpret_cast< float*>(_v) = _t->lineWidth(); break;
        case 3: *reinterpret_cast< uint*>(_v) = _t->drawingMode(); break;
        default: break;
        }
    } else if (_c == QMetaObject::WriteProperty) {
        auto *_t = static_cast<QuickGeometryItem *>(_o);
        (void)_t;
        void *_v = _a[0];
        switch (_id) {
        case 0: _t->setVertexData(*reinterpret_cast< QList<qreal>*>(_v)); break;
        case 1: _t->setVertexDataType(*reinterpret_cast< QuickGeometryItem::VertexDataType*>(_v)); break;
        case 2: _t->setLineWidth(*reinterpret_cast< float*>(_v)); break;
        case 3: _t->setDrawingMode(*reinterpret_cast< uint*>(_v)); break;
        default: break;
        }
    } else if (_c == QMetaObject::ResetProperty) {
    }
#endif // QT_NO_PROPERTIES
}

QT_INIT_METAOBJECT const QMetaObject QuickGeometryItem::staticMetaObject = { {
    QMetaObject::SuperData::link<QObject::staticMetaObject>(),
    qt_meta_stringdata_QuickGeometryItem.data,
    qt_meta_data_QuickGeometryItem,
    qt_static_metacall,
    nullptr,
    nullptr
} };


const QMetaObject *QuickGeometryItem::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *QuickGeometryItem::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_meta_stringdata_QuickGeometryItem.stringdata0))
        return static_cast<void*>(this);
    return QObject::qt_metacast(_clname);
}

int QuickGeometryItem::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QObject::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 5)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 5;
    } else if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 5)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 5;
    }
#ifndef QT_NO_PROPERTIES
    else if (_c == QMetaObject::ReadProperty || _c == QMetaObject::WriteProperty
            || _c == QMetaObject::ResetProperty || _c == QMetaObject::RegisterPropertyMetaType) {
        qt_static_metacall(this, _c, _id, _a);
        _id -= 4;
    } else if (_c == QMetaObject::QueryPropertyDesignable) {
        _id -= 4;
    } else if (_c == QMetaObject::QueryPropertyScriptable) {
        _id -= 4;
    } else if (_c == QMetaObject::QueryPropertyStored) {
        _id -= 4;
    } else if (_c == QMetaObject::QueryPropertyEditable) {
        _id -= 4;
    } else if (_c == QMetaObject::QueryPropertyUser) {
        _id -= 4;
    }
#endif // QT_NO_PROPERTIES
    return _id;
}

// SIGNAL 0
void QuickGeometryItem::vertexDataChanged(const QList<qreal> & _t1)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))) };
    QMetaObject::activate(this, &staticMetaObject, 0, _a);
}

// SIGNAL 1
void QuickGeometryItem::vertexDataTypeChanged(VertexDataType _t1)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))) };
    QMetaObject::activate(this, &staticMetaObject, 1, _a);
}

// SIGNAL 2
void QuickGeometryItem::drawingModeChanged(GLenum _t1)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))) };
    QMetaObject::activate(this, &staticMetaObject, 2, _a);
}

// SIGNAL 3
void QuickGeometryItem::lineWidthChanged(float _t1)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))) };
    QMetaObject::activate(this, &staticMetaObject, 3, _a);
}

// SIGNAL 4
void QuickGeometryItem::changed()
{
    QMetaObject::activate(this, &staticMetaObject, 4, nullptr);
}
struct qt_meta_stringdata_QuickPrimitiveItem_t {
    QByteArrayData data[8];
    char stringdata0[102];
};
#define QT_MOC_LITERAL(idx, ofs, len) \
    Q_STATIC_BYTE_ARRAY_DATA_HEADER_INITIALIZER_WITH_OFFSET(len, \
    qptrdiff(offsetof(qt_meta_stringdata_QuickPrimitiveItem_t, stringdata0) + ofs \
        - idx * sizeof(QByteArrayData)) \
    )
static const qt_meta_stringdata_QuickPrimitiveItem_t qt_meta_stringdata_QuickPrimitiveItem = {
    {
QT_MOC_LITERAL(0, 0, 18), // "QuickPrimitiveItem"
QT_MOC_LITERAL(1, 19, 19), // "geometryItemChanged"
QT_MOC_LITERAL(2, 39, 0), // ""
QT_MOC_LITERAL(3, 40, 18), // "QuickGeometryItem*"
QT_MOC_LITERAL(4, 59, 8), // "geometry"
QT_MOC_LITERAL(5, 68, 12), // "colorChanged"
QT_MOC_LITERAL(6, 81, 5), // "color"
QT_MOC_LITERAL(7, 87, 14) // "updateGeometry"

    },
    "QuickPrimitiveItem\0geometryItemChanged\0"
    "\0QuickGeometryItem*\0geometry\0colorChanged\0"
    "color\0updateGeometry"
};
#undef QT_MOC_LITERAL

static const uint qt_meta_data_QuickPrimitiveItem[] = {

 // content:
       8,       // revision
       0,       // classname
       0,    0, // classinfo
       3,   14, // methods
       2,   36, // properties
       0,    0, // enums/sets
       0,    0, // constructors
       0,       // flags
       2,       // signalCount

 // signals: name, argc, parameters, tag, flags
       1,    1,   29,    2, 0x06 /* Public */,
       5,    1,   32,    2, 0x06 /* Public */,

 // slots: name, argc, parameters, tag, flags
       7,    0,   35,    2, 0x08 /* Private */,

 // signals: parameters
    QMetaType::Void, 0x80000000 | 3,    4,
    QMetaType::Void, QMetaType::QColor,    6,

 // slots: parameters
    QMetaType::Void,

 // properties: name, type, flags
       4, 0x80000000 | 3, 0x0049500b,
       6, QMetaType::QColor, 0x00495103,

 // properties: notify_signal_id
       0,
       1,

       0        // eod
};

void QuickPrimitiveItem::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    if (_c == QMetaObject::InvokeMetaMethod) {
        auto *_t = static_cast<QuickPrimitiveItem *>(_o);
        (void)_t;
        switch (_id) {
        case 0: _t->geometryItemChanged((*reinterpret_cast< QuickGeometryItem*(*)>(_a[1]))); break;
        case 1: _t->colorChanged((*reinterpret_cast< const QColor(*)>(_a[1]))); break;
        case 2: _t->updateGeometry(); break;
        default: ;
        }
    } else if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        switch (_id) {
        default: *reinterpret_cast<int*>(_a[0]) = -1; break;
        case 0:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<int*>(_a[0]) = -1; break;
            case 0:
                *reinterpret_cast<int*>(_a[0]) = qRegisterMetaType< QuickGeometryItem* >(); break;
            }
            break;
        }
    } else if (_c == QMetaObject::IndexOfMethod) {
        int *result = reinterpret_cast<int *>(_a[0]);
        {
            using _t = void (QuickPrimitiveItem::*)(QuickGeometryItem * );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&QuickPrimitiveItem::geometryItemChanged)) {
                *result = 0;
                return;
            }
        }
        {
            using _t = void (QuickPrimitiveItem::*)(const QColor & );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&QuickPrimitiveItem::colorChanged)) {
                *result = 1;
                return;
            }
        }
    } else if (_c == QMetaObject::RegisterPropertyMetaType) {
        switch (_id) {
        default: *reinterpret_cast<int*>(_a[0]) = -1; break;
        case 0:
            *reinterpret_cast<int*>(_a[0]) = qRegisterMetaType< QuickGeometryItem* >(); break;
        }
    }

#ifndef QT_NO_PROPERTIES
    else if (_c == QMetaObject::ReadProperty) {
        auto *_t = static_cast<QuickPrimitiveItem *>(_o);
        (void)_t;
        void *_v = _a[0];
        switch (_id) {
        case 0: *reinterpret_cast< QuickGeometryItem**>(_v) = _t->geometryItem(); break;
        case 1: *reinterpret_cast< QColor*>(_v) = _t->color(); break;
        default: break;
        }
    } else if (_c == QMetaObject::WriteProperty) {
        auto *_t = static_cast<QuickPrimitiveItem *>(_o);
        (void)_t;
        void *_v = _a[0];
        switch (_id) {
        case 0: _t->setGeometryItem(*reinterpret_cast< QuickGeometryItem**>(_v)); break;
        case 1: _t->setColor(*reinterpret_cast< QColor*>(_v)); break;
        default: break;
        }
    } else if (_c == QMetaObject::ResetProperty) {
    }
#endif // QT_NO_PROPERTIES
}

QT_INIT_METAOBJECT const QMetaObject QuickPrimitiveItem::staticMetaObject = { {
    QMetaObject::SuperData::link<QQuickItem::staticMetaObject>(),
    qt_meta_stringdata_QuickPrimitiveItem.data,
    qt_meta_data_QuickPrimitiveItem,
    qt_static_metacall,
    nullptr,
    nullptr
} };


const QMetaObject *QuickPrimitiveItem::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *QuickPrimitiveItem::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_meta_stringdata_QuickPrimitiveItem.stringdata0))
        return static_cast<void*>(this);
    return QQuickItem::qt_metacast(_clname);
}

int QuickPrimitiveItem::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QQuickItem::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 3)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 3;
    } else if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 3)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 3;
    }
#ifndef QT_NO_PROPERTIES
    else if (_c == QMetaObject::ReadProperty || _c == QMetaObject::WriteProperty
            || _c == QMetaObject::ResetProperty || _c == QMetaObject::RegisterPropertyMetaType) {
        qt_static_metacall(this, _c, _id, _a);
        _id -= 2;
    } else if (_c == QMetaObject::QueryPropertyDesignable) {
        _id -= 2;
    } else if (_c == QMetaObject::QueryPropertyScriptable) {
        _id -= 2;
    } else if (_c == QMetaObject::QueryPropertyStored) {
        _id -= 2;
    } else if (_c == QMetaObject::QueryPropertyEditable) {
        _id -= 2;
    } else if (_c == QMetaObject::QueryPropertyUser) {
        _id -= 2;
    }
#endif // QT_NO_PROPERTIES
    return _id;
}

// SIGNAL 0
void QuickPrimitiveItem::geometryItemChanged(QuickGeometryItem * _t1)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))) };
    QMetaObject::activate(this, &staticMetaObject, 0, _a);
}

// SIGNAL 1
void QuickPrimitiveItem::colorChanged(const QColor & _t1)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))) };
    QMetaObject::activate(this, &staticMetaObject, 1, _a);
}
QT_WARNING_POP
QT_END_MOC_NAMESPACE
