/****************************************************************************
** Meta object code from reading C++ file 'statemachinescene.h'
**
** Created by: The Qt Meta Object Compiler version 67 (Qt 5.15.19)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include <memory>
#include "../../../../../KDStateMachineEditor-1.2.8/src/view/statemachinescene.h"
#include <QtCore/qbytearray.h>
#include <QtCore/qmetatype.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'statemachinescene.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 67
#error "This file was generated using the moc from 5.15.19. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

QT_BEGIN_MOC_NAMESPACE
QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
struct qt_meta_stringdata_KDSME__StateMachineScene_t {
    QByteArrayData data[39];
    char stringdata0[495];
};
#define QT_MOC_LITERAL(idx, ofs, len) \
    Q_STATIC_BYTE_ARRAY_DATA_HEADER_INITIALIZER_WITH_OFFSET(len, \
    qptrdiff(offsetof(qt_meta_stringdata_KDSME__StateMachineScene_t, stringdata0) + ofs \
        - idx * sizeof(QByteArrayData)) \
    )
static const qt_meta_stringdata_KDSME__StateMachineScene_t qt_meta_stringdata_KDSME__StateMachineScene = {
    {
QT_MOC_LITERAL(0, 0, 24), // "KDSME::StateMachineScene"
QT_MOC_LITERAL(1, 25, 19), // "stateMachineChanged"
QT_MOC_LITERAL(2, 45, 0), // ""
QT_MOC_LITERAL(3, 46, 20), // "KDSME::StateMachine*"
QT_MOC_LITERAL(4, 67, 12), // "stateMachine"
QT_MOC_LITERAL(5, 80, 16), // "rootStateChanged"
QT_MOC_LITERAL(6, 97, 13), // "KDSME::State*"
QT_MOC_LITERAL(7, 111, 5), // "state"
QT_MOC_LITERAL(8, 117, 18), // "currentItemChanged"
QT_MOC_LITERAL(9, 136, 15), // "KDSME::Element*"
QT_MOC_LITERAL(10, 152, 11), // "currentItem"
QT_MOC_LITERAL(11, 164, 11), // "zoomChanged"
QT_MOC_LITERAL(12, 176, 4), // "zoom"
QT_MOC_LITERAL(13, 181, 19), // "maximumDepthChanged"
QT_MOC_LITERAL(14, 201, 5), // "depth"
QT_MOC_LITERAL(15, 207, 6), // "layout"
QT_MOC_LITERAL(16, 214, 14), // "currentChanged"
QT_MOC_LITERAL(17, 229, 11), // "QModelIndex"
QT_MOC_LITERAL(18, 241, 7), // "current"
QT_MOC_LITERAL(19, 249, 8), // "previous"
QT_MOC_LITERAL(20, 258, 20), // "rowsAboutToBeRemoved"
QT_MOC_LITERAL(21, 279, 6), // "parent"
QT_MOC_LITERAL(22, 286, 5), // "start"
QT_MOC_LITERAL(23, 292, 3), // "end"
QT_MOC_LITERAL(24, 296, 12), // "rowsInserted"
QT_MOC_LITERAL(25, 309, 13), // "layoutChanged"
QT_MOC_LITERAL(26, 323, 12), // "collapseItem"
QT_MOC_LITERAL(27, 336, 10), // "expandItem"
QT_MOC_LITERAL(28, 347, 14), // "isItemExpanded"
QT_MOC_LITERAL(29, 362, 15), // "setItemExpanded"
QT_MOC_LITERAL(30, 378, 6), // "expand"
QT_MOC_LITERAL(31, 385, 14), // "isItemSelected"
QT_MOC_LITERAL(32, 400, 4), // "item"
QT_MOC_LITERAL(33, 405, 15), // "setItemSelected"
QT_MOC_LITERAL(34, 421, 8), // "selected"
QT_MOC_LITERAL(35, 430, 9), // "rootState"
QT_MOC_LITERAL(36, 440, 16), // "layoutProperties"
QT_MOC_LITERAL(37, 457, 24), // "KDSME::LayoutProperties*"
QT_MOC_LITERAL(38, 482, 12) // "maximumDepth"

    },
    "KDSME::StateMachineScene\0stateMachineChanged\0"
    "\0KDSME::StateMachine*\0stateMachine\0"
    "rootStateChanged\0KDSME::State*\0state\0"
    "currentItemChanged\0KDSME::Element*\0"
    "currentItem\0zoomChanged\0zoom\0"
    "maximumDepthChanged\0depth\0layout\0"
    "currentChanged\0QModelIndex\0current\0"
    "previous\0rowsAboutToBeRemoved\0parent\0"
    "start\0end\0rowsInserted\0layoutChanged\0"
    "collapseItem\0expandItem\0isItemExpanded\0"
    "setItemExpanded\0expand\0isItemSelected\0"
    "item\0setItemSelected\0selected\0rootState\0"
    "layoutProperties\0KDSME::LayoutProperties*\0"
    "maximumDepth"
};
#undef QT_MOC_LITERAL

static const uint qt_meta_data_KDSME__StateMachineScene[] = {

 // content:
       8,       // revision
       0,       // classname
       0,    0, // classinfo
      16,   14, // methods
       5,  152, // properties
       0,    0, // enums/sets
       0,    0, // constructors
       0,       // flags
       5,       // signalCount

 // signals: name, argc, parameters, tag, flags
       1,    1,   94,    2, 0x06 /* Public */,
       5,    1,   97,    2, 0x06 /* Public */,
       8,    1,  100,    2, 0x06 /* Public */,
      11,    1,  103,    2, 0x06 /* Public */,
      13,    1,  106,    2, 0x06 /* Public */,

 // slots: name, argc, parameters, tag, flags
      15,    0,  109,    2, 0x0a /* Public */,
      16,    2,  110,    2, 0x09 /* Protected */,
      20,    3,  115,    2, 0x09 /* Protected */,
      24,    3,  122,    2, 0x09 /* Protected */,
      25,    0,  129,    2, 0x09 /* Protected */,

 // methods: name, argc, parameters, tag, flags
      26,    1,  130,    2, 0x02 /* Public */,
      27,    1,  133,    2, 0x02 /* Public */,
      28,    1,  136,    2, 0x02 /* Public */,
      29,    2,  139,    2, 0x02 /* Public */,
      31,    1,  144,    2, 0x02 /* Public */,
      33,    2,  147,    2, 0x02 /* Public */,

 // signals: parameters
    QMetaType::Void, 0x80000000 | 3,    4,
    QMetaType::Void, 0x80000000 | 6,    7,
    QMetaType::Void, 0x80000000 | 9,   10,
    QMetaType::Void, QMetaType::QReal,   12,
    QMetaType::Void, QMetaType::Int,   14,

 // slots: parameters
    QMetaType::Void,
    QMetaType::Void, 0x80000000 | 17, 0x80000000 | 17,   18,   19,
    QMetaType::Void, 0x80000000 | 17, QMetaType::Int, QMetaType::Int,   21,   22,   23,
    QMetaType::Void, 0x80000000 | 17, QMetaType::Int, QMetaType::Int,   21,   22,   23,
    QMetaType::Void,

 // methods: parameters
    QMetaType::Void, 0x80000000 | 6,    7,
    QMetaType::Void, 0x80000000 | 6,    7,
    QMetaType::Bool, 0x80000000 | 6,    7,
    QMetaType::Void, 0x80000000 | 6, QMetaType::Bool,    7,   30,
    QMetaType::Bool, 0x80000000 | 9,   32,
    QMetaType::Void, 0x80000000 | 9, QMetaType::Bool,   32,   34,

 // properties: name, type, flags
      35, 0x80000000 | 6, 0x0049510b,
      10, 0x80000000 | 9, 0x0049510b,
      36, 0x80000000 | 37, 0x00095409,
      12, QMetaType::QReal, 0x00495903,
      38, QMetaType::Int, 0x00495903,

 // properties: notify_signal_id
       1,
       2,
       0,
       3,
       4,

       0        // eod
};

void KDSME::StateMachineScene::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    if (_c == QMetaObject::InvokeMetaMethod) {
        auto *_t = static_cast<StateMachineScene *>(_o);
        (void)_t;
        switch (_id) {
        case 0: _t->stateMachineChanged((*reinterpret_cast< KDSME::StateMachine*(*)>(_a[1]))); break;
        case 1: _t->rootStateChanged((*reinterpret_cast< KDSME::State*(*)>(_a[1]))); break;
        case 2: _t->currentItemChanged((*reinterpret_cast< KDSME::Element*(*)>(_a[1]))); break;
        case 3: _t->zoomChanged((*reinterpret_cast< qreal(*)>(_a[1]))); break;
        case 4: _t->maximumDepthChanged((*reinterpret_cast< int(*)>(_a[1]))); break;
        case 5: _t->layout(); break;
        case 6: _t->currentChanged((*reinterpret_cast< const QModelIndex(*)>(_a[1])),(*reinterpret_cast< const QModelIndex(*)>(_a[2]))); break;
        case 7: _t->rowsAboutToBeRemoved((*reinterpret_cast< const QModelIndex(*)>(_a[1])),(*reinterpret_cast< int(*)>(_a[2])),(*reinterpret_cast< int(*)>(_a[3]))); break;
        case 8: _t->rowsInserted((*reinterpret_cast< const QModelIndex(*)>(_a[1])),(*reinterpret_cast< int(*)>(_a[2])),(*reinterpret_cast< int(*)>(_a[3]))); break;
        case 9: _t->layoutChanged(); break;
        case 10: _t->collapseItem((*reinterpret_cast< KDSME::State*(*)>(_a[1]))); break;
        case 11: _t->expandItem((*reinterpret_cast< KDSME::State*(*)>(_a[1]))); break;
        case 12: { bool _r = _t->isItemExpanded((*reinterpret_cast< KDSME::State*(*)>(_a[1])));
            if (_a[0]) *reinterpret_cast< bool*>(_a[0]) = std::move(_r); }  break;
        case 13: _t->setItemExpanded((*reinterpret_cast< KDSME::State*(*)>(_a[1])),(*reinterpret_cast< bool(*)>(_a[2]))); break;
        case 14: { bool _r = _t->isItemSelected((*reinterpret_cast< KDSME::Element*(*)>(_a[1])));
            if (_a[0]) *reinterpret_cast< bool*>(_a[0]) = std::move(_r); }  break;
        case 15: _t->setItemSelected((*reinterpret_cast< KDSME::Element*(*)>(_a[1])),(*reinterpret_cast< bool(*)>(_a[2]))); break;
        default: ;
        }
    } else if (_c == QMetaObject::IndexOfMethod) {
        int *result = reinterpret_cast<int *>(_a[0]);
        {
            using _t = void (StateMachineScene::*)(KDSME::StateMachine * );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&StateMachineScene::stateMachineChanged)) {
                *result = 0;
                return;
            }
        }
        {
            using _t = void (StateMachineScene::*)(KDSME::State * );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&StateMachineScene::rootStateChanged)) {
                *result = 1;
                return;
            }
        }
        {
            using _t = void (StateMachineScene::*)(KDSME::Element * );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&StateMachineScene::currentItemChanged)) {
                *result = 2;
                return;
            }
        }
        {
            using _t = void (StateMachineScene::*)(qreal );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&StateMachineScene::zoomChanged)) {
                *result = 3;
                return;
            }
        }
        {
            using _t = void (StateMachineScene::*)(int );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&StateMachineScene::maximumDepthChanged)) {
                *result = 4;
                return;
            }
        }
    }
#ifndef QT_NO_PROPERTIES
    else if (_c == QMetaObject::ReadProperty) {
        auto *_t = static_cast<StateMachineScene *>(_o);
        (void)_t;
        void *_v = _a[0];
        switch (_id) {
        case 0: *reinterpret_cast< KDSME::State**>(_v) = _t->rootState(); break;
        case 1: *reinterpret_cast< KDSME::Element**>(_v) = _t->currentItem(); break;
        case 2: *reinterpret_cast< KDSME::LayoutProperties**>(_v) = _t->layoutProperties(); break;
        case 3: *reinterpret_cast< qreal*>(_v) = _t->zoom(); break;
        case 4: *reinterpret_cast< int*>(_v) = _t->maximumDepth(); break;
        default: break;
        }
    } else if (_c == QMetaObject::WriteProperty) {
        auto *_t = static_cast<StateMachineScene *>(_o);
        (void)_t;
        void *_v = _a[0];
        switch (_id) {
        case 0: _t->setRootState(*reinterpret_cast< KDSME::State**>(_v)); break;
        case 1: _t->setCurrentItem(*reinterpret_cast< KDSME::Element**>(_v)); break;
        case 3: _t->setZoom(*reinterpret_cast< qreal*>(_v)); break;
        case 4: _t->setMaximumDepth(*reinterpret_cast< int*>(_v)); break;
        default: break;
        }
    } else if (_c == QMetaObject::ResetProperty) {
    }
#endif // QT_NO_PROPERTIES
}

QT_INIT_METAOBJECT const QMetaObject KDSME::StateMachineScene::staticMetaObject = { {
    QMetaObject::SuperData::link<AbstractScene::staticMetaObject>(),
    qt_meta_stringdata_KDSME__StateMachineScene.data,
    qt_meta_data_KDSME__StateMachineScene,
    qt_static_metacall,
    nullptr,
    nullptr
} };


const QMetaObject *KDSME::StateMachineScene::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *KDSME::StateMachineScene::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_meta_stringdata_KDSME__StateMachineScene.stringdata0))
        return static_cast<void*>(this);
    return AbstractScene::qt_metacast(_clname);
}

int KDSME::StateMachineScene::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = AbstractScene::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 16)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 16;
    } else if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 16)
            *reinterpret_cast<int*>(_a[0]) = -1;
        _id -= 16;
    }
#ifndef QT_NO_PROPERTIES
    else if (_c == QMetaObject::ReadProperty || _c == QMetaObject::WriteProperty
            || _c == QMetaObject::ResetProperty || _c == QMetaObject::RegisterPropertyMetaType) {
        qt_static_metacall(this, _c, _id, _a);
        _id -= 5;
    } else if (_c == QMetaObject::QueryPropertyDesignable) {
        _id -= 5;
    } else if (_c == QMetaObject::QueryPropertyScriptable) {
        _id -= 5;
    } else if (_c == QMetaObject::QueryPropertyStored) {
        _id -= 5;
    } else if (_c == QMetaObject::QueryPropertyEditable) {
        _id -= 5;
    } else if (_c == QMetaObject::QueryPropertyUser) {
        _id -= 5;
    }
#endif // QT_NO_PROPERTIES
    return _id;
}

// SIGNAL 0
void KDSME::StateMachineScene::stateMachineChanged(KDSME::StateMachine * _t1)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))) };
    QMetaObject::activate(this, &staticMetaObject, 0, _a);
}

// SIGNAL 1
void KDSME::StateMachineScene::rootStateChanged(KDSME::State * _t1)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))) };
    QMetaObject::activate(this, &staticMetaObject, 1, _a);
}

// SIGNAL 2
void KDSME::StateMachineScene::currentItemChanged(KDSME::Element * _t1)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))) };
    QMetaObject::activate(this, &staticMetaObject, 2, _a);
}

// SIGNAL 3
void KDSME::StateMachineScene::zoomChanged(qreal _t1)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))) };
    QMetaObject::activate(this, &staticMetaObject, 3, _a);
}

// SIGNAL 4
void KDSME::StateMachineScene::maximumDepthChanged(int _t1)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))) };
    QMetaObject::activate(this, &staticMetaObject, 4, _a);
}
QT_WARNING_POP
QT_END_MOC_NAMESPACE
