/****************************************************************************
** Meta object code from reading C++ file 'state.h'
**
** Created by: The Qt Meta Object Compiler version 67 (Qt 5.15.19)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include <memory>
#include "../../../../../KDStateMachineEditor-1.2.8/src/core/model/state.h"
#include <QtCore/qbytearray.h>
#include <QtCore/qmetatype.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'state.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 67
#error "This file was generated using the moc from 5.15.19. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

QT_BEGIN_MOC_NAMESPACE
QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
struct qt_meta_stringdata_KDSME__State_t {
    QByteArrayData data[19];
    char stringdata0[237];
};
#define QT_MOC_LITERAL(idx, ofs, len) \
    Q_STATIC_BYTE_ARRAY_DATA_HEADER_INITIALIZER_WITH_OFFSET(len, \
    qptrdiff(offsetof(qt_meta_stringdata_KDSME__State_t, stringdata0) + ofs \
        - idx * sizeof(QByteArrayData)) \
    )
static const qt_meta_stringdata_KDSME__State_t qt_meta_stringdata_KDSME__State = {
    {
QT_MOC_LITERAL(0, 0, 12), // "KDSME::State"
QT_MOC_LITERAL(1, 13, 14), // "onEntryChanged"
QT_MOC_LITERAL(2, 28, 0), // ""
QT_MOC_LITERAL(3, 29, 7), // "onEntry"
QT_MOC_LITERAL(4, 37, 13), // "onExitChanged"
QT_MOC_LITERAL(5, 51, 6), // "onExit"
QT_MOC_LITERAL(6, 58, 16), // "childModeChanged"
QT_MOC_LITERAL(7, 75, 9), // "ChildMode"
QT_MOC_LITERAL(8, 85, 9), // "childMode"
QT_MOC_LITERAL(9, 95, 18), // "isCompositeChanged"
QT_MOC_LITERAL(10, 114, 11), // "isComposite"
QT_MOC_LITERAL(11, 126, 15), // "expandedChanged"
QT_MOC_LITERAL(12, 142, 8), // "expanded"
QT_MOC_LITERAL(13, 151, 11), // "parentState"
QT_MOC_LITERAL(14, 163, 13), // "KDSME::State*"
QT_MOC_LITERAL(15, 177, 7), // "machine"
QT_MOC_LITERAL(16, 185, 20), // "KDSME::StateMachine*"
QT_MOC_LITERAL(17, 206, 15), // "ExclusiveStates"
QT_MOC_LITERAL(18, 222, 14) // "ParallelStates"

    },
    "KDSME::State\0onEntryChanged\0\0onEntry\0"
    "onExitChanged\0onExit\0childModeChanged\0"
    "ChildMode\0childMode\0isCompositeChanged\0"
    "isComposite\0expandedChanged\0expanded\0"
    "parentState\0KDSME::State*\0machine\0"
    "KDSME::StateMachine*\0ExclusiveStates\0"
    "ParallelStates"
};
#undef QT_MOC_LITERAL

static const uint qt_meta_data_KDSME__State[] = {

 // content:
       8,       // revision
       0,       // classname
       0,    0, // classinfo
       7,   14, // methods
       5,   66, // properties
       1,   86, // enums/sets
       0,    0, // constructors
       0,       // flags
       5,       // signalCount

 // signals: name, argc, parameters, tag, flags
       1,    1,   49,    2, 0x06 /* Public */,
       4,    1,   52,    2, 0x06 /* Public */,
       6,    1,   55,    2, 0x06 /* Public */,
       9,    1,   58,    2, 0x06 /* Public */,
      11,    1,   61,    2, 0x06 /* Public */,

 // methods: name, argc, parameters, tag, flags
      13,    0,   64,    2, 0x02 /* Public */,
      15,    0,   65,    2, 0x02 /* Public */,

 // signals: parameters
    QMetaType::Void, QMetaType::QString,    3,
    QMetaType::Void, QMetaType::QString,    5,
    QMetaType::Void, 0x80000000 | 7,    8,
    QMetaType::Void, QMetaType::Bool,   10,
    QMetaType::Void, QMetaType::Bool,   12,

 // methods: parameters
    0x80000000 | 14,
    0x80000000 | 16,

 // properties: name, type, flags
       3, QMetaType::QString, 0x00495903,
       5, QMetaType::QString, 0x00495903,
       8, 0x80000000 | 7, 0x0049590b,
      10, QMetaType::Bool, 0x00495801,
      12, QMetaType::Bool, 0x00495903,

 // properties: notify_signal_id
       0,
       1,
       2,
       3,
       4,

 // enums: name, alias, flags, count, data
       7,    7, 0x0,    2,   91,

 // enum data: key, value
      17, uint(KDSME::State::ExclusiveStates),
      18, uint(KDSME::State::ParallelStates),

       0        // eod
};

void KDSME::State::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    if (_c == QMetaObject::InvokeMetaMethod) {
        auto *_t = static_cast<State *>(_o);
        (void)_t;
        switch (_id) {
        case 0: _t->onEntryChanged((*reinterpret_cast< const QString(*)>(_a[1]))); break;
        case 1: _t->onExitChanged((*reinterpret_cast< const QString(*)>(_a[1]))); break;
        case 2: _t->childModeChanged((*reinterpret_cast< ChildMode(*)>(_a[1]))); break;
        case 3: _t->isCompositeChanged((*reinterpret_cast< bool(*)>(_a[1]))); break;
        case 4: _t->expandedChanged((*reinterpret_cast< bool(*)>(_a[1]))); break;
        case 5: { KDSME::State* _r = _t->parentState();
            if (_a[0]) *reinterpret_cast< KDSME::State**>(_a[0]) = std::move(_r); }  break;
        case 6: { KDSME::StateMachine* _r = _t->machine();
            if (_a[0]) *reinterpret_cast< KDSME::StateMachine**>(_a[0]) = std::move(_r); }  break;
        default: ;
        }
    } else if (_c == QMetaObject::IndexOfMethod) {
        int *result = reinterpret_cast<int *>(_a[0]);
        {
            using _t = void (State::*)(const QString & );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&State::onEntryChanged)) {
                *result = 0;
                return;
            }
        }
        {
            using _t = void (State::*)(const QString & );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&State::onExitChanged)) {
                *result = 1;
                return;
            }
        }
        {
            using _t = void (State::*)(ChildMode );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&State::childModeChanged)) {
                *result = 2;
                return;
            }
        }
        {
            using _t = void (State::*)(bool );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&State::isCompositeChanged)) {
                *result = 3;
                return;
            }
        }
        {
            using _t = void (State::*)(bool );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&State::expandedChanged)) {
                *result = 4;
                return;
            }
        }
    }
#ifndef QT_NO_PROPERTIES
    else if (_c == QMetaObject::ReadProperty) {
        auto *_t = static_cast<State *>(_o);
        (void)_t;
        void *_v = _a[0];
        switch (_id) {
        case 0: *reinterpret_cast< QString*>(_v) = _t->onEntry(); break;
        case 1: *reinterpret_cast< QString*>(_v) = _t->onExit(); break;
        case 2: *reinterpret_cast< ChildMode*>(_v) = _t->childMode(); break;
        case 3: *reinterpret_cast< bool*>(_v) = _t->isComposite(); break;
        case 4: *reinterpret_cast< bool*>(_v) = _t->isExpanded(); break;
        default: break;
        }
    } else if (_c == QMetaObject::WriteProperty) {
        auto *_t = static_cast<State *>(_o);
        (void)_t;
        void *_v = _a[0];
        switch (_id) {
        case 0: _t->setOnEntry(*reinterpret_cast< QString*>(_v)); break;
        case 1: _t->setOnExit(*reinterpret_cast< QString*>(_v)); break;
        case 2: _t->setChildMode(*reinterpret_cast< ChildMode*>(_v)); break;
        case 4: _t->setExpanded(*reinterpret_cast< bool*>(_v)); break;
        default: break;
        }
    } else if (_c == QMetaObject::ResetProperty) {
    }
#endif // QT_NO_PROPERTIES
}

QT_INIT_METAOBJECT const QMetaObject KDSME::State::staticMetaObject = { {
    QMetaObject::SuperData::link<Element::staticMetaObject>(),
    qt_meta_stringdata_KDSME__State.data,
    qt_meta_data_KDSME__State,
    qt_static_metacall,
    nullptr,
    nullptr
} };


const QMetaObject *KDSME::State::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *KDSME::State::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_meta_stringdata_KDSME__State.stringdata0))
        return static_cast<void*>(this);
    return Element::qt_metacast(_clname);
}

int KDSME::State::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = Element::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 7)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 7;
    } else if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 7)
            *reinterpret_cast<int*>(_a[0]) = -1;
        _id -= 7;
    }
#ifndef QT_NO_PROPERTIES
    else if (_c == QMetaObject::ReadProperty || _c == QMetaObject::WriteProperty
            || _c == QMetaObject::ResetProperty || _c == QMetaObject::RegisterPropertyMetaType) {
        qt_static_metacall(this, _c, _id, _a);
        _id -= 5;
    } else if (_c == QMetaObject::QueryPropertyDesignable) {
        _id -= 5;
    } else if (_c == QMetaObject::QueryPropertyScriptable) {
        _id -= 5;
    } else if (_c == QMetaObject::QueryPropertyStored) {
        _id -= 5;
    } else if (_c == QMetaObject::QueryPropertyEditable) {
        _id -= 5;
    } else if (_c == QMetaObject::QueryPropertyUser) {
        _id -= 5;
    }
#endif // QT_NO_PROPERTIES
    return _id;
}

// SIGNAL 0
void KDSME::State::onEntryChanged(const QString & _t1)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))) };
    QMetaObject::activate(this, &staticMetaObject, 0, _a);
}

// SIGNAL 1
void KDSME::State::onExitChanged(const QString & _t1)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))) };
    QMetaObject::activate(this, &staticMetaObject, 1, _a);
}

// SIGNAL 2
void KDSME::State::childModeChanged(ChildMode _t1)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))) };
    QMetaObject::activate(this, &staticMetaObject, 2, _a);
}

// SIGNAL 3
void KDSME::State::isCompositeChanged(bool _t1)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))) };
    QMetaObject::activate(this, &staticMetaObject, 3, _a);
}

// SIGNAL 4
void KDSME::State::expandedChanged(bool _t1)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))) };
    QMetaObject::activate(this, &staticMetaObject, 4, _a);
}
struct qt_meta_stringdata_KDSME__HistoryState_t {
    QByteArrayData data[12];
    char stringdata0[151];
};
#define QT_MOC_LITERAL(idx, ofs, len) \
    Q_STATIC_BYTE_ARRAY_DATA_HEADER_INITIALIZER_WITH_OFFSET(len, \
    qptrdiff(offsetof(qt_meta_stringdata_KDSME__HistoryState_t, stringdata0) + ofs \
        - idx * sizeof(QByteArrayData)) \
    )
static const qt_meta_stringdata_KDSME__HistoryState_t qt_meta_stringdata_KDSME__HistoryState = {
    {
QT_MOC_LITERAL(0, 0, 19), // "KDSME::HistoryState"
QT_MOC_LITERAL(1, 20, 19), // "defaultStateChanged"
QT_MOC_LITERAL(2, 40, 0), // ""
QT_MOC_LITERAL(3, 41, 6), // "State*"
QT_MOC_LITERAL(4, 48, 5), // "state"
QT_MOC_LITERAL(5, 54, 18), // "historyTypeChanged"
QT_MOC_LITERAL(6, 73, 12), // "defaultState"
QT_MOC_LITERAL(7, 86, 13), // "KDSME::State*"
QT_MOC_LITERAL(8, 100, 11), // "historyType"
QT_MOC_LITERAL(9, 112, 11), // "HistoryType"
QT_MOC_LITERAL(10, 124, 14), // "ShallowHistory"
QT_MOC_LITERAL(11, 139, 11) // "DeepHistory"

    },
    "KDSME::HistoryState\0defaultStateChanged\0"
    "\0State*\0state\0historyTypeChanged\0"
    "defaultState\0KDSME::State*\0historyType\0"
    "HistoryType\0ShallowHistory\0DeepHistory"
};
#undef QT_MOC_LITERAL

static const uint qt_meta_data_KDSME__HistoryState[] = {

 // content:
       8,       // revision
       0,       // classname
       0,    0, // classinfo
       2,   14, // methods
       2,   28, // properties
       1,   36, // enums/sets
       0,    0, // constructors
       0,       // flags
       2,       // signalCount

 // signals: name, argc, parameters, tag, flags
       1,    1,   24,    2, 0x06 /* Public */,
       5,    0,   27,    2, 0x06 /* Public */,

 // signals: parameters
    QMetaType::Void, 0x80000000 | 3,    4,
    QMetaType::Void,

 // properties: name, type, flags
       6, 0x80000000 | 7, 0x0049590b,
       8, 0x80000000 | 9, 0x0049590b,

 // properties: notify_signal_id
       0,
       1,

 // enums: name, alias, flags, count, data
       9,    9, 0x0,    2,   41,

 // enum data: key, value
      10, uint(KDSME::HistoryState::ShallowHistory),
      11, uint(KDSME::HistoryState::DeepHistory),

       0        // eod
};

void KDSME::HistoryState::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    if (_c == QMetaObject::InvokeMetaMethod) {
        auto *_t = static_cast<HistoryState *>(_o);
        (void)_t;
        switch (_id) {
        case 0: _t->defaultStateChanged((*reinterpret_cast< State*(*)>(_a[1]))); break;
        case 1: _t->historyTypeChanged(); break;
        default: ;
        }
    } else if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        switch (_id) {
        default: *reinterpret_cast<int*>(_a[0]) = -1; break;
        case 0:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<int*>(_a[0]) = -1; break;
            case 0:
                *reinterpret_cast<int*>(_a[0]) = qRegisterMetaType< State* >(); break;
            }
            break;
        }
    } else if (_c == QMetaObject::IndexOfMethod) {
        int *result = reinterpret_cast<int *>(_a[0]);
        {
            using _t = void (HistoryState::*)(State * );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&HistoryState::defaultStateChanged)) {
                *result = 0;
                return;
            }
        }
        {
            using _t = void (HistoryState::*)();
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&HistoryState::historyTypeChanged)) {
                *result = 1;
                return;
            }
        }
    } else if (_c == QMetaObject::RegisterPropertyMetaType) {
        switch (_id) {
        default: *reinterpret_cast<int*>(_a[0]) = -1; break;
        case 0:
            *reinterpret_cast<int*>(_a[0]) = qRegisterMetaType< KDSME::State* >(); break;
        }
    }

#ifndef QT_NO_PROPERTIES
    else if (_c == QMetaObject::ReadProperty) {
        auto *_t = static_cast<HistoryState *>(_o);
        (void)_t;
        void *_v = _a[0];
        switch (_id) {
        case 0: *reinterpret_cast< KDSME::State**>(_v) = _t->defaultState(); break;
        case 1: *reinterpret_cast< HistoryType*>(_v) = _t->historyType(); break;
        default: break;
        }
    } else if (_c == QMetaObject::WriteProperty) {
        auto *_t = static_cast<HistoryState *>(_o);
        (void)_t;
        void *_v = _a[0];
        switch (_id) {
        case 0: _t->setDefaultState(*reinterpret_cast< KDSME::State**>(_v)); break;
        case 1: _t->setHistoryType(*reinterpret_cast< HistoryType*>(_v)); break;
        default: break;
        }
    } else if (_c == QMetaObject::ResetProperty) {
    }
#endif // QT_NO_PROPERTIES
}

QT_INIT_METAOBJECT const QMetaObject KDSME::HistoryState::staticMetaObject = { {
    QMetaObject::SuperData::link<State::staticMetaObject>(),
    qt_meta_stringdata_KDSME__HistoryState.data,
    qt_meta_data_KDSME__HistoryState,
    qt_static_metacall,
    nullptr,
    nullptr
} };


const QMetaObject *KDSME::HistoryState::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *KDSME::HistoryState::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_meta_stringdata_KDSME__HistoryState.stringdata0))
        return static_cast<void*>(this);
    return State::qt_metacast(_clname);
}

int KDSME::HistoryState::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = State::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 2)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 2;
    } else if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 2)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 2;
    }
#ifndef QT_NO_PROPERTIES
    else if (_c == QMetaObject::ReadProperty || _c == QMetaObject::WriteProperty
            || _c == QMetaObject::ResetProperty || _c == QMetaObject::RegisterPropertyMetaType) {
        qt_static_metacall(this, _c, _id, _a);
        _id -= 2;
    } else if (_c == QMetaObject::QueryPropertyDesignable) {
        _id -= 2;
    } else if (_c == QMetaObject::QueryPropertyScriptable) {
        _id -= 2;
    } else if (_c == QMetaObject::QueryPropertyStored) {
        _id -= 2;
    } else if (_c == QMetaObject::QueryPropertyEditable) {
        _id -= 2;
    } else if (_c == QMetaObject::QueryPropertyUser) {
        _id -= 2;
    }
#endif // QT_NO_PROPERTIES
    return _id;
}

// SIGNAL 0
void KDSME::HistoryState::defaultStateChanged(State * _t1)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))) };
    QMetaObject::activate(this, &staticMetaObject, 0, _a);
}

// SIGNAL 1
void KDSME::HistoryState::historyTypeChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 1, nullptr);
}
struct qt_meta_stringdata_KDSME__FinalState_t {
    QByteArrayData data[1];
    char stringdata0[18];
};
#define QT_MOC_LITERAL(idx, ofs, len) \
    Q_STATIC_BYTE_ARRAY_DATA_HEADER_INITIALIZER_WITH_OFFSET(len, \
    qptrdiff(offsetof(qt_meta_stringdata_KDSME__FinalState_t, stringdata0) + ofs \
        - idx * sizeof(QByteArrayData)) \
    )
static const qt_meta_stringdata_KDSME__FinalState_t qt_meta_stringdata_KDSME__FinalState = {
    {
QT_MOC_LITERAL(0, 0, 17) // "KDSME::FinalState"

    },
    "KDSME::FinalState"
};
#undef QT_MOC_LITERAL

static const uint qt_meta_data_KDSME__FinalState[] = {

 // content:
       8,       // revision
       0,       // classname
       0,    0, // classinfo
       0,    0, // methods
       0,    0, // properties
       0,    0, // enums/sets
       0,    0, // constructors
       0,       // flags
       0,       // signalCount

       0        // eod
};

void KDSME::FinalState::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    (void)_o;
    (void)_id;
    (void)_c;
    (void)_a;
}

QT_INIT_METAOBJECT const QMetaObject KDSME::FinalState::staticMetaObject = { {
    QMetaObject::SuperData::link<State::staticMetaObject>(),
    qt_meta_stringdata_KDSME__FinalState.data,
    qt_meta_data_KDSME__FinalState,
    qt_static_metacall,
    nullptr,
    nullptr
} };


const QMetaObject *KDSME::FinalState::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *KDSME::FinalState::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_meta_stringdata_KDSME__FinalState.stringdata0))
        return static_cast<void*>(this);
    return State::qt_metacast(_clname);
}

int KDSME::FinalState::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = State::qt_metacall(_c, _id, _a);
    return _id;
}
struct qt_meta_stringdata_KDSME__StateMachine_t {
    QByteArrayData data[6];
    char stringdata0[109];
};
#define QT_MOC_LITERAL(idx, ofs, len) \
    Q_STATIC_BYTE_ARRAY_DATA_HEADER_INITIALIZER_WITH_OFFSET(len, \
    qptrdiff(offsetof(qt_meta_stringdata_KDSME__StateMachine_t, stringdata0) + ofs \
        - idx * sizeof(QByteArrayData)) \
    )
static const qt_meta_stringdata_KDSME__StateMachine_t qt_meta_stringdata_KDSME__StateMachine = {
    {
QT_MOC_LITERAL(0, 0, 19), // "KDSME::StateMachine"
QT_MOC_LITERAL(1, 20, 24), // "runtimeControllerChanged"
QT_MOC_LITERAL(2, 45, 0), // ""
QT_MOC_LITERAL(3, 46, 18), // "RuntimeController*"
QT_MOC_LITERAL(4, 65, 17), // "runtimeController"
QT_MOC_LITERAL(5, 83, 25) // "KDSME::RuntimeController*"

    },
    "KDSME::StateMachine\0runtimeControllerChanged\0"
    "\0RuntimeController*\0runtimeController\0"
    "KDSME::RuntimeController*"
};
#undef QT_MOC_LITERAL

static const uint qt_meta_data_KDSME__StateMachine[] = {

 // content:
       8,       // revision
       0,       // classname
       0,    0, // classinfo
       1,   14, // methods
       1,   22, // properties
       0,    0, // enums/sets
       0,    0, // constructors
       0,       // flags
       1,       // signalCount

 // signals: name, argc, parameters, tag, flags
       1,    1,   19,    2, 0x06 /* Public */,

 // signals: parameters
    QMetaType::Void, 0x80000000 | 3,    4,

 // properties: name, type, flags
       4, 0x80000000 | 5, 0x0049510b,

 // properties: notify_signal_id
       0,

       0        // eod
};

void KDSME::StateMachine::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    if (_c == QMetaObject::InvokeMetaMethod) {
        auto *_t = static_cast<StateMachine *>(_o);
        (void)_t;
        switch (_id) {
        case 0: _t->runtimeControllerChanged((*reinterpret_cast< RuntimeController*(*)>(_a[1]))); break;
        default: ;
        }
    } else if (_c == QMetaObject::IndexOfMethod) {
        int *result = reinterpret_cast<int *>(_a[0]);
        {
            using _t = void (StateMachine::*)(RuntimeController * );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&StateMachine::runtimeControllerChanged)) {
                *result = 0;
                return;
            }
        }
    }
#ifndef QT_NO_PROPERTIES
    else if (_c == QMetaObject::ReadProperty) {
        auto *_t = static_cast<StateMachine *>(_o);
        (void)_t;
        void *_v = _a[0];
        switch (_id) {
        case 0: *reinterpret_cast< KDSME::RuntimeController**>(_v) = _t->runtimeController(); break;
        default: break;
        }
    } else if (_c == QMetaObject::WriteProperty) {
        auto *_t = static_cast<StateMachine *>(_o);
        (void)_t;
        void *_v = _a[0];
        switch (_id) {
        case 0: _t->setRuntimeController(*reinterpret_cast< KDSME::RuntimeController**>(_v)); break;
        default: break;
        }
    } else if (_c == QMetaObject::ResetProperty) {
    }
#endif // QT_NO_PROPERTIES
}

QT_INIT_METAOBJECT const QMetaObject KDSME::StateMachine::staticMetaObject = { {
    QMetaObject::SuperData::link<State::staticMetaObject>(),
    qt_meta_stringdata_KDSME__StateMachine.data,
    qt_meta_data_KDSME__StateMachine,
    qt_static_metacall,
    nullptr,
    nullptr
} };


const QMetaObject *KDSME::StateMachine::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *KDSME::StateMachine::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_meta_stringdata_KDSME__StateMachine.stringdata0))
        return static_cast<void*>(this);
    return State::qt_metacast(_clname);
}

int KDSME::StateMachine::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = State::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 1)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 1;
    } else if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 1)
            *reinterpret_cast<int*>(_a[0]) = -1;
        _id -= 1;
    }
#ifndef QT_NO_PROPERTIES
    else if (_c == QMetaObject::ReadProperty || _c == QMetaObject::WriteProperty
            || _c == QMetaObject::ResetProperty || _c == QMetaObject::RegisterPropertyMetaType) {
        qt_static_metacall(this, _c, _id, _a);
        _id -= 1;
    } else if (_c == QMetaObject::QueryPropertyDesignable) {
        _id -= 1;
    } else if (_c == QMetaObject::QueryPropertyScriptable) {
        _id -= 1;
    } else if (_c == QMetaObject::QueryPropertyStored) {
        _id -= 1;
    } else if (_c == QMetaObject::QueryPropertyEditable) {
        _id -= 1;
    } else if (_c == QMetaObject::QueryPropertyUser) {
        _id -= 1;
    }
#endif // QT_NO_PROPERTIES
    return _id;
}

// SIGNAL 0
void KDSME::StateMachine::runtimeControllerChanged(RuntimeController * _t1)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))) };
    QMetaObject::activate(this, &staticMetaObject, 0, _a);
}
struct qt_meta_stringdata_KDSME__PseudoState_t {
    QByteArrayData data[6];
    char stringdata0[55];
};
#define QT_MOC_LITERAL(idx, ofs, len) \
    Q_STATIC_BYTE_ARRAY_DATA_HEADER_INITIALIZER_WITH_OFFSET(len, \
    qptrdiff(offsetof(qt_meta_stringdata_KDSME__PseudoState_t, stringdata0) + ofs \
        - idx * sizeof(QByteArrayData)) \
    )
static const qt_meta_stringdata_KDSME__PseudoState_t qt_meta_stringdata_KDSME__PseudoState = {
    {
QT_MOC_LITERAL(0, 0, 18), // "KDSME::PseudoState"
QT_MOC_LITERAL(1, 19, 11), // "kindChanged"
QT_MOC_LITERAL(2, 31, 0), // ""
QT_MOC_LITERAL(3, 32, 4), // "Kind"
QT_MOC_LITERAL(4, 37, 4), // "kind"
QT_MOC_LITERAL(5, 42, 12) // "InitialState"

    },
    "KDSME::PseudoState\0kindChanged\0\0Kind\0"
    "kind\0InitialState"
};
#undef QT_MOC_LITERAL

static const uint qt_meta_data_KDSME__PseudoState[] = {

 // content:
       8,       // revision
       0,       // classname
       0,    0, // classinfo
       1,   14, // methods
       1,   22, // properties
       1,   26, // enums/sets
       0,    0, // constructors
       0,       // flags
       1,       // signalCount

 // signals: name, argc, parameters, tag, flags
       1,    1,   19,    2, 0x06 /* Public */,

 // signals: parameters
    QMetaType::Void, 0x80000000 | 3,    4,

 // properties: name, type, flags
       4, 0x80000000 | 3, 0x0049590b,

 // properties: notify_signal_id
       0,

 // enums: name, alias, flags, count, data
       3,    3, 0x0,    1,   31,

 // enum data: key, value
       5, uint(KDSME::PseudoState::InitialState),

       0        // eod
};

void KDSME::PseudoState::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    if (_c == QMetaObject::InvokeMetaMethod) {
        auto *_t = static_cast<PseudoState *>(_o);
        (void)_t;
        switch (_id) {
        case 0: _t->kindChanged((*reinterpret_cast< Kind(*)>(_a[1]))); break;
        default: ;
        }
    } else if (_c == QMetaObject::IndexOfMethod) {
        int *result = reinterpret_cast<int *>(_a[0]);
        {
            using _t = void (PseudoState::*)(Kind );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&PseudoState::kindChanged)) {
                *result = 0;
                return;
            }
        }
    }
#ifndef QT_NO_PROPERTIES
    else if (_c == QMetaObject::ReadProperty) {
        auto *_t = static_cast<PseudoState *>(_o);
        (void)_t;
        void *_v = _a[0];
        switch (_id) {
        case 0: *reinterpret_cast< Kind*>(_v) = _t->kind(); break;
        default: break;
        }
    } else if (_c == QMetaObject::WriteProperty) {
        auto *_t = static_cast<PseudoState *>(_o);
        (void)_t;
        void *_v = _a[0];
        switch (_id) {
        case 0: _t->setKind(*reinterpret_cast< Kind*>(_v)); break;
        default: break;
        }
    } else if (_c == QMetaObject::ResetProperty) {
    }
#endif // QT_NO_PROPERTIES
}

QT_INIT_METAOBJECT const QMetaObject KDSME::PseudoState::staticMetaObject = { {
    QMetaObject::SuperData::link<State::staticMetaObject>(),
    qt_meta_stringdata_KDSME__PseudoState.data,
    qt_meta_data_KDSME__PseudoState,
    qt_static_metacall,
    nullptr,
    nullptr
} };


const QMetaObject *KDSME::PseudoState::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *KDSME::PseudoState::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_meta_stringdata_KDSME__PseudoState.stringdata0))
        return static_cast<void*>(this);
    return State::qt_metacast(_clname);
}

int KDSME::PseudoState::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = State::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 1)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 1;
    } else if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 1)
            *reinterpret_cast<int*>(_a[0]) = -1;
        _id -= 1;
    }
#ifndef QT_NO_PROPERTIES
    else if (_c == QMetaObject::ReadProperty || _c == QMetaObject::WriteProperty
            || _c == QMetaObject::ResetProperty || _c == QMetaObject::RegisterPropertyMetaType) {
        qt_static_metacall(this, _c, _id, _a);
        _id -= 1;
    } else if (_c == QMetaObject::QueryPropertyDesignable) {
        _id -= 1;
    } else if (_c == QMetaObject::QueryPropertyScriptable) {
        _id -= 1;
    } else if (_c == QMetaObject::QueryPropertyStored) {
        _id -= 1;
    } else if (_c == QMetaObject::QueryPropertyEditable) {
        _id -= 1;
    } else if (_c == QMetaObject::QueryPropertyUser) {
        _id -= 1;
    }
#endif // QT_NO_PROPERTIES
    return _id;
}

// SIGNAL 0
void KDSME::PseudoState::kindChanged(Kind _t1)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))) };
    QMetaObject::activate(this, &staticMetaObject, 0, _a);
}
QT_WARNING_POP
QT_END_MOC_NAMESPACE
