/****************************************************************************
** Meta object code from reading C++ file 'transition.h'
**
** Created by: The Qt Meta Object Compiler version 67 (Qt 5.15.19)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include <memory>
#include "../../../../../KDStateMachineEditor-1.2.8/src/core/model/transition.h"
#include <QtCore/qbytearray.h>
#include <QtCore/qmetatype.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'transition.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 67
#error "This file was generated using the moc from 5.15.19. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

QT_BEGIN_MOC_NAMESPACE
QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
struct qt_meta_stringdata_KDSME__Transition_t {
    QByteArrayData data[19];
    char stringdata0[235];
};
#define QT_MOC_LITERAL(idx, ofs, len) \
    Q_STATIC_BYTE_ARRAY_DATA_HEADER_INITIALIZER_WITH_OFFSET(len, \
    qptrdiff(offsetof(qt_meta_stringdata_KDSME__Transition_t, stringdata0) + ofs \
        - idx * sizeof(QByteArrayData)) \
    )
static const qt_meta_stringdata_KDSME__Transition_t qt_meta_stringdata_KDSME__Transition = {
    {
QT_MOC_LITERAL(0, 0, 17), // "KDSME::Transition"
QT_MOC_LITERAL(1, 18, 18), // "sourceStateChanged"
QT_MOC_LITERAL(2, 37, 0), // ""
QT_MOC_LITERAL(3, 38, 6), // "State*"
QT_MOC_LITERAL(4, 45, 11), // "sourceState"
QT_MOC_LITERAL(5, 57, 18), // "targetStateChanged"
QT_MOC_LITERAL(6, 76, 11), // "targetState"
QT_MOC_LITERAL(7, 88, 12), // "guardChanged"
QT_MOC_LITERAL(8, 101, 5), // "guard"
QT_MOC_LITERAL(9, 107, 12), // "shapeChanged"
QT_MOC_LITERAL(10, 120, 12), // "QPainterPath"
QT_MOC_LITERAL(11, 133, 4), // "path"
QT_MOC_LITERAL(12, 138, 24), // "labelBoundingRectChanged"
QT_MOC_LITERAL(13, 163, 4), // "rect"
QT_MOC_LITERAL(14, 168, 7), // "machine"
QT_MOC_LITERAL(15, 176, 20), // "KDSME::StateMachine*"
QT_MOC_LITERAL(16, 197, 13), // "KDSME::State*"
QT_MOC_LITERAL(17, 211, 5), // "shape"
QT_MOC_LITERAL(18, 217, 17) // "labelBoundingRect"

    },
    "KDSME::Transition\0sourceStateChanged\0"
    "\0State*\0sourceState\0targetStateChanged\0"
    "targetState\0guardChanged\0guard\0"
    "shapeChanged\0QPainterPath\0path\0"
    "labelBoundingRectChanged\0rect\0machine\0"
    "KDSME::StateMachine*\0KDSME::State*\0"
    "shape\0labelBoundingRect"
};
#undef QT_MOC_LITERAL

static const uint qt_meta_data_KDSME__Transition[] = {

 // content:
       8,       // revision
       0,       // classname
       0,    0, // classinfo
       6,   14, // methods
       5,   60, // properties
       0,    0, // enums/sets
       0,    0, // constructors
       0,       // flags
       5,       // signalCount

 // signals: name, argc, parameters, tag, flags
       1,    1,   44,    2, 0x06 /* Public */,
       5,    1,   47,    2, 0x06 /* Public */,
       7,    1,   50,    2, 0x06 /* Public */,
       9,    1,   53,    2, 0x06 /* Public */,
      12,    1,   56,    2, 0x06 /* Public */,

 // methods: name, argc, parameters, tag, flags
      14,    0,   59,    2, 0x02 /* Public */,

 // signals: parameters
    QMetaType::Void, 0x80000000 | 3,    4,
    QMetaType::Void, 0x80000000 | 3,    6,
    QMetaType::Void, QMetaType::QString,    8,
    QMetaType::Void, 0x80000000 | 10,   11,
    QMetaType::Void, QMetaType::QRectF,   13,

 // methods: parameters
    0x80000000 | 15,

 // properties: name, type, flags
       4, 0x80000000 | 16, 0x0049590b,
       6, 0x80000000 | 16, 0x0049590b,
       8, QMetaType::QString, 0x00495903,
      17, 0x80000000 | 10, 0x0049590b,
      18, QMetaType::QRectF, 0x00495903,

 // properties: notify_signal_id
       0,
       1,
       2,
       3,
       4,

       0        // eod
};

void KDSME::Transition::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    if (_c == QMetaObject::InvokeMetaMethod) {
        auto *_t = static_cast<Transition *>(_o);
        (void)_t;
        switch (_id) {
        case 0: _t->sourceStateChanged((*reinterpret_cast< State*(*)>(_a[1]))); break;
        case 1: _t->targetStateChanged((*reinterpret_cast< State*(*)>(_a[1]))); break;
        case 2: _t->guardChanged((*reinterpret_cast< const QString(*)>(_a[1]))); break;
        case 3: _t->shapeChanged((*reinterpret_cast< const QPainterPath(*)>(_a[1]))); break;
        case 4: _t->labelBoundingRectChanged((*reinterpret_cast< const QRectF(*)>(_a[1]))); break;
        case 5: { KDSME::StateMachine* _r = _t->machine();
            if (_a[0]) *reinterpret_cast< KDSME::StateMachine**>(_a[0]) = std::move(_r); }  break;
        default: ;
        }
    } else if (_c == QMetaObject::IndexOfMethod) {
        int *result = reinterpret_cast<int *>(_a[0]);
        {
            using _t = void (Transition::*)(State * );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&Transition::sourceStateChanged)) {
                *result = 0;
                return;
            }
        }
        {
            using _t = void (Transition::*)(State * );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&Transition::targetStateChanged)) {
                *result = 1;
                return;
            }
        }
        {
            using _t = void (Transition::*)(const QString & );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&Transition::guardChanged)) {
                *result = 2;
                return;
            }
        }
        {
            using _t = void (Transition::*)(const QPainterPath & );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&Transition::shapeChanged)) {
                *result = 3;
                return;
            }
        }
        {
            using _t = void (Transition::*)(const QRectF & );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&Transition::labelBoundingRectChanged)) {
                *result = 4;
                return;
            }
        }
    }
#ifndef QT_NO_PROPERTIES
    else if (_c == QMetaObject::ReadProperty) {
        auto *_t = static_cast<Transition *>(_o);
        (void)_t;
        void *_v = _a[0];
        switch (_id) {
        case 0: *reinterpret_cast< KDSME::State**>(_v) = _t->sourceState(); break;
        case 1: *reinterpret_cast< KDSME::State**>(_v) = _t->targetState(); break;
        case 2: *reinterpret_cast< QString*>(_v) = _t->guard(); break;
        case 3: *reinterpret_cast< QPainterPath*>(_v) = _t->shape(); break;
        case 4: *reinterpret_cast< QRectF*>(_v) = _t->labelBoundingRect(); break;
        default: break;
        }
    } else if (_c == QMetaObject::WriteProperty) {
        auto *_t = static_cast<Transition *>(_o);
        (void)_t;
        void *_v = _a[0];
        switch (_id) {
        case 0: _t->setSourceState(*reinterpret_cast< KDSME::State**>(_v)); break;
        case 1: _t->setTargetState(*reinterpret_cast< KDSME::State**>(_v)); break;
        case 2: _t->setGuard(*reinterpret_cast< QString*>(_v)); break;
        case 3: _t->setShape(*reinterpret_cast< QPainterPath*>(_v)); break;
        case 4: _t->setLabelBoundingRect(*reinterpret_cast< QRectF*>(_v)); break;
        default: break;
        }
    } else if (_c == QMetaObject::ResetProperty) {
    }
#endif // QT_NO_PROPERTIES
}

QT_INIT_METAOBJECT const QMetaObject KDSME::Transition::staticMetaObject = { {
    QMetaObject::SuperData::link<Element::staticMetaObject>(),
    qt_meta_stringdata_KDSME__Transition.data,
    qt_meta_data_KDSME__Transition,
    qt_static_metacall,
    nullptr,
    nullptr
} };


const QMetaObject *KDSME::Transition::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *KDSME::Transition::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_meta_stringdata_KDSME__Transition.stringdata0))
        return static_cast<void*>(this);
    return Element::qt_metacast(_clname);
}

int KDSME::Transition::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = Element::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 6)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 6;
    } else if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 6)
            *reinterpret_cast<int*>(_a[0]) = -1;
        _id -= 6;
    }
#ifndef QT_NO_PROPERTIES
    else if (_c == QMetaObject::ReadProperty || _c == QMetaObject::WriteProperty
            || _c == QMetaObject::ResetProperty || _c == QMetaObject::RegisterPropertyMetaType) {
        qt_static_metacall(this, _c, _id, _a);
        _id -= 5;
    } else if (_c == QMetaObject::QueryPropertyDesignable) {
        _id -= 5;
    } else if (_c == QMetaObject::QueryPropertyScriptable) {
        _id -= 5;
    } else if (_c == QMetaObject::QueryPropertyStored) {
        _id -= 5;
    } else if (_c == QMetaObject::QueryPropertyEditable) {
        _id -= 5;
    } else if (_c == QMetaObject::QueryPropertyUser) {
        _id -= 5;
    }
#endif // QT_NO_PROPERTIES
    return _id;
}

// SIGNAL 0
void KDSME::Transition::sourceStateChanged(State * _t1)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))) };
    QMetaObject::activate(this, &staticMetaObject, 0, _a);
}

// SIGNAL 1
void KDSME::Transition::targetStateChanged(State * _t1)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))) };
    QMetaObject::activate(this, &staticMetaObject, 1, _a);
}

// SIGNAL 2
void KDSME::Transition::guardChanged(const QString & _t1)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))) };
    QMetaObject::activate(this, &staticMetaObject, 2, _a);
}

// SIGNAL 3
void KDSME::Transition::shapeChanged(const QPainterPath & _t1)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))) };
    QMetaObject::activate(this, &staticMetaObject, 3, _a);
}

// SIGNAL 4
void KDSME::Transition::labelBoundingRectChanged(const QRectF & _t1)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))) };
    QMetaObject::activate(this, &staticMetaObject, 4, _a);
}
struct qt_meta_stringdata_KDSME__SignalTransition_t {
    QByteArrayData data[4];
    char stringdata0[46];
};
#define QT_MOC_LITERAL(idx, ofs, len) \
    Q_STATIC_BYTE_ARRAY_DATA_HEADER_INITIALIZER_WITH_OFFSET(len, \
    qptrdiff(offsetof(qt_meta_stringdata_KDSME__SignalTransition_t, stringdata0) + ofs \
        - idx * sizeof(QByteArrayData)) \
    )
static const qt_meta_stringdata_KDSME__SignalTransition_t qt_meta_stringdata_KDSME__SignalTransition = {
    {
QT_MOC_LITERAL(0, 0, 23), // "KDSME::SignalTransition"
QT_MOC_LITERAL(1, 24, 13), // "signalChanged"
QT_MOC_LITERAL(2, 38, 0), // ""
QT_MOC_LITERAL(3, 39, 6) // "signal"

    },
    "KDSME::SignalTransition\0signalChanged\0"
    "\0signal"
};
#undef QT_MOC_LITERAL

static const uint qt_meta_data_KDSME__SignalTransition[] = {

 // content:
       8,       // revision
       0,       // classname
       0,    0, // classinfo
       1,   14, // methods
       1,   22, // properties
       0,    0, // enums/sets
       0,    0, // constructors
       0,       // flags
       1,       // signalCount

 // signals: name, argc, parameters, tag, flags
       1,    1,   19,    2, 0x06 /* Public */,

 // signals: parameters
    QMetaType::Void, QMetaType::QString,    3,

 // properties: name, type, flags
       3, QMetaType::QString, 0x00495103,

 // properties: notify_signal_id
       0,

       0        // eod
};

void KDSME::SignalTransition::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    if (_c == QMetaObject::InvokeMetaMethod) {
        auto *_t = static_cast<SignalTransition *>(_o);
        (void)_t;
        switch (_id) {
        case 0: _t->signalChanged((*reinterpret_cast< const QString(*)>(_a[1]))); break;
        default: ;
        }
    } else if (_c == QMetaObject::IndexOfMethod) {
        int *result = reinterpret_cast<int *>(_a[0]);
        {
            using _t = void (SignalTransition::*)(const QString & );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&SignalTransition::signalChanged)) {
                *result = 0;
                return;
            }
        }
    }
#ifndef QT_NO_PROPERTIES
    else if (_c == QMetaObject::ReadProperty) {
        auto *_t = static_cast<SignalTransition *>(_o);
        (void)_t;
        void *_v = _a[0];
        switch (_id) {
        case 0: *reinterpret_cast< QString*>(_v) = _t->signal(); break;
        default: break;
        }
    } else if (_c == QMetaObject::WriteProperty) {
        auto *_t = static_cast<SignalTransition *>(_o);
        (void)_t;
        void *_v = _a[0];
        switch (_id) {
        case 0: _t->setSignal(*reinterpret_cast< QString*>(_v)); break;
        default: break;
        }
    } else if (_c == QMetaObject::ResetProperty) {
    }
#endif // QT_NO_PROPERTIES
}

QT_INIT_METAOBJECT const QMetaObject KDSME::SignalTransition::staticMetaObject = { {
    QMetaObject::SuperData::link<Transition::staticMetaObject>(),
    qt_meta_stringdata_KDSME__SignalTransition.data,
    qt_meta_data_KDSME__SignalTransition,
    qt_static_metacall,
    nullptr,
    nullptr
} };


const QMetaObject *KDSME::SignalTransition::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *KDSME::SignalTransition::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_meta_stringdata_KDSME__SignalTransition.stringdata0))
        return static_cast<void*>(this);
    return Transition::qt_metacast(_clname);
}

int KDSME::SignalTransition::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = Transition::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 1)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 1;
    } else if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 1)
            *reinterpret_cast<int*>(_a[0]) = -1;
        _id -= 1;
    }
#ifndef QT_NO_PROPERTIES
    else if (_c == QMetaObject::ReadProperty || _c == QMetaObject::WriteProperty
            || _c == QMetaObject::ResetProperty || _c == QMetaObject::RegisterPropertyMetaType) {
        qt_static_metacall(this, _c, _id, _a);
        _id -= 1;
    } else if (_c == QMetaObject::QueryPropertyDesignable) {
        _id -= 1;
    } else if (_c == QMetaObject::QueryPropertyScriptable) {
        _id -= 1;
    } else if (_c == QMetaObject::QueryPropertyStored) {
        _id -= 1;
    } else if (_c == QMetaObject::QueryPropertyEditable) {
        _id -= 1;
    } else if (_c == QMetaObject::QueryPropertyUser) {
        _id -= 1;
    }
#endif // QT_NO_PROPERTIES
    return _id;
}

// SIGNAL 0
void KDSME::SignalTransition::signalChanged(const QString & _t1)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))) };
    QMetaObject::activate(this, &staticMetaObject, 0, _a);
}
struct qt_meta_stringdata_KDSME__TimeoutTransition_t {
    QByteArrayData data[4];
    char stringdata0[49];
};
#define QT_MOC_LITERAL(idx, ofs, len) \
    Q_STATIC_BYTE_ARRAY_DATA_HEADER_INITIALIZER_WITH_OFFSET(len, \
    qptrdiff(offsetof(qt_meta_stringdata_KDSME__TimeoutTransition_t, stringdata0) + ofs \
        - idx * sizeof(QByteArrayData)) \
    )
static const qt_meta_stringdata_KDSME__TimeoutTransition_t qt_meta_stringdata_KDSME__TimeoutTransition = {
    {
QT_MOC_LITERAL(0, 0, 24), // "KDSME::TimeoutTransition"
QT_MOC_LITERAL(1, 25, 14), // "timeoutChanged"
QT_MOC_LITERAL(2, 40, 0), // ""
QT_MOC_LITERAL(3, 41, 7) // "timeout"

    },
    "KDSME::TimeoutTransition\0timeoutChanged\0"
    "\0timeout"
};
#undef QT_MOC_LITERAL

static const uint qt_meta_data_KDSME__TimeoutTransition[] = {

 // content:
       8,       // revision
       0,       // classname
       0,    0, // classinfo
       1,   14, // methods
       1,   22, // properties
       0,    0, // enums/sets
       0,    0, // constructors
       0,       // flags
       1,       // signalCount

 // signals: name, argc, parameters, tag, flags
       1,    1,   19,    2, 0x06 /* Public */,

 // signals: parameters
    QMetaType::Void, QMetaType::Int,    3,

 // properties: name, type, flags
       3, QMetaType::Int, 0x00495103,

 // properties: notify_signal_id
       0,

       0        // eod
};

void KDSME::TimeoutTransition::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    if (_c == QMetaObject::InvokeMetaMethod) {
        auto *_t = static_cast<TimeoutTransition *>(_o);
        (void)_t;
        switch (_id) {
        case 0: _t->timeoutChanged((*reinterpret_cast< int(*)>(_a[1]))); break;
        default: ;
        }
    } else if (_c == QMetaObject::IndexOfMethod) {
        int *result = reinterpret_cast<int *>(_a[0]);
        {
            using _t = void (TimeoutTransition::*)(int );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&TimeoutTransition::timeoutChanged)) {
                *result = 0;
                return;
            }
        }
    }
#ifndef QT_NO_PROPERTIES
    else if (_c == QMetaObject::ReadProperty) {
        auto *_t = static_cast<TimeoutTransition *>(_o);
        (void)_t;
        void *_v = _a[0];
        switch (_id) {
        case 0: *reinterpret_cast< int*>(_v) = _t->timeout(); break;
        default: break;
        }
    } else if (_c == QMetaObject::WriteProperty) {
        auto *_t = static_cast<TimeoutTransition *>(_o);
        (void)_t;
        void *_v = _a[0];
        switch (_id) {
        case 0: _t->setTimeout(*reinterpret_cast< int*>(_v)); break;
        default: break;
        }
    } else if (_c == QMetaObject::ResetProperty) {
    }
#endif // QT_NO_PROPERTIES
}

QT_INIT_METAOBJECT const QMetaObject KDSME::TimeoutTransition::staticMetaObject = { {
    QMetaObject::SuperData::link<Transition::staticMetaObject>(),
    qt_meta_stringdata_KDSME__TimeoutTransition.data,
    qt_meta_data_KDSME__TimeoutTransition,
    qt_static_metacall,
    nullptr,
    nullptr
} };


const QMetaObject *KDSME::TimeoutTransition::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *KDSME::TimeoutTransition::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_meta_stringdata_KDSME__TimeoutTransition.stringdata0))
        return static_cast<void*>(this);
    return Transition::qt_metacast(_clname);
}

int KDSME::TimeoutTransition::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = Transition::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 1)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 1;
    } else if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 1)
            *reinterpret_cast<int*>(_a[0]) = -1;
        _id -= 1;
    }
#ifndef QT_NO_PROPERTIES
    else if (_c == QMetaObject::ReadProperty || _c == QMetaObject::WriteProperty
            || _c == QMetaObject::ResetProperty || _c == QMetaObject::RegisterPropertyMetaType) {
        qt_static_metacall(this, _c, _id, _a);
        _id -= 1;
    } else if (_c == QMetaObject::QueryPropertyDesignable) {
        _id -= 1;
    } else if (_c == QMetaObject::QueryPropertyScriptable) {
        _id -= 1;
    } else if (_c == QMetaObject::QueryPropertyStored) {
        _id -= 1;
    } else if (_c == QMetaObject::QueryPropertyEditable) {
        _id -= 1;
    } else if (_c == QMetaObject::QueryPropertyUser) {
        _id -= 1;
    }
#endif // QT_NO_PROPERTIES
    return _id;
}

// SIGNAL 0
void KDSME::TimeoutTransition::timeoutChanged(int _t1)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))) };
    QMetaObject::activate(this, &staticMetaObject, 0, _a);
}
QT_WARNING_POP
QT_END_MOC_NAMESPACE
