/****************************************************************************
** Meta object code from reading C++ file 'element.h'
**
** Created by: The Qt Meta Object Compiler version 67 (Qt 5.15.19)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include <memory>
#include "../../../../../KDStateMachineEditor-1.2.8/src/core/model/element.h"
#include <QtCore/qbytearray.h>
#include <QtCore/qmetatype.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'element.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 67
#error "This file was generated using the moc from 5.15.19. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

QT_BEGIN_MOC_NAMESPACE
QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
struct qt_meta_stringdata_KDSME__Element_t {
    QByteArrayData data[40];
    char stringdata0[474];
};
#define QT_MOC_LITERAL(idx, ofs, len) \
    Q_STATIC_BYTE_ARRAY_DATA_HEADER_INITIALIZER_WITH_OFFSET(len, \
    qptrdiff(offsetof(qt_meta_stringdata_KDSME__Element_t, stringdata0) + ofs \
        - idx * sizeof(QByteArrayData)) \
    )
static const qt_meta_stringdata_KDSME__Element_t qt_meta_stringdata_KDSME__Element = {
    {
QT_MOC_LITERAL(0, 0, 14), // "KDSME::Element"
QT_MOC_LITERAL(1, 15, 13), // "parentChanged"
QT_MOC_LITERAL(2, 29, 0), // ""
QT_MOC_LITERAL(3, 30, 8), // "Element*"
QT_MOC_LITERAL(4, 39, 6), // "parent"
QT_MOC_LITERAL(5, 46, 12), // "flagsChanged"
QT_MOC_LITERAL(6, 59, 5), // "Flags"
QT_MOC_LITERAL(7, 65, 5), // "flags"
QT_MOC_LITERAL(8, 71, 12), // "labelChanged"
QT_MOC_LITERAL(9, 84, 5), // "label"
QT_MOC_LITERAL(10, 90, 10), // "posChanged"
QT_MOC_LITERAL(11, 101, 3), // "pos"
QT_MOC_LITERAL(12, 105, 13), // "heightChanged"
QT_MOC_LITERAL(13, 119, 6), // "height"
QT_MOC_LITERAL(14, 126, 12), // "widthChanged"
QT_MOC_LITERAL(15, 139, 5), // "width"
QT_MOC_LITERAL(16, 145, 14), // "visibleChanged"
QT_MOC_LITERAL(17, 160, 7), // "visible"
QT_MOC_LITERAL(18, 168, 15), // "selectedChanged"
QT_MOC_LITERAL(19, 184, 8), // "selected"
QT_MOC_LITERAL(20, 193, 12), // "typeToString"
QT_MOC_LITERAL(21, 206, 11), // "const char*"
QT_MOC_LITERAL(22, 218, 4), // "Type"
QT_MOC_LITERAL(23, 223, 4), // "type"
QT_MOC_LITERAL(24, 228, 12), // "stringToType"
QT_MOC_LITERAL(25, 241, 15), // "KDSME::Element*"
QT_MOC_LITERAL(26, 257, 11), // "ElementType"
QT_MOC_LITERAL(27, 269, 14), // "TransitionType"
QT_MOC_LITERAL(28, 284, 20), // "SignalTransitionType"
QT_MOC_LITERAL(29, 305, 21), // "TimeoutTransitionType"
QT_MOC_LITERAL(30, 327, 9), // "StateType"
QT_MOC_LITERAL(31, 337, 16), // "StateMachineType"
QT_MOC_LITERAL(32, 354, 16), // "HistoryStateType"
QT_MOC_LITERAL(33, 371, 14), // "FinalStateType"
QT_MOC_LITERAL(34, 386, 15), // "PseudoStateType"
QT_MOC_LITERAL(35, 402, 4), // "Flag"
QT_MOC_LITERAL(36, 407, 7), // "NoFlags"
QT_MOC_LITERAL(37, 415, 20), // "ElementIsDragEnabled"
QT_MOC_LITERAL(38, 436, 19), // "ElementIsSelectable"
QT_MOC_LITERAL(39, 456, 17) // "ElementIsEditable"

    },
    "KDSME::Element\0parentChanged\0\0Element*\0"
    "parent\0flagsChanged\0Flags\0flags\0"
    "labelChanged\0label\0posChanged\0pos\0"
    "heightChanged\0height\0widthChanged\0"
    "width\0visibleChanged\0visible\0"
    "selectedChanged\0selected\0typeToString\0"
    "const char*\0Type\0type\0stringToType\0"
    "KDSME::Element*\0ElementType\0TransitionType\0"
    "SignalTransitionType\0TimeoutTransitionType\0"
    "StateType\0StateMachineType\0HistoryStateType\0"
    "FinalStateType\0PseudoStateType\0Flag\0"
    "NoFlags\0ElementIsDragEnabled\0"
    "ElementIsSelectable\0ElementIsEditable"
};
#undef QT_MOC_LITERAL

static const uint qt_meta_data_KDSME__Element[] = {

 // content:
       8,       // revision
       0,       // classname
       0,    0, // classinfo
      10,   14, // methods
       9,   94, // properties
       2,  130, // enums/sets
       0,    0, // constructors
       0,       // flags
       8,       // signalCount

 // signals: name, argc, parameters, tag, flags
       1,    1,   64,    2, 0x06 /* Public */,
       5,    1,   67,    2, 0x06 /* Public */,
       8,    1,   70,    2, 0x06 /* Public */,
      10,    1,   73,    2, 0x06 /* Public */,
      12,    1,   76,    2, 0x06 /* Public */,
      14,    1,   79,    2, 0x06 /* Public */,
      16,    1,   82,    2, 0x06 /* Public */,
      18,    1,   85,    2, 0x06 /* Public */,

 // methods: name, argc, parameters, tag, flags
      20,    1,   88,    2, 0x02 /* Public */,
      24,    1,   91,    2, 0x02 /* Public */,

 // signals: parameters
    QMetaType::Void, 0x80000000 | 3,    4,
    QMetaType::Void, 0x80000000 | 6,    7,
    QMetaType::Void, QMetaType::QString,    9,
    QMetaType::Void, QMetaType::QPointF,   11,
    QMetaType::Void, QMetaType::QReal,   13,
    QMetaType::Void, QMetaType::QReal,   15,
    QMetaType::Void, QMetaType::Bool,   17,
    QMetaType::Void, QMetaType::Bool,   19,

 // methods: parameters
    0x80000000 | 21, 0x80000000 | 22,   23,
    0x80000000 | 22, 0x80000000 | 21,   23,

 // properties: name, type, flags
       4, 0x80000000 | 25, 0x0049580b,
      23, 0x80000000 | 22, 0x00095c09,
       7, 0x80000000 | 6, 0x0049590b,
       9, QMetaType::QString, 0x00495903,
      11, QMetaType::QPointF, 0x00495903,
      15, QMetaType::QReal, 0x00495903,
      13, QMetaType::QReal, 0x00495903,
      17, QMetaType::Bool, 0x00495903,
      19, QMetaType::Bool, 0x00495903,

 // properties: notify_signal_id
       0,
       0,
       1,
       2,
       3,
       5,
       4,
       6,
       7,

 // enums: name, alias, flags, count, data
      22,   22, 0x0,    9,  140,
       6,   35, 0x0,    4,  158,

 // enum data: key, value
      26, uint(KDSME::Element::ElementType),
      27, uint(KDSME::Element::TransitionType),
      28, uint(KDSME::Element::SignalTransitionType),
      29, uint(KDSME::Element::TimeoutTransitionType),
      30, uint(KDSME::Element::StateType),
      31, uint(KDSME::Element::StateMachineType),
      32, uint(KDSME::Element::HistoryStateType),
      33, uint(KDSME::Element::FinalStateType),
      34, uint(KDSME::Element::PseudoStateType),
      36, uint(KDSME::Element::NoFlags),
      37, uint(KDSME::Element::ElementIsDragEnabled),
      38, uint(KDSME::Element::ElementIsSelectable),
      39, uint(KDSME::Element::ElementIsEditable),

       0        // eod
};

void KDSME::Element::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    if (_c == QMetaObject::InvokeMetaMethod) {
        auto *_t = static_cast<Element *>(_o);
        (void)_t;
        switch (_id) {
        case 0: _t->parentChanged((*reinterpret_cast< Element*(*)>(_a[1]))); break;
        case 1: _t->flagsChanged((*reinterpret_cast< Flags(*)>(_a[1]))); break;
        case 2: _t->labelChanged((*reinterpret_cast< const QString(*)>(_a[1]))); break;
        case 3: _t->posChanged((*reinterpret_cast< const QPointF(*)>(_a[1]))); break;
        case 4: _t->heightChanged((*reinterpret_cast< qreal(*)>(_a[1]))); break;
        case 5: _t->widthChanged((*reinterpret_cast< qreal(*)>(_a[1]))); break;
        case 6: _t->visibleChanged((*reinterpret_cast< bool(*)>(_a[1]))); break;
        case 7: _t->selectedChanged((*reinterpret_cast< bool(*)>(_a[1]))); break;
        case 8: { const char* _r = _t->typeToString((*reinterpret_cast< Type(*)>(_a[1])));
            if (_a[0]) *reinterpret_cast< const char**>(_a[0]) = std::move(_r); }  break;
        case 9: { Type _r = _t->stringToType((*reinterpret_cast< const char*(*)>(_a[1])));
            if (_a[0]) *reinterpret_cast< Type*>(_a[0]) = std::move(_r); }  break;
        default: ;
        }
    } else if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        switch (_id) {
        default: *reinterpret_cast<int*>(_a[0]) = -1; break;
        case 0:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<int*>(_a[0]) = -1; break;
            case 0:
                *reinterpret_cast<int*>(_a[0]) = qRegisterMetaType< Element* >(); break;
            }
            break;
        }
    } else if (_c == QMetaObject::IndexOfMethod) {
        int *result = reinterpret_cast<int *>(_a[0]);
        {
            using _t = void (Element::*)(Element * );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&Element::parentChanged)) {
                *result = 0;
                return;
            }
        }
        {
            using _t = void (Element::*)(Flags );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&Element::flagsChanged)) {
                *result = 1;
                return;
            }
        }
        {
            using _t = void (Element::*)(const QString & );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&Element::labelChanged)) {
                *result = 2;
                return;
            }
        }
        {
            using _t = void (Element::*)(const QPointF & );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&Element::posChanged)) {
                *result = 3;
                return;
            }
        }
        {
            using _t = void (Element::*)(qreal );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&Element::heightChanged)) {
                *result = 4;
                return;
            }
        }
        {
            using _t = void (Element::*)(qreal );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&Element::widthChanged)) {
                *result = 5;
                return;
            }
        }
        {
            using _t = void (Element::*)(bool );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&Element::visibleChanged)) {
                *result = 6;
                return;
            }
        }
        {
            using _t = void (Element::*)(bool );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&Element::selectedChanged)) {
                *result = 7;
                return;
            }
        }
    } else if (_c == QMetaObject::RegisterPropertyMetaType) {
        switch (_id) {
        default: *reinterpret_cast<int*>(_a[0]) = -1; break;
        case 0:
            *reinterpret_cast<int*>(_a[0]) = qRegisterMetaType< KDSME::Element* >(); break;
        }
    }

#ifndef QT_NO_PROPERTIES
    else if (_c == QMetaObject::ReadProperty) {
        auto *_t = static_cast<Element *>(_o);
        (void)_t;
        void *_v = _a[0];
        switch (_id) {
        case 0: *reinterpret_cast< KDSME::Element**>(_v) = _t->parentElement(); break;
        case 1: *reinterpret_cast< Type*>(_v) = _t->type(); break;
        case 2: *reinterpret_cast< Flags*>(_v) = _t->flags(); break;
        case 3: *reinterpret_cast< QString*>(_v) = _t->label(); break;
        case 4: *reinterpret_cast< QPointF*>(_v) = _t->pos(); break;
        case 5: *reinterpret_cast< qreal*>(_v) = _t->width(); break;
        case 6: *reinterpret_cast< qreal*>(_v) = _t->height(); break;
        case 7: *reinterpret_cast< bool*>(_v) = _t->isVisible(); break;
        case 8: *reinterpret_cast< bool*>(_v) = _t->isSelected(); break;
        default: break;
        }
    } else if (_c == QMetaObject::WriteProperty) {
        auto *_t = static_cast<Element *>(_o);
        (void)_t;
        void *_v = _a[0];
        switch (_id) {
        case 0: _t->setParentElement(*reinterpret_cast< KDSME::Element**>(_v)); break;
        case 2: _t->setFlags(*reinterpret_cast< Flags*>(_v)); break;
        case 3: _t->setLabel(*reinterpret_cast< QString*>(_v)); break;
        case 4: _t->setPos(*reinterpret_cast< QPointF*>(_v)); break;
        case 5: _t->setWidth(*reinterpret_cast< qreal*>(_v)); break;
        case 6: _t->setHeight(*reinterpret_cast< qreal*>(_v)); break;
        case 7: _t->setVisible(*reinterpret_cast< bool*>(_v)); break;
        case 8: _t->setSelected(*reinterpret_cast< bool*>(_v)); break;
        default: break;
        }
    } else if (_c == QMetaObject::ResetProperty) {
    }
#endif // QT_NO_PROPERTIES
}

QT_INIT_METAOBJECT const QMetaObject KDSME::Element::staticMetaObject = { {
    QMetaObject::SuperData::link<QObject::staticMetaObject>(),
    qt_meta_stringdata_KDSME__Element.data,
    qt_meta_data_KDSME__Element,
    qt_static_metacall,
    nullptr,
    nullptr
} };


const QMetaObject *KDSME::Element::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *KDSME::Element::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_meta_stringdata_KDSME__Element.stringdata0))
        return static_cast<void*>(this);
    return QObject::qt_metacast(_clname);
}

int KDSME::Element::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QObject::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 10)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 10;
    } else if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 10)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 10;
    }
#ifndef QT_NO_PROPERTIES
    else if (_c == QMetaObject::ReadProperty || _c == QMetaObject::WriteProperty
            || _c == QMetaObject::ResetProperty || _c == QMetaObject::RegisterPropertyMetaType) {
        qt_static_metacall(this, _c, _id, _a);
        _id -= 9;
    } else if (_c == QMetaObject::QueryPropertyDesignable) {
        _id -= 9;
    } else if (_c == QMetaObject::QueryPropertyScriptable) {
        _id -= 9;
    } else if (_c == QMetaObject::QueryPropertyStored) {
        _id -= 9;
    } else if (_c == QMetaObject::QueryPropertyEditable) {
        _id -= 9;
    } else if (_c == QMetaObject::QueryPropertyUser) {
        _id -= 9;
    }
#endif // QT_NO_PROPERTIES
    return _id;
}

// SIGNAL 0
void KDSME::Element::parentChanged(Element * _t1)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))) };
    QMetaObject::activate(this, &staticMetaObject, 0, _a);
}

// SIGNAL 1
void KDSME::Element::flagsChanged(Flags _t1)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))) };
    QMetaObject::activate(this, &staticMetaObject, 1, _a);
}

// SIGNAL 2
void KDSME::Element::labelChanged(const QString & _t1)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))) };
    QMetaObject::activate(this, &staticMetaObject, 2, _a);
}

// SIGNAL 3
void KDSME::Element::posChanged(const QPointF & _t1)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))) };
    QMetaObject::activate(this, &staticMetaObject, 3, _a);
}

// SIGNAL 4
void KDSME::Element::heightChanged(qreal _t1)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))) };
    QMetaObject::activate(this, &staticMetaObject, 4, _a);
}

// SIGNAL 5
void KDSME::Element::widthChanged(qreal _t1)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))) };
    QMetaObject::activate(this, &staticMetaObject, 5, _a);
}

// SIGNAL 6
void KDSME::Element::visibleChanged(bool _t1)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))) };
    QMetaObject::activate(this, &staticMetaObject, 6, _a);
}

// SIGNAL 7
void KDSME::Element::selectedChanged(bool _t1)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))) };
    QMetaObject::activate(this, &staticMetaObject, 7, _a);
}
QT_WARNING_POP
QT_END_MOC_NAMESPACE
