/****************************************************************************
** Meta object code from reading C++ file 'rep_debuginterface_replica.h'
**
** Created by: The Qt Meta Object Compiler version 67 (Qt 5.15.19)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include <memory>
#include "rep_debuginterface_replica.h"
#include <QtCore/qbytearray.h>
#include <QtCore/qmetatype.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'rep_debuginterface_replica.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 67
#error "This file was generated using the moc from 5.15.19. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

QT_BEGIN_MOC_NAMESPACE
QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
struct qt_meta_stringdata_DebugInterfaceReplica_t {
    QByteArrayData data[35];
    char stringdata0[581];
};
#define QT_MOC_LITERAL(idx, ofs, len) \
    Q_STATIC_BYTE_ARRAY_DATA_HEADER_INITIALIZER_WITH_OFFSET(len, \
    qptrdiff(offsetof(qt_meta_stringdata_DebugInterfaceReplica_t, stringdata0) + ofs \
        - idx * sizeof(QByteArrayData)) \
    )
static const qt_meta_stringdata_DebugInterfaceReplica_t qt_meta_stringdata_DebugInterfaceReplica = {
    {
QT_MOC_LITERAL(0, 0, 21), // "DebugInterfaceReplica"
QT_MOC_LITERAL(1, 22, 17), // "RemoteObject Type"
QT_MOC_LITERAL(2, 40, 14), // "DebugInterface"
QT_MOC_LITERAL(3, 55, 22), // "RemoteObject Signature"
QT_MOC_LITERAL(4, 78, 40), // "308c8d427f749fcb653ed368d456c..."
QT_MOC_LITERAL(5, 119, 13), // "statusChanged"
QT_MOC_LITERAL(6, 133, 0), // ""
QT_MOC_LITERAL(7, 134, 16), // "haveStateMachine"
QT_MOC_LITERAL(8, 151, 7), // "running"
QT_MOC_LITERAL(9, 159, 7), // "message"
QT_MOC_LITERAL(10, 167, 10), // "stateAdded"
QT_MOC_LITERAL(11, 178, 30), // "KDSME::DebugInterface::StateId"
QT_MOC_LITERAL(12, 209, 5), // "state"
QT_MOC_LITERAL(13, 215, 6), // "parent"
QT_MOC_LITERAL(14, 222, 11), // "hasChildren"
QT_MOC_LITERAL(15, 234, 5), // "label"
QT_MOC_LITERAL(16, 240, 32), // "KDSME::DebugInterface::StateType"
QT_MOC_LITERAL(17, 273, 4), // "type"
QT_MOC_LITERAL(18, 278, 16), // "connectToInitial"
QT_MOC_LITERAL(19, 295, 12), // "stateEntered"
QT_MOC_LITERAL(20, 308, 11), // "stateExited"
QT_MOC_LITERAL(21, 320, 15), // "transitionAdded"
QT_MOC_LITERAL(22, 336, 35), // "KDSME::DebugInterface::Transi..."
QT_MOC_LITERAL(23, 372, 6), // "source"
QT_MOC_LITERAL(24, 379, 6), // "target"
QT_MOC_LITERAL(25, 386, 25), // "stateConfigurationChanged"
QT_MOC_LITERAL(26, 412, 48), // "KDSME::DebugInterface::StateM..."
QT_MOC_LITERAL(27, 461, 6), // "config"
QT_MOC_LITERAL(28, 468, 19), // "maximumDepthChanged"
QT_MOC_LITERAL(29, 488, 5), // "depth"
QT_MOC_LITERAL(30, 494, 19), // "transitionTriggered"
QT_MOC_LITERAL(31, 514, 10), // "transition"
QT_MOC_LITERAL(32, 525, 22), // "aboutToRepopulateGraph"
QT_MOC_LITERAL(33, 548, 16), // "graphRepopulated"
QT_MOC_LITERAL(34, 565, 15) // "repopulateGraph"

    },
    "DebugInterfaceReplica\0RemoteObject Type\0"
    "DebugInterface\0RemoteObject Signature\0"
    "308c8d427f749fcb653ed368d456c47d4bb7f095\0"
    "statusChanged\0\0haveStateMachine\0running\0"
    "message\0stateAdded\0KDSME::DebugInterface::StateId\0"
    "state\0parent\0hasChildren\0label\0"
    "KDSME::DebugInterface::StateType\0type\0"
    "connectToInitial\0stateEntered\0stateExited\0"
    "transitionAdded\0KDSME::DebugInterface::TransitionId\0"
    "source\0target\0stateConfigurationChanged\0"
    "KDSME::DebugInterface::StateMachineConfiguration\0"
    "config\0maximumDepthChanged\0depth\0"
    "transitionTriggered\0transition\0"
    "aboutToRepopulateGraph\0graphRepopulated\0"
    "repopulateGraph"
};
#undef QT_MOC_LITERAL

static const uint qt_meta_data_DebugInterfaceReplica[] = {

 // content:
       8,       // revision
       0,       // classname
       2,   14, // classinfo
      12,   18, // methods
       0,    0, // properties
       0,    0, // enums/sets
       0,    0, // constructors
       0,       // flags
      11,       // signalCount

 // classinfo: key, value
       1,    2,
       3,    4,

 // signals: name, argc, parameters, tag, flags
       5,    2,   78,    6, 0x06 /* Public */,
       9,    1,   83,    6, 0x06 /* Public */,
      10,    6,   86,    6, 0x06 /* Public */,
      19,    1,   99,    6, 0x06 /* Public */,
      20,    1,  102,    6, 0x06 /* Public */,
      21,    4,  105,    6, 0x06 /* Public */,
      25,    1,  114,    6, 0x06 /* Public */,
      28,    1,  117,    6, 0x06 /* Public */,
      30,    2,  120,    6, 0x06 /* Public */,
      32,    0,  125,    6, 0x06 /* Public */,
      33,    0,  126,    6, 0x06 /* Public */,

 // slots: name, argc, parameters, tag, flags
      34,    0,  127,    6, 0x0a /* Public */,

 // signals: parameters
    QMetaType::Void, QMetaType::Bool, QMetaType::Bool,    7,    8,
    QMetaType::Void, QMetaType::QString,    9,
    QMetaType::Void, 0x80000000 | 11, 0x80000000 | 11, QMetaType::Bool, QMetaType::QString, 0x80000000 | 16, QMetaType::Bool,   12,   13,   14,   15,   17,   18,
    QMetaType::Void, 0x80000000 | 11,   12,
    QMetaType::Void, 0x80000000 | 11,   12,
    QMetaType::Void, 0x80000000 | 22, 0x80000000 | 11, 0x80000000 | 11, QMetaType::QString,   12,   23,   24,   15,
    QMetaType::Void, 0x80000000 | 26,   27,
    QMetaType::Void, QMetaType::Int,   29,
    QMetaType::Void, 0x80000000 | 22, QMetaType::QString,   31,   15,
    QMetaType::Void,
    QMetaType::Void,

 // slots: parameters
    QMetaType::Void,

       0        // eod
};

void DebugInterfaceReplica::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    if (_c == QMetaObject::InvokeMetaMethod) {
        auto *_t = static_cast<DebugInterfaceReplica *>(_o);
        (void)_t;
        switch (_id) {
        case 0: _t->statusChanged((*reinterpret_cast< bool(*)>(_a[1])),(*reinterpret_cast< bool(*)>(_a[2]))); break;
        case 1: _t->message((*reinterpret_cast< const QString(*)>(_a[1]))); break;
        case 2: _t->stateAdded((*reinterpret_cast< KDSME::DebugInterface::StateId(*)>(_a[1])),(*reinterpret_cast< KDSME::DebugInterface::StateId(*)>(_a[2])),(*reinterpret_cast< bool(*)>(_a[3])),(*reinterpret_cast< const QString(*)>(_a[4])),(*reinterpret_cast< KDSME::DebugInterface::StateType(*)>(_a[5])),(*reinterpret_cast< bool(*)>(_a[6]))); break;
        case 3: _t->stateEntered((*reinterpret_cast< KDSME::DebugInterface::StateId(*)>(_a[1]))); break;
        case 4: _t->stateExited((*reinterpret_cast< KDSME::DebugInterface::StateId(*)>(_a[1]))); break;
        case 5: _t->transitionAdded((*reinterpret_cast< KDSME::DebugInterface::TransitionId(*)>(_a[1])),(*reinterpret_cast< KDSME::DebugInterface::StateId(*)>(_a[2])),(*reinterpret_cast< KDSME::DebugInterface::StateId(*)>(_a[3])),(*reinterpret_cast< const QString(*)>(_a[4]))); break;
        case 6: _t->stateConfigurationChanged((*reinterpret_cast< const KDSME::DebugInterface::StateMachineConfiguration(*)>(_a[1]))); break;
        case 7: _t->maximumDepthChanged((*reinterpret_cast< int(*)>(_a[1]))); break;
        case 8: _t->transitionTriggered((*reinterpret_cast< KDSME::DebugInterface::TransitionId(*)>(_a[1])),(*reinterpret_cast< const QString(*)>(_a[2]))); break;
        case 9: _t->aboutToRepopulateGraph(); break;
        case 10: _t->graphRepopulated(); break;
        case 11: _t->repopulateGraph(); break;
        default: ;
        }
    } else if (_c == QMetaObject::IndexOfMethod) {
        int *result = reinterpret_cast<int *>(_a[0]);
        {
            using _t = void (DebugInterfaceReplica::*)(bool , bool );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&DebugInterfaceReplica::statusChanged)) {
                *result = 0;
                return;
            }
        }
        {
            using _t = void (DebugInterfaceReplica::*)(const QString & );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&DebugInterfaceReplica::message)) {
                *result = 1;
                return;
            }
        }
        {
            using _t = void (DebugInterfaceReplica::*)(KDSME::DebugInterface::StateId , KDSME::DebugInterface::StateId , bool , const QString & , KDSME::DebugInterface::StateType , bool );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&DebugInterfaceReplica::stateAdded)) {
                *result = 2;
                return;
            }
        }
        {
            using _t = void (DebugInterfaceReplica::*)(KDSME::DebugInterface::StateId );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&DebugInterfaceReplica::stateEntered)) {
                *result = 3;
                return;
            }
        }
        {
            using _t = void (DebugInterfaceReplica::*)(KDSME::DebugInterface::StateId );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&DebugInterfaceReplica::stateExited)) {
                *result = 4;
                return;
            }
        }
        {
            using _t = void (DebugInterfaceReplica::*)(KDSME::DebugInterface::TransitionId , KDSME::DebugInterface::StateId , KDSME::DebugInterface::StateId , const QString & );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&DebugInterfaceReplica::transitionAdded)) {
                *result = 5;
                return;
            }
        }
        {
            using _t = void (DebugInterfaceReplica::*)(const KDSME::DebugInterface::StateMachineConfiguration & );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&DebugInterfaceReplica::stateConfigurationChanged)) {
                *result = 6;
                return;
            }
        }
        {
            using _t = void (DebugInterfaceReplica::*)(int );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&DebugInterfaceReplica::maximumDepthChanged)) {
                *result = 7;
                return;
            }
        }
        {
            using _t = void (DebugInterfaceReplica::*)(KDSME::DebugInterface::TransitionId , const QString & );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&DebugInterfaceReplica::transitionTriggered)) {
                *result = 8;
                return;
            }
        }
        {
            using _t = void (DebugInterfaceReplica::*)();
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&DebugInterfaceReplica::aboutToRepopulateGraph)) {
                *result = 9;
                return;
            }
        }
        {
            using _t = void (DebugInterfaceReplica::*)();
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&DebugInterfaceReplica::graphRepopulated)) {
                *result = 10;
                return;
            }
        }
    }
}

QT_INIT_METAOBJECT const QMetaObject DebugInterfaceReplica::staticMetaObject = { {
    QMetaObject::SuperData::link<QRemoteObjectReplica::staticMetaObject>(),
    qt_meta_stringdata_DebugInterfaceReplica.data,
    qt_meta_data_DebugInterfaceReplica,
    qt_static_metacall,
    nullptr,
    nullptr
} };


const QMetaObject *DebugInterfaceReplica::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *DebugInterfaceReplica::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_meta_stringdata_DebugInterfaceReplica.stringdata0))
        return static_cast<void*>(this);
    return QRemoteObjectReplica::qt_metacast(_clname);
}

int DebugInterfaceReplica::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QRemoteObjectReplica::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 12)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 12;
    } else if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 12)
            *reinterpret_cast<int*>(_a[0]) = -1;
        _id -= 12;
    }
    return _id;
}

// SIGNAL 0
void DebugInterfaceReplica::statusChanged(bool _t1, bool _t2)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))), const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t2))) };
    QMetaObject::activate(this, &staticMetaObject, 0, _a);
}

// SIGNAL 1
void DebugInterfaceReplica::message(const QString & _t1)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))) };
    QMetaObject::activate(this, &staticMetaObject, 1, _a);
}

// SIGNAL 2
void DebugInterfaceReplica::stateAdded(KDSME::DebugInterface::StateId _t1, KDSME::DebugInterface::StateId _t2, bool _t3, const QString & _t4, KDSME::DebugInterface::StateType _t5, bool _t6)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))), const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t2))), const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t3))), const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t4))), const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t5))), const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t6))) };
    QMetaObject::activate(this, &staticMetaObject, 2, _a);
}

// SIGNAL 3
void DebugInterfaceReplica::stateEntered(KDSME::DebugInterface::StateId _t1)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))) };
    QMetaObject::activate(this, &staticMetaObject, 3, _a);
}

// SIGNAL 4
void DebugInterfaceReplica::stateExited(KDSME::DebugInterface::StateId _t1)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))) };
    QMetaObject::activate(this, &staticMetaObject, 4, _a);
}

// SIGNAL 5
void DebugInterfaceReplica::transitionAdded(KDSME::DebugInterface::TransitionId _t1, KDSME::DebugInterface::StateId _t2, KDSME::DebugInterface::StateId _t3, const QString & _t4)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))), const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t2))), const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t3))), const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t4))) };
    QMetaObject::activate(this, &staticMetaObject, 5, _a);
}

// SIGNAL 6
void DebugInterfaceReplica::stateConfigurationChanged(const KDSME::DebugInterface::StateMachineConfiguration & _t1)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))) };
    QMetaObject::activate(this, &staticMetaObject, 6, _a);
}

// SIGNAL 7
void DebugInterfaceReplica::maximumDepthChanged(int _t1)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))) };
    QMetaObject::activate(this, &staticMetaObject, 7, _a);
}

// SIGNAL 8
void DebugInterfaceReplica::transitionTriggered(KDSME::DebugInterface::TransitionId _t1, const QString & _t2)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))), const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t2))) };
    QMetaObject::activate(this, &staticMetaObject, 8, _a);
}

// SIGNAL 9
void DebugInterfaceReplica::aboutToRepopulateGraph()
{
    QMetaObject::activate(this, &staticMetaObject, 9, nullptr);
}

// SIGNAL 10
void DebugInterfaceReplica::graphRepopulated()
{
    QMetaObject::activate(this, &staticMetaObject, 10, nullptr);
}
QT_WARNING_POP
QT_END_MOC_NAMESPACE
