/****************************************************************************
** Meta object code from reading C++ file 'createelementcommand_p.h'
**
** Created by: The Qt Meta Object Compiler version 67 (Qt 5.15.19)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include <memory>
#include "../../../../../KDStateMachineEditor-1.2.8/src/view/command/createelementcommand_p.h"
#include <QtCore/qbytearray.h>
#include <QtCore/qmetatype.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'createelementcommand_p.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 67
#error "This file was generated using the moc from 5.15.19. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

QT_BEGIN_MOC_NAMESPACE
QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
struct qt_meta_stringdata_KDSME__CreateElementCommand_t {
    QByteArrayData data[10];
    char stringdata0[141];
};
#define QT_MOC_LITERAL(idx, ofs, len) \
    Q_STATIC_BYTE_ARRAY_DATA_HEADER_INITIALIZER_WITH_OFFSET(len, \
    qptrdiff(offsetof(qt_meta_stringdata_KDSME__CreateElementCommand_t, stringdata0) + ofs \
        - idx * sizeof(QByteArrayData)) \
    )
static const qt_meta_stringdata_KDSME__CreateElementCommand_t qt_meta_stringdata_KDSME__CreateElementCommand = {
    {
QT_MOC_LITERAL(0, 0, 27), // "KDSME::CreateElementCommand"
QT_MOC_LITERAL(1, 28, 20), // "parentElementChanged"
QT_MOC_LITERAL(2, 49, 0), // ""
QT_MOC_LITERAL(3, 50, 8), // "Element*"
QT_MOC_LITERAL(4, 59, 13), // "parentElement"
QT_MOC_LITERAL(5, 73, 11), // "typeChanged"
QT_MOC_LITERAL(6, 85, 13), // "Element::Type"
QT_MOC_LITERAL(7, 99, 4), // "type"
QT_MOC_LITERAL(8, 104, 15), // "KDSME::Element*"
QT_MOC_LITERAL(9, 120, 20) // "KDSME::Element::Type"

    },
    "KDSME::CreateElementCommand\0"
    "parentElementChanged\0\0Element*\0"
    "parentElement\0typeChanged\0Element::Type\0"
    "type\0KDSME::Element*\0KDSME::Element::Type"
};
#undef QT_MOC_LITERAL

static const uint qt_meta_data_KDSME__CreateElementCommand[] = {

 // content:
       8,       // revision
       0,       // classname
       0,    0, // classinfo
       2,   14, // methods
       2,   30, // properties
       0,    0, // enums/sets
       0,    0, // constructors
       0,       // flags
       2,       // signalCount

 // signals: name, argc, parameters, tag, flags
       1,    1,   24,    2, 0x06 /* Public */,
       5,    1,   27,    2, 0x06 /* Public */,

 // signals: parameters
    QMetaType::Void, 0x80000000 | 3,    4,
    QMetaType::Void, 0x80000000 | 6,    7,

 // properties: name, type, flags
       4, 0x80000000 | 8, 0x0049510b,
       7, 0x80000000 | 9, 0x0049510b,

 // properties: notify_signal_id
       0,
       1,

       0        // eod
};

void KDSME::CreateElementCommand::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    if (_c == QMetaObject::InvokeMetaMethod) {
        auto *_t = static_cast<CreateElementCommand *>(_o);
        (void)_t;
        switch (_id) {
        case 0: _t->parentElementChanged((*reinterpret_cast< Element*(*)>(_a[1]))); break;
        case 1: _t->typeChanged((*reinterpret_cast< Element::Type(*)>(_a[1]))); break;
        default: ;
        }
    } else if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        switch (_id) {
        default: *reinterpret_cast<int*>(_a[0]) = -1; break;
        case 0:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<int*>(_a[0]) = -1; break;
            case 0:
                *reinterpret_cast<int*>(_a[0]) = qRegisterMetaType< Element* >(); break;
            }
            break;
        }
    } else if (_c == QMetaObject::IndexOfMethod) {
        int *result = reinterpret_cast<int *>(_a[0]);
        {
            using _t = void (CreateElementCommand::*)(Element * );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&CreateElementCommand::parentElementChanged)) {
                *result = 0;
                return;
            }
        }
        {
            using _t = void (CreateElementCommand::*)(Element::Type );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&CreateElementCommand::typeChanged)) {
                *result = 1;
                return;
            }
        }
    } else if (_c == QMetaObject::RegisterPropertyMetaType) {
        switch (_id) {
        default: *reinterpret_cast<int*>(_a[0]) = -1; break;
        case 0:
            *reinterpret_cast<int*>(_a[0]) = qRegisterMetaType< KDSME::Element* >(); break;
        case 1:
            *reinterpret_cast<int*>(_a[0]) = qRegisterMetaType< KDSME::Element::Type >(); break;
        }
    }

#ifndef QT_NO_PROPERTIES
    else if (_c == QMetaObject::ReadProperty) {
        auto *_t = static_cast<CreateElementCommand *>(_o);
        (void)_t;
        void *_v = _a[0];
        switch (_id) {
        case 0: *reinterpret_cast< KDSME::Element**>(_v) = _t->parentElement(); break;
        case 1: *reinterpret_cast< KDSME::Element::Type*>(_v) = _t->type(); break;
        default: break;
        }
    } else if (_c == QMetaObject::WriteProperty) {
        auto *_t = static_cast<CreateElementCommand *>(_o);
        (void)_t;
        void *_v = _a[0];
        switch (_id) {
        case 0: _t->setParentElement(*reinterpret_cast< KDSME::Element**>(_v)); break;
        case 1: _t->setType(*reinterpret_cast< KDSME::Element::Type*>(_v)); break;
        default: break;
        }
    } else if (_c == QMetaObject::ResetProperty) {
    }
#endif // QT_NO_PROPERTIES
}

static const QMetaObject::SuperData qt_meta_extradata_KDSME__CreateElementCommand[] = {
    QMetaObject::SuperData::link<KDSME::Element::staticMetaObject>(),
    nullptr
};

QT_INIT_METAOBJECT const QMetaObject KDSME::CreateElementCommand::staticMetaObject = { {
    QMetaObject::SuperData::link<Command::staticMetaObject>(),
    qt_meta_stringdata_KDSME__CreateElementCommand.data,
    qt_meta_data_KDSME__CreateElementCommand,
    qt_static_metacall,
    qt_meta_extradata_KDSME__CreateElementCommand,
    nullptr
} };


const QMetaObject *KDSME::CreateElementCommand::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *KDSME::CreateElementCommand::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_meta_stringdata_KDSME__CreateElementCommand.stringdata0))
        return static_cast<void*>(this);
    return Command::qt_metacast(_clname);
}

int KDSME::CreateElementCommand::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = Command::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 2)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 2;
    } else if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 2)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 2;
    }
#ifndef QT_NO_PROPERTIES
    else if (_c == QMetaObject::ReadProperty || _c == QMetaObject::WriteProperty
            || _c == QMetaObject::ResetProperty || _c == QMetaObject::RegisterPropertyMetaType) {
        qt_static_metacall(this, _c, _id, _a);
        _id -= 2;
    } else if (_c == QMetaObject::QueryPropertyDesignable) {
        _id -= 2;
    } else if (_c == QMetaObject::QueryPropertyScriptable) {
        _id -= 2;
    } else if (_c == QMetaObject::QueryPropertyStored) {
        _id -= 2;
    } else if (_c == QMetaObject::QueryPropertyEditable) {
        _id -= 2;
    } else if (_c == QMetaObject::QueryPropertyUser) {
        _id -= 2;
    }
#endif // QT_NO_PROPERTIES
    return _id;
}

// SIGNAL 0
void KDSME::CreateElementCommand::parentElementChanged(Element * _t1)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))) };
    QMetaObject::activate(this, &staticMetaObject, 0, _a);
}

// SIGNAL 1
void KDSME::CreateElementCommand::typeChanged(Element::Type _t1)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))) };
    QMetaObject::activate(this, &staticMetaObject, 1, _a);
}
QT_WARNING_POP
QT_END_MOC_NAMESPACE
