/****************************************************************************
** Meta object code from reading C++ file 'abstractscene.h'
**
** Created by: The Qt Meta Object Compiler version 67 (Qt 5.15.19)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include <memory>
#include "../../../../../KDStateMachineEditor-1.2.8/src/view/abstractscene.h"
#include <QtCore/qbytearray.h>
#include <QtCore/qmetatype.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'abstractscene.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 67
#error "This file was generated using the moc from 5.15.19. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

QT_BEGIN_MOC_NAMESPACE
QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
struct qt_meta_stringdata_KDSME__AbstractScene_t {
    QByteArrayData data[38];
    char stringdata0[512];
};
#define QT_MOC_LITERAL(idx, ofs, len) \
    Q_STATIC_BYTE_ARRAY_DATA_HEADER_INITIALIZER_WITH_OFFSET(len, \
    qptrdiff(offsetof(qt_meta_stringdata_KDSME__AbstractScene_t, stringdata0) + ofs \
        - idx * sizeof(QByteArrayData)) \
    )
static const qt_meta_stringdata_KDSME__AbstractScene_t qt_meta_stringdata_KDSME__AbstractScene = {
    {
QT_MOC_LITERAL(0, 0, 20), // "KDSME::AbstractScene"
QT_MOC_LITERAL(1, 21, 12), // "modelChanged"
QT_MOC_LITERAL(2, 34, 0), // ""
QT_MOC_LITERAL(3, 35, 19), // "QAbstractItemModel*"
QT_MOC_LITERAL(4, 55, 5), // "model"
QT_MOC_LITERAL(5, 61, 12), // "stateChanged"
QT_MOC_LITERAL(6, 74, 9), // "ViewState"
QT_MOC_LITERAL(7, 84, 5), // "state"
QT_MOC_LITERAL(8, 90, 19), // "instantiatorChanged"
QT_MOC_LITERAL(9, 110, 12), // "instantiator"
QT_MOC_LITERAL(10, 123, 24), // "contextMenuPolicyChanged"
QT_MOC_LITERAL(11, 148, 21), // "Qt::ContextMenuPolicy"
QT_MOC_LITERAL(12, 170, 17), // "contextMenuPolicy"
QT_MOC_LITERAL(13, 188, 22), // "customContextMenuEvent"
QT_MOC_LITERAL(14, 211, 37), // "KDSME::AbstractSceneContextMe..."
QT_MOC_LITERAL(15, 249, 5), // "event"
QT_MOC_LITERAL(16, 255, 15), // "setCurrentIndex"
QT_MOC_LITERAL(17, 271, 11), // "QModelIndex"
QT_MOC_LITERAL(18, 283, 5), // "index"
QT_MOC_LITERAL(19, 289, 16), // "selectionChanged"
QT_MOC_LITERAL(20, 306, 14), // "QItemSelection"
QT_MOC_LITERAL(21, 321, 8), // "selected"
QT_MOC_LITERAL(22, 330, 10), // "deselected"
QT_MOC_LITERAL(23, 341, 14), // "currentChanged"
QT_MOC_LITERAL(24, 356, 7), // "current"
QT_MOC_LITERAL(25, 364, 8), // "previous"
QT_MOC_LITERAL(26, 373, 12), // "rowsInserted"
QT_MOC_LITERAL(27, 386, 6), // "parent"
QT_MOC_LITERAL(28, 393, 5), // "start"
QT_MOC_LITERAL(29, 399, 3), // "end"
QT_MOC_LITERAL(30, 403, 20), // "rowsAboutToBeRemoved"
QT_MOC_LITERAL(31, 424, 13), // "layoutChanged"
QT_MOC_LITERAL(32, 438, 12), // "setViewState"
QT_MOC_LITERAL(33, 451, 11), // "QQuickItem*"
QT_MOC_LITERAL(34, 463, 9), // "viewState"
QT_MOC_LITERAL(35, 473, 17), // "ContextMenuPolicy"
QT_MOC_LITERAL(36, 491, 7), // "NoState"
QT_MOC_LITERAL(37, 499, 12) // "RefreshState"

    },
    "KDSME::AbstractScene\0modelChanged\0\0"
    "QAbstractItemModel*\0model\0stateChanged\0"
    "ViewState\0state\0instantiatorChanged\0"
    "instantiator\0contextMenuPolicyChanged\0"
    "Qt::ContextMenuPolicy\0contextMenuPolicy\0"
    "customContextMenuEvent\0"
    "KDSME::AbstractSceneContextMenuEvent*\0"
    "event\0setCurrentIndex\0QModelIndex\0"
    "index\0selectionChanged\0QItemSelection\0"
    "selected\0deselected\0currentChanged\0"
    "current\0previous\0rowsInserted\0parent\0"
    "start\0end\0rowsAboutToBeRemoved\0"
    "layoutChanged\0setViewState\0QQuickItem*\0"
    "viewState\0ContextMenuPolicy\0NoState\0"
    "RefreshState"
};
#undef QT_MOC_LITERAL

static const uint qt_meta_data_KDSME__AbstractScene[] = {

 // content:
       8,       // revision
       0,       // classname
       0,    0, // classinfo
      12,   14, // methods
       4,  120, // properties
       1,  136, // enums/sets
       0,    0, // constructors
       0,       // flags
       5,       // signalCount

 // signals: name, argc, parameters, tag, flags
       1,    1,   74,    2, 0x06 /* Public */,
       5,    1,   77,    2, 0x06 /* Public */,
       8,    1,   80,    2, 0x06 /* Public */,
      10,    1,   83,    2, 0x06 /* Public */,
      13,    1,   86,    2, 0x06 /* Public */,

 // slots: name, argc, parameters, tag, flags
      16,    1,   89,    2, 0x0a /* Public */,
      19,    2,   92,    2, 0x09 /* Protected */,
      23,    2,   97,    2, 0x09 /* Protected */,
      26,    3,  102,    2, 0x09 /* Protected */,
      30,    3,  109,    2, 0x09 /* Protected */,
      31,    0,  116,    2, 0x09 /* Protected */,
      32,    1,  117,    2, 0x09 /* Protected */,

 // signals: parameters
    QMetaType::Void, 0x80000000 | 3,    4,
    QMetaType::Void, 0x80000000 | 6,    7,
    QMetaType::Void, QMetaType::QObjectStar,    9,
    QMetaType::Void, 0x80000000 | 11,   12,
    QMetaType::Void, 0x80000000 | 14,   15,

 // slots: parameters
    QMetaType::Void, 0x80000000 | 17,   18,
    QMetaType::Void, 0x80000000 | 20, 0x80000000 | 20,   21,   22,
    QMetaType::Void, 0x80000000 | 17, 0x80000000 | 17,   24,   25,
    QMetaType::Void, 0x80000000 | 17, QMetaType::Int, QMetaType::Int,   27,   28,   29,
    QMetaType::Void, 0x80000000 | 17, QMetaType::Int, QMetaType::Int,   27,   28,   29,
    QMetaType::Void,
    QMetaType::Void, 0x80000000 | 6,    7,

 // properties: name, type, flags
       4, 0x80000000 | 3, 0x0049510b,
       9, 0x80000000 | 33, 0x0049510b,
      34, 0x80000000 | 6, 0x00495809,
      35, 0x80000000 | 11, 0x0049590b,

 // properties: notify_signal_id
       0,
       2,
       1,
       3,

 // enums: name, alias, flags, count, data
       6,    6, 0x0,    2,  141,

 // enum data: key, value
      36, uint(KDSME::AbstractScene::NoState),
      37, uint(KDSME::AbstractScene::RefreshState),

       0        // eod
};

void KDSME::AbstractScene::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    if (_c == QMetaObject::InvokeMetaMethod) {
        auto *_t = static_cast<AbstractScene *>(_o);
        (void)_t;
        switch (_id) {
        case 0: _t->modelChanged((*reinterpret_cast< QAbstractItemModel*(*)>(_a[1]))); break;
        case 1: _t->stateChanged((*reinterpret_cast< ViewState(*)>(_a[1]))); break;
        case 2: _t->instantiatorChanged((*reinterpret_cast< QObject*(*)>(_a[1]))); break;
        case 3: _t->contextMenuPolicyChanged((*reinterpret_cast< Qt::ContextMenuPolicy(*)>(_a[1]))); break;
        case 4: _t->customContextMenuEvent((*reinterpret_cast< KDSME::AbstractSceneContextMenuEvent*(*)>(_a[1]))); break;
        case 5: _t->setCurrentIndex((*reinterpret_cast< const QModelIndex(*)>(_a[1]))); break;
        case 6: _t->selectionChanged((*reinterpret_cast< const QItemSelection(*)>(_a[1])),(*reinterpret_cast< const QItemSelection(*)>(_a[2]))); break;
        case 7: _t->currentChanged((*reinterpret_cast< const QModelIndex(*)>(_a[1])),(*reinterpret_cast< const QModelIndex(*)>(_a[2]))); break;
        case 8: _t->rowsInserted((*reinterpret_cast< const QModelIndex(*)>(_a[1])),(*reinterpret_cast< int(*)>(_a[2])),(*reinterpret_cast< int(*)>(_a[3]))); break;
        case 9: _t->rowsAboutToBeRemoved((*reinterpret_cast< const QModelIndex(*)>(_a[1])),(*reinterpret_cast< int(*)>(_a[2])),(*reinterpret_cast< int(*)>(_a[3]))); break;
        case 10: _t->layoutChanged(); break;
        case 11: _t->setViewState((*reinterpret_cast< ViewState(*)>(_a[1]))); break;
        default: ;
        }
    } else if (_c == QMetaObject::IndexOfMethod) {
        int *result = reinterpret_cast<int *>(_a[0]);
        {
            using _t = void (AbstractScene::*)(QAbstractItemModel * );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&AbstractScene::modelChanged)) {
                *result = 0;
                return;
            }
        }
        {
            using _t = void (AbstractScene::*)(ViewState );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&AbstractScene::stateChanged)) {
                *result = 1;
                return;
            }
        }
        {
            using _t = void (AbstractScene::*)(QObject * );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&AbstractScene::instantiatorChanged)) {
                *result = 2;
                return;
            }
        }
        {
            using _t = void (AbstractScene::*)(Qt::ContextMenuPolicy );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&AbstractScene::contextMenuPolicyChanged)) {
                *result = 3;
                return;
            }
        }
        {
            using _t = void (AbstractScene::*)(KDSME::AbstractSceneContextMenuEvent * );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&AbstractScene::customContextMenuEvent)) {
                *result = 4;
                return;
            }
        }
    } else if (_c == QMetaObject::RegisterPropertyMetaType) {
        switch (_id) {
        default: *reinterpret_cast<int*>(_a[0]) = -1; break;
        case 1:
            *reinterpret_cast<int*>(_a[0]) = qRegisterMetaType< QQuickItem* >(); break;
        }
    }

#ifndef QT_NO_PROPERTIES
    else if (_c == QMetaObject::ReadProperty) {
        auto *_t = static_cast<AbstractScene *>(_o);
        (void)_t;
        void *_v = _a[0];
        switch (_id) {
        case 0: *reinterpret_cast< QAbstractItemModel**>(_v) = _t->model(); break;
        case 1: *reinterpret_cast< QQuickItem**>(_v) = _t->instantiator(); break;
        case 2: *reinterpret_cast< ViewState*>(_v) = _t->viewState(); break;
        case 3: *reinterpret_cast< Qt::ContextMenuPolicy*>(_v) = _t->contextMenuPolicy(); break;
        default: break;
        }
    } else if (_c == QMetaObject::WriteProperty) {
        auto *_t = static_cast<AbstractScene *>(_o);
        (void)_t;
        void *_v = _a[0];
        switch (_id) {
        case 0: _t->setModel(*reinterpret_cast< QAbstractItemModel**>(_v)); break;
        case 1: _t->setInstantiator(*reinterpret_cast< QQuickItem**>(_v)); break;
        case 3: _t->setContextMenuPolicy(*reinterpret_cast< Qt::ContextMenuPolicy*>(_v)); break;
        default: break;
        }
    } else if (_c == QMetaObject::ResetProperty) {
    }
#endif // QT_NO_PROPERTIES
}

QT_INIT_METAOBJECT const QMetaObject KDSME::AbstractScene::staticMetaObject = { {
    QMetaObject::SuperData::link<QQuickItem::staticMetaObject>(),
    qt_meta_stringdata_KDSME__AbstractScene.data,
    qt_meta_data_KDSME__AbstractScene,
    qt_static_metacall,
    nullptr,
    nullptr
} };


const QMetaObject *KDSME::AbstractScene::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *KDSME::AbstractScene::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_meta_stringdata_KDSME__AbstractScene.stringdata0))
        return static_cast<void*>(this);
    return QQuickItem::qt_metacast(_clname);
}

int KDSME::AbstractScene::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QQuickItem::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 12)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 12;
    } else if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 12)
            *reinterpret_cast<int*>(_a[0]) = -1;
        _id -= 12;
    }
#ifndef QT_NO_PROPERTIES
    else if (_c == QMetaObject::ReadProperty || _c == QMetaObject::WriteProperty
            || _c == QMetaObject::ResetProperty || _c == QMetaObject::RegisterPropertyMetaType) {
        qt_static_metacall(this, _c, _id, _a);
        _id -= 4;
    } else if (_c == QMetaObject::QueryPropertyDesignable) {
        _id -= 4;
    } else if (_c == QMetaObject::QueryPropertyScriptable) {
        _id -= 4;
    } else if (_c == QMetaObject::QueryPropertyStored) {
        _id -= 4;
    } else if (_c == QMetaObject::QueryPropertyEditable) {
        _id -= 4;
    } else if (_c == QMetaObject::QueryPropertyUser) {
        _id -= 4;
    }
#endif // QT_NO_PROPERTIES
    return _id;
}

// SIGNAL 0
void KDSME::AbstractScene::modelChanged(QAbstractItemModel * _t1)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))) };
    QMetaObject::activate(this, &staticMetaObject, 0, _a);
}

// SIGNAL 1
void KDSME::AbstractScene::stateChanged(ViewState _t1)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))) };
    QMetaObject::activate(this, &staticMetaObject, 1, _a);
}

// SIGNAL 2
void KDSME::AbstractScene::instantiatorChanged(QObject * _t1)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))) };
    QMetaObject::activate(this, &staticMetaObject, 2, _a);
}

// SIGNAL 3
void KDSME::AbstractScene::contextMenuPolicyChanged(Qt::ContextMenuPolicy _t1)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))) };
    QMetaObject::activate(this, &staticMetaObject, 3, _a);
}

// SIGNAL 4
void KDSME::AbstractScene::customContextMenuEvent(KDSME::AbstractSceneContextMenuEvent * _t1)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))) };
    QMetaObject::activate(this, &staticMetaObject, 4, _a);
}
QT_WARNING_POP
QT_END_MOC_NAMESPACE
