/****************************************************************************
** Meta object code from reading C++ file 'runtimecontroller.h'
**
** Created by: The Qt Meta Object Compiler version 67 (Qt 5.15.19)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include <memory>
#include "../../../../../KDStateMachineEditor-1.2.8/src/core/model/runtimecontroller.h"
#include <QtCore/qbytearray.h>
#include <QtCore/qmetatype.h>
#include <QtCore/QSet>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'runtimecontroller.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 67
#error "This file was generated using the moc from 5.15.19. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

QT_BEGIN_MOC_NAMESPACE
QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
struct qt_meta_stringdata_KDSME__RuntimeController_t {
    QByteArrayData data[21];
    char stringdata0[329];
};
#define QT_MOC_LITERAL(idx, ofs, len) \
    Q_STATIC_BYTE_ARRAY_DATA_HEADER_INITIALIZER_WITH_OFFSET(len, \
    qptrdiff(offsetof(qt_meta_stringdata_KDSME__RuntimeController_t, stringdata0) + ofs \
        - idx * sizeof(QByteArrayData)) \
    )
static const qt_meta_stringdata_KDSME__RuntimeController_t qt_meta_stringdata_KDSME__RuntimeController = {
    {
QT_MOC_LITERAL(0, 0, 24), // "KDSME::RuntimeController"
QT_MOC_LITERAL(1, 25, 26), // "activeConfigurationChanged"
QT_MOC_LITERAL(2, 52, 0), // ""
QT_MOC_LITERAL(3, 53, 12), // "QSet<State*>"
QT_MOC_LITERAL(4, 66, 13), // "configuration"
QT_MOC_LITERAL(5, 80, 21), // "lastTransitionChanged"
QT_MOC_LITERAL(6, 102, 11), // "Transition*"
QT_MOC_LITERAL(7, 114, 10), // "transition"
QT_MOC_LITERAL(8, 125, 16), // "isRunningChanged"
QT_MOC_LITERAL(9, 142, 9), // "isRunning"
QT_MOC_LITERAL(10, 152, 19), // "activeRegionChanged"
QT_MOC_LITERAL(11, 172, 6), // "region"
QT_MOC_LITERAL(12, 179, 18), // "activenessForState"
QT_MOC_LITERAL(13, 198, 13), // "KDSME::State*"
QT_MOC_LITERAL(14, 212, 5), // "state"
QT_MOC_LITERAL(15, 218, 23), // "activenessForTransition"
QT_MOC_LITERAL(16, 242, 18), // "KDSME::Transition*"
QT_MOC_LITERAL(17, 261, 19), // "activeConfiguration"
QT_MOC_LITERAL(18, 281, 19), // "QSet<KDSME::State*>"
QT_MOC_LITERAL(19, 301, 14), // "lastTransition"
QT_MOC_LITERAL(20, 316, 12) // "activeRegion"

    },
    "KDSME::RuntimeController\0"
    "activeConfigurationChanged\0\0QSet<State*>\0"
    "configuration\0lastTransitionChanged\0"
    "Transition*\0transition\0isRunningChanged\0"
    "isRunning\0activeRegionChanged\0region\0"
    "activenessForState\0KDSME::State*\0state\0"
    "activenessForTransition\0KDSME::Transition*\0"
    "activeConfiguration\0QSet<KDSME::State*>\0"
    "lastTransition\0activeRegion"
};
#undef QT_MOC_LITERAL

static const uint qt_meta_data_KDSME__RuntimeController[] = {

 // content:
       8,       // revision
       0,       // classname
       0,    0, // classinfo
       6,   14, // methods
       4,   62, // properties
       0,    0, // enums/sets
       0,    0, // constructors
       0,       // flags
       4,       // signalCount

 // signals: name, argc, parameters, tag, flags
       1,    1,   44,    2, 0x06 /* Public */,
       5,    1,   47,    2, 0x06 /* Public */,
       8,    1,   50,    2, 0x06 /* Public */,
      10,    1,   53,    2, 0x06 /* Public */,

 // methods: name, argc, parameters, tag, flags
      12,    1,   56,    2, 0x02 /* Public */,
      15,    1,   59,    2, 0x02 /* Public */,

 // signals: parameters
    QMetaType::Void, 0x80000000 | 3,    4,
    QMetaType::Void, 0x80000000 | 6,    7,
    QMetaType::Void, QMetaType::Bool,    9,
    QMetaType::Void, QMetaType::QRectF,   11,

 // methods: parameters
    QMetaType::Float, 0x80000000 | 13,   14,
    QMetaType::Float, 0x80000000 | 16,    7,

 // properties: name, type, flags
      17, 0x80000000 | 18, 0x00495009,
      19, 0x80000000 | 16, 0x00495009,
       9, QMetaType::Bool, 0x00495001,
      20, QMetaType::QRectF, 0x00495001,

 // properties: notify_signal_id
       0,
       1,
       2,
       3,

       0        // eod
};

void KDSME::RuntimeController::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    if (_c == QMetaObject::InvokeMetaMethod) {
        auto *_t = static_cast<RuntimeController *>(_o);
        (void)_t;
        switch (_id) {
        case 0: _t->activeConfigurationChanged((*reinterpret_cast< const QSet<State*>(*)>(_a[1]))); break;
        case 1: _t->lastTransitionChanged((*reinterpret_cast< Transition*(*)>(_a[1]))); break;
        case 2: _t->isRunningChanged((*reinterpret_cast< bool(*)>(_a[1]))); break;
        case 3: _t->activeRegionChanged((*reinterpret_cast< const QRectF(*)>(_a[1]))); break;
        case 4: { float _r = _t->activenessForState((*reinterpret_cast< KDSME::State*(*)>(_a[1])));
            if (_a[0]) *reinterpret_cast< float*>(_a[0]) = std::move(_r); }  break;
        case 5: { float _r = _t->activenessForTransition((*reinterpret_cast< KDSME::Transition*(*)>(_a[1])));
            if (_a[0]) *reinterpret_cast< float*>(_a[0]) = std::move(_r); }  break;
        default: ;
        }
    } else if (_c == QMetaObject::IndexOfMethod) {
        int *result = reinterpret_cast<int *>(_a[0]);
        {
            using _t = void (RuntimeController::*)(const QSet<State*> & );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&RuntimeController::activeConfigurationChanged)) {
                *result = 0;
                return;
            }
        }
        {
            using _t = void (RuntimeController::*)(Transition * );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&RuntimeController::lastTransitionChanged)) {
                *result = 1;
                return;
            }
        }
        {
            using _t = void (RuntimeController::*)(bool );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&RuntimeController::isRunningChanged)) {
                *result = 2;
                return;
            }
        }
        {
            using _t = void (RuntimeController::*)(const QRectF & );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&RuntimeController::activeRegionChanged)) {
                *result = 3;
                return;
            }
        }
    }
#ifndef QT_NO_PROPERTIES
    else if (_c == QMetaObject::ReadProperty) {
        auto *_t = static_cast<RuntimeController *>(_o);
        (void)_t;
        void *_v = _a[0];
        switch (_id) {
        case 0: *reinterpret_cast< QSet<KDSME::State*>*>(_v) = _t->activeConfiguration(); break;
        case 1: *reinterpret_cast< KDSME::Transition**>(_v) = _t->lastTransition(); break;
        case 2: *reinterpret_cast< bool*>(_v) = _t->isRunning(); break;
        case 3: *reinterpret_cast< QRectF*>(_v) = _t->activeRegion(); break;
        default: break;
        }
    } else if (_c == QMetaObject::WriteProperty) {
    } else if (_c == QMetaObject::ResetProperty) {
    }
#endif // QT_NO_PROPERTIES
}

QT_INIT_METAOBJECT const QMetaObject KDSME::RuntimeController::staticMetaObject = { {
    QMetaObject::SuperData::link<QObject::staticMetaObject>(),
    qt_meta_stringdata_KDSME__RuntimeController.data,
    qt_meta_data_KDSME__RuntimeController,
    qt_static_metacall,
    nullptr,
    nullptr
} };


const QMetaObject *KDSME::RuntimeController::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *KDSME::RuntimeController::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_meta_stringdata_KDSME__RuntimeController.stringdata0))
        return static_cast<void*>(this);
    return QObject::qt_metacast(_clname);
}

int KDSME::RuntimeController::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QObject::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 6)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 6;
    } else if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 6)
            *reinterpret_cast<int*>(_a[0]) = -1;
        _id -= 6;
    }
#ifndef QT_NO_PROPERTIES
    else if (_c == QMetaObject::ReadProperty || _c == QMetaObject::WriteProperty
            || _c == QMetaObject::ResetProperty || _c == QMetaObject::RegisterPropertyMetaType) {
        qt_static_metacall(this, _c, _id, _a);
        _id -= 4;
    } else if (_c == QMetaObject::QueryPropertyDesignable) {
        _id -= 4;
    } else if (_c == QMetaObject::QueryPropertyScriptable) {
        _id -= 4;
    } else if (_c == QMetaObject::QueryPropertyStored) {
        _id -= 4;
    } else if (_c == QMetaObject::QueryPropertyEditable) {
        _id -= 4;
    } else if (_c == QMetaObject::QueryPropertyUser) {
        _id -= 4;
    }
#endif // QT_NO_PROPERTIES
    return _id;
}

// SIGNAL 0
void KDSME::RuntimeController::activeConfigurationChanged(const QSet<State*> & _t1)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))) };
    QMetaObject::activate(this, &staticMetaObject, 0, _a);
}

// SIGNAL 1
void KDSME::RuntimeController::lastTransitionChanged(Transition * _t1)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))) };
    QMetaObject::activate(this, &staticMetaObject, 1, _a);
}

// SIGNAL 2
void KDSME::RuntimeController::isRunningChanged(bool _t1)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))) };
    QMetaObject::activate(this, &staticMetaObject, 2, _a);
}

// SIGNAL 3
void KDSME::RuntimeController::activeRegionChanged(const QRectF & _t1)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))) };
    QMetaObject::activate(this, &staticMetaObject, 3, _a);
}
QT_WARNING_POP
QT_END_MOC_NAMESPACE
