/****************************************************************************
** Meta object code from reading C++ file 'statemachineview.h'
**
** Created by: The Qt Meta Object Compiler version 67 (Qt 5.15.19)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include <memory>
#include "../../../../../KDStateMachineEditor-1.2.8/src/view/widgets/statemachineview.h"
#include <QtCore/qbytearray.h>
#include <QtCore/qmetatype.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'statemachineview.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 67
#error "This file was generated using the moc from 5.15.19. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

QT_BEGIN_MOC_NAMESPACE
QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
struct qt_meta_stringdata_KDSME__StateMachineView_t {
    QByteArrayData data[22];
    char stringdata0[334];
};
#define QT_MOC_LITERAL(idx, ofs, len) \
    Q_STATIC_BYTE_ARRAY_DATA_HEADER_INITIALIZER_WITH_OFFSET(len, \
    qptrdiff(offsetof(qt_meta_stringdata_KDSME__StateMachineView_t, stringdata0) + ofs \
        - idx * sizeof(QByteArrayData)) \
    )
static const qt_meta_stringdata_KDSME__StateMachineView_t qt_meta_stringdata_KDSME__StateMachineView = {
    {
QT_MOC_LITERAL(0, 0, 23), // "KDSME::StateMachineView"
QT_MOC_LITERAL(1, 24, 12), // "sceneChanged"
QT_MOC_LITERAL(2, 37, 0), // ""
QT_MOC_LITERAL(3, 38, 25), // "KDSME::StateMachineScene*"
QT_MOC_LITERAL(4, 64, 5), // "scene"
QT_MOC_LITERAL(5, 70, 16), // "themeNameChanged"
QT_MOC_LITERAL(6, 87, 9), // "themeName"
QT_MOC_LITERAL(7, 97, 21), // "onStateMachineChanged"
QT_MOC_LITERAL(8, 119, 20), // "KDSME::StateMachine*"
QT_MOC_LITERAL(9, 140, 11), // "sendCommand"
QT_MOC_LITERAL(10, 152, 15), // "KDSME::Command*"
QT_MOC_LITERAL(11, 168, 3), // "cmd"
QT_MOC_LITERAL(12, 172, 9), // "fitInView"
QT_MOC_LITERAL(13, 182, 18), // "changeStateMachine"
QT_MOC_LITERAL(14, 201, 12), // "stateMachine"
QT_MOC_LITERAL(15, 214, 13), // "deleteElement"
QT_MOC_LITERAL(16, 228, 15), // "KDSME::Element*"
QT_MOC_LITERAL(17, 244, 7), // "element"
QT_MOC_LITERAL(18, 252, 17), // "commandController"
QT_MOC_LITERAL(19, 270, 25), // "KDSME::CommandController*"
QT_MOC_LITERAL(20, 296, 14), // "editController"
QT_MOC_LITERAL(21, 311, 22) // "KDSME::EditController*"

    },
    "KDSME::StateMachineView\0sceneChanged\0"
    "\0KDSME::StateMachineScene*\0scene\0"
    "themeNameChanged\0themeName\0"
    "onStateMachineChanged\0KDSME::StateMachine*\0"
    "sendCommand\0KDSME::Command*\0cmd\0"
    "fitInView\0changeStateMachine\0stateMachine\0"
    "deleteElement\0KDSME::Element*\0element\0"
    "commandController\0KDSME::CommandController*\0"
    "editController\0KDSME::EditController*"
};
#undef QT_MOC_LITERAL

static const uint qt_meta_data_KDSME__StateMachineView[] = {

 // content:
       8,       // revision
       0,       // classname
       0,    0, // classinfo
       7,   14, // methods
       4,   68, // properties
       0,    0, // enums/sets
       0,    0, // constructors
       0,       // flags
       2,       // signalCount

 // signals: name, argc, parameters, tag, flags
       1,    1,   49,    2, 0x06 /* Public */,
       5,    1,   52,    2, 0x06 /* Public */,

 // slots: name, argc, parameters, tag, flags
       7,    1,   55,    2, 0x08 /* Private */,

 // methods: name, argc, parameters, tag, flags
       9,    1,   58,    2, 0x02 /* Public */,
      12,    0,   61,    2, 0x02 /* Public */,
      13,    1,   62,    2, 0x02 /* Public */,
      15,    1,   65,    2, 0x02 /* Public */,

 // signals: parameters
    QMetaType::Void, 0x80000000 | 3,    4,
    QMetaType::Void, QMetaType::QString,    6,

 // slots: parameters
    QMetaType::Void, 0x80000000 | 8,    2,

 // methods: parameters
    QMetaType::Void, 0x80000000 | 10,   11,
    QMetaType::Void,
    QMetaType::Void, 0x80000000 | 8,   14,
    QMetaType::Void, 0x80000000 | 16,   17,

 // properties: name, type, flags
       4, 0x80000000 | 3, 0x0049510b,
      18, 0x80000000 | 19, 0x00095409,
      20, 0x80000000 | 21, 0x00095409,
       6, QMetaType::QString, 0x00495103,

 // properties: notify_signal_id
       0,
       0,
       0,
       1,

       0        // eod
};

void KDSME::StateMachineView::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    if (_c == QMetaObject::InvokeMetaMethod) {
        auto *_t = static_cast<StateMachineView *>(_o);
        (void)_t;
        switch (_id) {
        case 0: _t->sceneChanged((*reinterpret_cast< KDSME::StateMachineScene*(*)>(_a[1]))); break;
        case 1: _t->themeNameChanged((*reinterpret_cast< const QString(*)>(_a[1]))); break;
        case 2: _t->d->onStateMachineChanged((*reinterpret_cast< KDSME::StateMachine*(*)>(_a[1]))); break;
        case 3: _t->sendCommand((*reinterpret_cast< KDSME::Command*(*)>(_a[1]))); break;
        case 4: _t->fitInView(); break;
        case 5: _t->changeStateMachine((*reinterpret_cast< KDSME::StateMachine*(*)>(_a[1]))); break;
        case 6: _t->deleteElement((*reinterpret_cast< KDSME::Element*(*)>(_a[1]))); break;
        default: ;
        }
    } else if (_c == QMetaObject::IndexOfMethod) {
        int *result = reinterpret_cast<int *>(_a[0]);
        {
            using _t = void (StateMachineView::*)(KDSME::StateMachineScene * );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&StateMachineView::sceneChanged)) {
                *result = 0;
                return;
            }
        }
        {
            using _t = void (StateMachineView::*)(const QString & );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&StateMachineView::themeNameChanged)) {
                *result = 1;
                return;
            }
        }
    }
#ifndef QT_NO_PROPERTIES
    else if (_c == QMetaObject::ReadProperty) {
        auto *_t = static_cast<StateMachineView *>(_o);
        (void)_t;
        void *_v = _a[0];
        switch (_id) {
        case 0: *reinterpret_cast< KDSME::StateMachineScene**>(_v) = _t->scene(); break;
        case 1: *reinterpret_cast< KDSME::CommandController**>(_v) = _t->commandController(); break;
        case 2: *reinterpret_cast< KDSME::EditController**>(_v) = _t->editController(); break;
        case 3: *reinterpret_cast< QString*>(_v) = _t->themeName(); break;
        default: break;
        }
    } else if (_c == QMetaObject::WriteProperty) {
        auto *_t = static_cast<StateMachineView *>(_o);
        (void)_t;
        void *_v = _a[0];
        switch (_id) {
        case 0: _t->setScene(*reinterpret_cast< KDSME::StateMachineScene**>(_v)); break;
        case 3: _t->setThemeName(*reinterpret_cast< QString*>(_v)); break;
        default: break;
        }
    } else if (_c == QMetaObject::ResetProperty) {
    }
#endif // QT_NO_PROPERTIES
}

QT_INIT_METAOBJECT const QMetaObject KDSME::StateMachineView::staticMetaObject = { {
    QMetaObject::SuperData::link<QQuickWidget::staticMetaObject>(),
    qt_meta_stringdata_KDSME__StateMachineView.data,
    qt_meta_data_KDSME__StateMachineView,
    qt_static_metacall,
    nullptr,
    nullptr
} };


const QMetaObject *KDSME::StateMachineView::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *KDSME::StateMachineView::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_meta_stringdata_KDSME__StateMachineView.stringdata0))
        return static_cast<void*>(this);
    return QQuickWidget::qt_metacast(_clname);
}

int KDSME::StateMachineView::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QQuickWidget::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 7)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 7;
    } else if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 7)
            *reinterpret_cast<int*>(_a[0]) = -1;
        _id -= 7;
    }
#ifndef QT_NO_PROPERTIES
    else if (_c == QMetaObject::ReadProperty || _c == QMetaObject::WriteProperty
            || _c == QMetaObject::ResetProperty || _c == QMetaObject::RegisterPropertyMetaType) {
        qt_static_metacall(this, _c, _id, _a);
        _id -= 4;
    } else if (_c == QMetaObject::QueryPropertyDesignable) {
        _id -= 4;
    } else if (_c == QMetaObject::QueryPropertyScriptable) {
        _id -= 4;
    } else if (_c == QMetaObject::QueryPropertyStored) {
        _id -= 4;
    } else if (_c == QMetaObject::QueryPropertyEditable) {
        _id -= 4;
    } else if (_c == QMetaObject::QueryPropertyUser) {
        _id -= 4;
    }
#endif // QT_NO_PROPERTIES
    return _id;
}

// SIGNAL 0
void KDSME::StateMachineView::sceneChanged(KDSME::StateMachineScene * _t1)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))) };
    QMetaObject::activate(this, &staticMetaObject, 0, _a);
}

// SIGNAL 1
void KDSME::StateMachineView::themeNameChanged(const QString & _t1)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))) };
    QMetaObject::activate(this, &staticMetaObject, 1, _a);
}
QT_WARNING_POP
QT_END_MOC_NAMESPACE
