/****************************************************************************
** Meta object code from reading C++ file 'propertyeditor.h'
**
** Created by: The Qt Meta Object Compiler version 67 (Qt 5.15.19)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include <memory>
#include "../../../../../KDStateMachineEditor-1.2.8/src/view/widgets/propertyeditor.h"
#include <QtCore/qbytearray.h>
#include <QtCore/qmetatype.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'propertyeditor.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 67
#error "This file was generated using the moc from 5.15.19. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

QT_BEGIN_MOC_NAMESPACE
QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
struct qt_meta_stringdata_KDSME__PropertyEditor_t {
    QByteArrayData data[15];
    char stringdata0[215];
};
#define QT_MOC_LITERAL(idx, ofs, len) \
    Q_STATIC_BYTE_ARRAY_DATA_HEADER_INITIALIZER_WITH_OFFSET(len, \
    qptrdiff(offsetof(qt_meta_stringdata_KDSME__PropertyEditor_t, stringdata0) + ofs \
        - idx * sizeof(QByteArrayData)) \
    )
static const qt_meta_stringdata_KDSME__PropertyEditor_t qt_meta_stringdata_KDSME__PropertyEditor = {
    {
QT_MOC_LITERAL(0, 0, 21), // "KDSME::PropertyEditor"
QT_MOC_LITERAL(1, 22, 20), // "updateSimpleProperty"
QT_MOC_LITERAL(2, 43, 0), // ""
QT_MOC_LITERAL(3, 44, 14), // "setInitalState"
QT_MOC_LITERAL(4, 59, 5), // "label"
QT_MOC_LITERAL(5, 65, 15), // "setDefaultState"
QT_MOC_LITERAL(6, 81, 14), // "setSourceState"
QT_MOC_LITERAL(7, 96, 14), // "setTargetState"
QT_MOC_LITERAL(8, 111, 16), // "childModeChanged"
QT_MOC_LITERAL(9, 128, 14), // "currentChanged"
QT_MOC_LITERAL(10, 143, 11), // "QModelIndex"
QT_MOC_LITERAL(11, 155, 7), // "current"
QT_MOC_LITERAL(12, 163, 8), // "previous"
QT_MOC_LITERAL(13, 172, 19), // "modelAboutToBeReset"
QT_MOC_LITERAL(14, 192, 22) // "loadFromCurrentElement"

    },
    "KDSME::PropertyEditor\0updateSimpleProperty\0"
    "\0setInitalState\0label\0setDefaultState\0"
    "setSourceState\0setTargetState\0"
    "childModeChanged\0currentChanged\0"
    "QModelIndex\0current\0previous\0"
    "modelAboutToBeReset\0loadFromCurrentElement"
};
#undef QT_MOC_LITERAL

static const uint qt_meta_data_KDSME__PropertyEditor[] = {

 // content:
       8,       // revision
       0,       // classname
       0,    0, // classinfo
       9,   14, // methods
       0,    0, // properties
       0,    0, // enums/sets
       0,    0, // constructors
       0,       // flags
       0,       // signalCount

 // slots: name, argc, parameters, tag, flags
       1,    0,   59,    2, 0x08 /* Private */,
       3,    1,   60,    2, 0x08 /* Private */,
       5,    1,   63,    2, 0x08 /* Private */,
       6,    1,   66,    2, 0x08 /* Private */,
       7,    1,   69,    2, 0x08 /* Private */,
       8,    0,   72,    2, 0x08 /* Private */,
       9,    2,   73,    2, 0x08 /* Private */,
      13,    0,   78,    2, 0x08 /* Private */,
      14,    0,   79,    2, 0x08 /* Private */,

 // slots: parameters
    QMetaType::Void,
    QMetaType::Void, QMetaType::QString,    4,
    QMetaType::Void, QMetaType::QString,    4,
    QMetaType::Void, QMetaType::QString,    4,
    QMetaType::Void, QMetaType::QString,    4,
    QMetaType::Void,
    QMetaType::Void, 0x80000000 | 10, 0x80000000 | 10,   11,   12,
    QMetaType::Void,
    QMetaType::Void,

       0        // eod
};

void KDSME::PropertyEditor::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    if (_c == QMetaObject::InvokeMetaMethod) {
        auto *_t = static_cast<PropertyEditor *>(_o);
        (void)_t;
        switch (_id) {
        case 0: _t->d->updateSimpleProperty(); break;
        case 1: _t->d->setInitalState((*reinterpret_cast< const QString(*)>(_a[1]))); break;
        case 2: _t->d->setDefaultState((*reinterpret_cast< const QString(*)>(_a[1]))); break;
        case 3: _t->d->setSourceState((*reinterpret_cast< const QString(*)>(_a[1]))); break;
        case 4: _t->d->setTargetState((*reinterpret_cast< const QString(*)>(_a[1]))); break;
        case 5: _t->d->childModeChanged(); break;
        case 6: _t->d->currentChanged((*reinterpret_cast< const QModelIndex(*)>(_a[1])),(*reinterpret_cast< const QModelIndex(*)>(_a[2]))); break;
        case 7: _t->d->modelAboutToBeReset(); break;
        case 8: _t->d->loadFromCurrentElement(); break;
        default: ;
        }
    }
}

QT_INIT_METAOBJECT const QMetaObject KDSME::PropertyEditor::staticMetaObject = { {
    QMetaObject::SuperData::link<QStackedWidget::staticMetaObject>(),
    qt_meta_stringdata_KDSME__PropertyEditor.data,
    qt_meta_data_KDSME__PropertyEditor,
    qt_static_metacall,
    nullptr,
    nullptr
} };


const QMetaObject *KDSME::PropertyEditor::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *KDSME::PropertyEditor::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_meta_stringdata_KDSME__PropertyEditor.stringdata0))
        return static_cast<void*>(this);
    return QStackedWidget::qt_metacast(_clname);
}

int KDSME::PropertyEditor::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QStackedWidget::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 9)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 9;
    } else if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 9)
            *reinterpret_cast<int*>(_a[0]) = -1;
        _id -= 9;
    }
    return _id;
}
QT_WARNING_POP
QT_END_MOC_NAMESPACE
