/********************************************************************************
** Form generated from reading UI file 'statepropertyeditor.ui'
**
** Created by: Qt User Interface Compiler version 5.15.19
**
** WARNING! All changes made in this file will be lost when recompiling UI file!
********************************************************************************/

#ifndef UI_STATEPROPERTYEDITOR_H
#define UI_STATEPROPERTYEDITOR_H

#include <QtCore/QVariant>
#include <QtWidgets/QApplication>
#include <QtWidgets/QComboBox>
#include <QtWidgets/QFormLayout>
#include <QtWidgets/QHBoxLayout>
#include <QtWidgets/QLabel>
#include <QtWidgets/QLineEdit>
#include <QtWidgets/QSpacerItem>
#include <QtWidgets/QVBoxLayout>
#include <QtWidgets/QWidget>
#include "widgets/codeeditor_p.h"

QT_BEGIN_NAMESPACE

class Ui_StatePropertyEditor
{
public:
    QVBoxLayout *verticalLayout_3;
    QFormLayout *formLayout;
    QLabel *label;
    QLineEdit *labelLineEdit;
    QLabel *initialStateLabel;
    QComboBox *initialStateComboBox;
    QLabel *childModeLabel;
    QComboBox *childModeEdit;
    QLabel *historyTypeLabel;
    QComboBox *historyTypeEdit;
    QLabel *defaultStateLabel;
    QComboBox *defaultStateComboBox;
    QVBoxLayout *verticalLayout_2;
    QHBoxLayout *horizontalLayout_2;
    QLabel *label_3;
    QSpacerItem *horizontalSpacer_2;
    CodeEditor *onEntryEditor;
    QVBoxLayout *verticalLayout;
    QHBoxLayout *horizontalLayout;
    QLabel *label_4;
    QSpacerItem *horizontalSpacer;
    CodeEditor *onExitEditor;

    void setupUi(QWidget *StatePropertyEditor)
    {
        if (StatePropertyEditor->objectName().isEmpty())
            StatePropertyEditor->setObjectName(QString::fromUtf8("StatePropertyEditor"));
        StatePropertyEditor->resize(392, 325);
        verticalLayout_3 = new QVBoxLayout(StatePropertyEditor);
        verticalLayout_3->setObjectName(QString::fromUtf8("verticalLayout_3"));
        formLayout = new QFormLayout();
        formLayout->setObjectName(QString::fromUtf8("formLayout"));
        formLayout->setFieldGrowthPolicy(QFormLayout::AllNonFixedFieldsGrow);
        label = new QLabel(StatePropertyEditor);
        label->setObjectName(QString::fromUtf8("label"));
        label->setAlignment(Qt::AlignRight|Qt::AlignTrailing|Qt::AlignVCenter);

        formLayout->setWidget(0, QFormLayout::LabelRole, label);

        labelLineEdit = new QLineEdit(StatePropertyEditor);
        labelLineEdit->setObjectName(QString::fromUtf8("labelLineEdit"));

        formLayout->setWidget(0, QFormLayout::FieldRole, labelLineEdit);

        initialStateLabel = new QLabel(StatePropertyEditor);
        initialStateLabel->setObjectName(QString::fromUtf8("initialStateLabel"));
        initialStateLabel->setAlignment(Qt::AlignRight|Qt::AlignTrailing|Qt::AlignVCenter);

        formLayout->setWidget(1, QFormLayout::LabelRole, initialStateLabel);

        initialStateComboBox = new QComboBox(StatePropertyEditor);
        initialStateComboBox->setObjectName(QString::fromUtf8("initialStateComboBox"));
        initialStateComboBox->setEditable(true);

        formLayout->setWidget(1, QFormLayout::FieldRole, initialStateComboBox);

        childModeLabel = new QLabel(StatePropertyEditor);
        childModeLabel->setObjectName(QString::fromUtf8("childModeLabel"));

        formLayout->setWidget(3, QFormLayout::LabelRole, childModeLabel);

        childModeEdit = new QComboBox(StatePropertyEditor);
        childModeEdit->addItem(QString());
        childModeEdit->addItem(QString());
        childModeEdit->setObjectName(QString::fromUtf8("childModeEdit"));

        formLayout->setWidget(3, QFormLayout::FieldRole, childModeEdit);

        historyTypeLabel = new QLabel(StatePropertyEditor);
        historyTypeLabel->setObjectName(QString::fromUtf8("historyTypeLabel"));

        formLayout->setWidget(4, QFormLayout::LabelRole, historyTypeLabel);

        historyTypeEdit = new QComboBox(StatePropertyEditor);
        historyTypeEdit->addItem(QString());
        historyTypeEdit->addItem(QString());
        historyTypeEdit->setObjectName(QString::fromUtf8("historyTypeEdit"));

        formLayout->setWidget(4, QFormLayout::FieldRole, historyTypeEdit);

        defaultStateLabel = new QLabel(StatePropertyEditor);
        defaultStateLabel->setObjectName(QString::fromUtf8("defaultStateLabel"));
        defaultStateLabel->setAlignment(Qt::AlignRight|Qt::AlignTrailing|Qt::AlignVCenter);

        formLayout->setWidget(2, QFormLayout::LabelRole, defaultStateLabel);

        defaultStateComboBox = new QComboBox(StatePropertyEditor);
        defaultStateComboBox->setObjectName(QString::fromUtf8("defaultStateComboBox"));
        defaultStateComboBox->setEditable(true);

        formLayout->setWidget(2, QFormLayout::FieldRole, defaultStateComboBox);


        verticalLayout_3->addLayout(formLayout);

        verticalLayout_2 = new QVBoxLayout();
        verticalLayout_2->setObjectName(QString::fromUtf8("verticalLayout_2"));
        horizontalLayout_2 = new QHBoxLayout();
        horizontalLayout_2->setObjectName(QString::fromUtf8("horizontalLayout_2"));
        label_3 = new QLabel(StatePropertyEditor);
        label_3->setObjectName(QString::fromUtf8("label_3"));
        label_3->setAlignment(Qt::AlignRight|Qt::AlignTrailing|Qt::AlignVCenter);

        horizontalLayout_2->addWidget(label_3);

        horizontalSpacer_2 = new QSpacerItem(40, 20, QSizePolicy::Expanding, QSizePolicy::Minimum);

        horizontalLayout_2->addItem(horizontalSpacer_2);


        verticalLayout_2->addLayout(horizontalLayout_2);

        onEntryEditor = new CodeEditor(StatePropertyEditor);
        onEntryEditor->setObjectName(QString::fromUtf8("onEntryEditor"));
        QSizePolicy sizePolicy(QSizePolicy::Preferred, QSizePolicy::Expanding);
        sizePolicy.setHorizontalStretch(0);
        sizePolicy.setVerticalStretch(0);
        sizePolicy.setHeightForWidth(onEntryEditor->sizePolicy().hasHeightForWidth());
        onEntryEditor->setSizePolicy(sizePolicy);
        onEntryEditor->setFocusPolicy(Qt::StrongFocus);

        verticalLayout_2->addWidget(onEntryEditor);


        verticalLayout_3->addLayout(verticalLayout_2);

        verticalLayout = new QVBoxLayout();
        verticalLayout->setObjectName(QString::fromUtf8("verticalLayout"));
        horizontalLayout = new QHBoxLayout();
        horizontalLayout->setObjectName(QString::fromUtf8("horizontalLayout"));
        label_4 = new QLabel(StatePropertyEditor);
        label_4->setObjectName(QString::fromUtf8("label_4"));
        label_4->setAlignment(Qt::AlignRight|Qt::AlignTrailing|Qt::AlignVCenter);

        horizontalLayout->addWidget(label_4);

        horizontalSpacer = new QSpacerItem(40, 20, QSizePolicy::Expanding, QSizePolicy::Minimum);

        horizontalLayout->addItem(horizontalSpacer);


        verticalLayout->addLayout(horizontalLayout);

        onExitEditor = new CodeEditor(StatePropertyEditor);
        onExitEditor->setObjectName(QString::fromUtf8("onExitEditor"));
        sizePolicy.setHeightForWidth(onExitEditor->sizePolicy().hasHeightForWidth());
        onExitEditor->setSizePolicy(sizePolicy);
        onExitEditor->setFocusPolicy(Qt::StrongFocus);

        verticalLayout->addWidget(onExitEditor);


        verticalLayout_3->addLayout(verticalLayout);

#if QT_CONFIG(shortcut)
        label->setBuddy(labelLineEdit);
        initialStateLabel->setBuddy(initialStateComboBox);
        childModeLabel->setBuddy(childModeEdit);
        historyTypeLabel->setBuddy(historyTypeEdit);
        defaultStateLabel->setBuddy(initialStateComboBox);
        label_3->setBuddy(onEntryEditor);
        label_4->setBuddy(onExitEditor);
#endif // QT_CONFIG(shortcut)

        retranslateUi(StatePropertyEditor);

        QMetaObject::connectSlotsByName(StatePropertyEditor);
    } // setupUi

    void retranslateUi(QWidget *StatePropertyEditor)
    {
        StatePropertyEditor->setWindowTitle(QCoreApplication::translate("StatePropertyEditor", "Form", nullptr));
        label->setText(QCoreApplication::translate("StatePropertyEditor", "&Label:", nullptr));
        initialStateLabel->setText(QCoreApplication::translate("StatePropertyEditor", "&Initial state:", nullptr));
        childModeLabel->setText(QCoreApplication::translate("StatePropertyEditor", "&Child mode:", nullptr));
        childModeEdit->setItemText(0, QCoreApplication::translate("StatePropertyEditor", "Exclusive", nullptr));
        childModeEdit->setItemText(1, QCoreApplication::translate("StatePropertyEditor", "Parallel", nullptr));

        historyTypeLabel->setText(QCoreApplication::translate("StatePropertyEditor", "&History type:", nullptr));
        historyTypeEdit->setItemText(0, QCoreApplication::translate("StatePropertyEditor", "Shallow", nullptr));
        historyTypeEdit->setItemText(1, QCoreApplication::translate("StatePropertyEditor", "Deep", nullptr));

        defaultStateLabel->setText(QCoreApplication::translate("StatePropertyEditor", "&Default state:", nullptr));
        label_3->setText(QCoreApplication::translate("StatePropertyEditor", "&OnEntry action:", nullptr));
        label_4->setText(QCoreApplication::translate("StatePropertyEditor", "OnE&xit action:", nullptr));
    } // retranslateUi

};

namespace Ui {
    class StatePropertyEditor: public Ui_StatePropertyEditor {};
} // namespace Ui

QT_END_NAMESPACE

#endif // UI_STATEPROPERTYEDITOR_H
